/////////////////////////////////////////////////////////////////////
//
//     This confidential and proprietary software may be used only
//   as authorized by a licensing agreement from Synopsys Inc.
//   In the event of publication, the following notice is applicable:
//
//                   (C) COPYRIGHT 2003 SYNOPSYS INC.
//                          ALL RIGHTS RESERVED
//
//  The entire notice above must be reproduced on all authorized
//  copies.
//
//  Filename    : DW_hsata_vtb_defs.v
//  Author      : Tom Marshall
//  Date        : 03/09/2005
//  Description : Additional definitions file for DW_hsata
//                 Verilog Tetsbench.
//
//  Modification History :
//
//  03/09/05 - Initial Release by TRM.
//
/////////////////////////////////////////////////////////////////////

//#define $display dbg_printf

#define TCC_SATA_PHY_VIRT_ADDR 0xF0860000

#define ahb_read_single(a,b,c,d)    ((d) = *((volatile int *)(c)))
#define ahb_write_single(a,b,c,d)   (*((volatile int *)(c)) = (d))
#define ahb_read(a,b)               ((b) = *((volatile unsigned char *)((TCC_SATA_PHY_VIRT_ADDR) + (a))))
#define ahb_write(a,b)              (*((volatile unsigned char *)((TCC_SATA_PHY_VIRT_ADDR) + (a))) = (unsigned char)(b))

#define AHSATA_NUM_PORTS	 1
#define S_HADDR_WIDTH 32
#define AHB_ADDR_WIDTH S_HADDR_WIDTH
#define HDATA_WIDTH S_HADDR_WIDTH
#define AHB_DATA_WIDTH ((S_HDATA_WIDTH == 32) ? 32 : 64)

#if 0
static int run_error;
static int check_error;
static int err_count;
#endif

// Register Addresses

// Generic Registers
#define ATB_GCSR_CAP       0x00000000
#define ATB_GCSR_GHC       0x00000004
#define ATB_GCSR_IS        0x00000008
#define ATB_GCSR_PI        0x0000000C
#define ATB_GCSR_VS        0x00000010
#define ATB_GCSR_CCC_CTL   0x00000014
#define ATB_GCSR_CCC_PORTS 0x00000018
#define ATB_GCSR_EMLOC     0x0000001C
#define ATB_GCSR_EMCTL     0x00000020
#define ATB_GCSR_BISTAFR   0x000000A0
#define ATB_GCSR_BISTCR    0x000000A4
#define ATB_GCSR_BISTFCTR  0x000000A8
#define ATB_GCSR_BISTSR    0x000000AC
#define ATB_GCSR_BISTDECR  0x000000B0
#define ATB_GCSR_GPARAM1R  0x000000E8
#define ATB_GCSR_GPARAM2R  0x000000EC
#define ATB_GCSR_PPARAMR   0x000000F0
#define ATB_GCSR_TESTR     0x000000F4
#define ATB_GCSR_VERSIONR  0x000000F8
#define ATB_GCSR_IDR       0x000000FC
#define ATB_GCSR_OOBR      0x000000BC

// Generic Registers Write Mask
// ********* NOTE *************
// These simple masks are not correct
// and not complete!!!!!!!!!!!
// Correct masks & testing on
// individula register bits needs
// to be done by CRV testing
// exensively.
#define RTB_GCSR_CAP       0x00000000
#define RTB_GCSR_GHC       0x80000002
#define RTB_GCSR_IS        0x00000000
#define RTB_GCSR_PI        0x00000000
#define RTB_GCSR_VS        0x00000000
#define RTB_GCSR_CCC_CTL   0x00000000
#define RTB_GCSR_CCC_PORTS 0x00000000
#define RTB_GCSR_EMLOC     0x00000000
#define RTB_GCSR_EMCTL     0x00000000
#define RTB_GCSR_BISTAFR   0x00000000
#define RTB_GCSR_BISTCR    0x0000077F
#define RTB_GCSR_BISTFCTR  0x00000000
#define RTB_GCSR_BISTSR    0x00000000
#define RTB_GCSR_BISTDECR  0x00000000
#define RTB_GCSR_GPARAM1R  0x00000000
#define RTB_GCSR_GPARAM2R  0x00000000
#define RTB_GCSR_PPARAMR   0x00000000
#define RTB_GCSR_TESTR     0x00070001
#define RTB_GCSR_VERSIONR  0x00000000
#define RTB_GCSR_IDR       0x00000000


// Port Registers
// Must add Port offset to the following
#define ATB_PCSR_CLB       0x00000000
#define ATB_PCSR_CLBU      0x00000004
#define ATB_PCSR_FB        0x00000008
#define ATB_PCSR_FBU       0x0000000C
#define ATB_PCSR_IS        0x00000010
#define ATB_PCSR_IE        0x00000014
#define ATB_PCSR_CMD       0x00000018
#define ATB_PCSR_TFD       0x00000020
#define ATB_PCSR_SIG       0x00000024
#define ATB_PCSR_SSTS      0x00000028
#define ATB_PCSR_SCTL      0x0000002C
#define ATB_PCSR_SERR      0x00000030
#define ATB_PCSR_SACT      0x00000034
#define ATB_PCSR_CI        0x00000038
#define ATB_PCSR_SNTF      0x0000003C
#define ATB_PCSR_DMACR     0x00000070
#define ATB_PCSR_PHYCR     0x00000078
#define ATB_PCSR_PHYSR     0x0000007C

// Port Registers Write Mask
// ********* NOTE *************
// These simple masks are not correct
// and not complete!!!!!!!!!!!
// Correct masks & testing on
// individula register bits needs
// to be done by CRV testing
// exensively.
#define RTB_PCSR_CLB       0xFFFFFC00
#define RTB_PCSR_CLBU      0xFFFFFFFF
#define RTB_PCSR_FB        0xFFFFFF00
#define RTB_PCSR_FBU       0xFFFFFFFF
#define RTB_PCSR_IS        0x7D80003F
#define RTB_PCSR_IE        0x7DC0007F
#define RTB_PCSR_CMD       0x0F82C017
#define RTB_PCSR_TFD       0x0000FFFF
#define RTB_PCSR_SIG       0xFFFFFFFF
#define RTB_PCSR_SSTS      0x00000113
#define RTB_PCSR_SCTL      0x00000330
#define RTB_PCSR_SERR      0x07EF0f03
#define RTB_PCSR_SACT      0xFFFFFFFF
#define RTB_PCSR_CI        0xFFFFFFFF
#define RTB_PCSR_SNTF      0x0000FFFF
#define RTB_PCSR_DMACR     0x000000FF
#define RTB_PCSR_PHYCR     0xFFFFFFFF
#define RTB_PCSR_PHYSR     0x00000000


// Generic & Port Offsets
// Includes Host Located at 0x01000000 offset
#define ATB_GL_OFFSET      0x01000000
#define ATB_P0_OFFSET      0x01000100
#define ATB_P1_OFFSET      0x01000180
#define ATB_P2_OFFSET      0x01000200
#define ATB_P3_OFFSET      0x01000280
#define ATB_P4_OFFSET      0x01000300
#define ATB_P5_OFFSET      0x01000380
#define ATB_P6_OFFSET      0x01000400
#define ATB_P7_OFFSET      0x01000480


// These need to be here to prevent errors in existing code
// Register Addresses
#define CDR0   0x01000000
#define CDR1   0x01000004
#define CDR2   0x01000008
#define CDR3   0x0100000C
#define CDR4   0x01000010
#define CDR5   0x01000014
#define CDR6   0x01000018
#define CDR7   0x0100001C
#define CLR0   0x01000020
#define SCR0   0x01000024
#define SCR2   0x0100002C
#define DMACR  0x01000070
#define DBTSR  0x01000074

// Bit 0 CDR7 Access
#define CDR7_ERR 0

// Other Defines
#define BSYDRQ_TIMEOUT 1000
#define H_SREG 8
#define H_PIOREG 4096
#define CDR6_RESET_MASK 0xef
#define MAX_DWORDS_XFR (2048 * 8)

#define REG_FIS_H2D 0x27

// Command Types

// NDC Commands
#define CMD_NOP         0x00

// PIO Commands
#define WRITE_SECTOR    0x30
#define READ_SECTOR     0x20

// DMA Commands
#define WRITE_DMA       0xca
#define READ_DMA        0xc8

// DMA Defines
#define SS_2    2
#define SS_4    4
#define SS_8    8
#define SS_16   16
#define SS_32   32
#define SS_64   64
#define SS_128  128
#define SS_256  256
#define SS_512  512
#define SS_1024 1024
#define SS_2048 2048
#define SS_8192 8192

#define SECTOR_SIZE           512
#define SECTOR_SIZE_DW        (512 / 4)

// DMA controller block sizes... in Dwords.
#define DMA_BLK_SIZE_DW_0         0
#define DMA_BLK_SIZE_DW_32        32
#define DMA_BLK_SIZE_DW_64        64
#define DMA_BLK_SIZE_DW_256       256
#define DMA_BLK_SIZE_DW_1024      1024
#define DMA_BLK_SIZE_DW_2048      2048
#define DMA_MAX_BLOCK_SIZE_DWORDS #DMA_BLK_SIZE_DW_32
//#define DMA_MAX_BLOCK_SIZE_DWORDS #DMA_BLK_SIZE_DW_1024

// DMA Start Write Address
#define DMA_START_ADDR  0x40000400

//
// DUT Base address (for all HBA registers)
//
#define HBA_BASE_ADDRESS   0xf0810000

//
// Arbitrary addresses somewhere in slave 2
//
//#define SYS_MEM_FB_START   0x02000000
//#define SYS_MEM_CLB_START  0x02100000
//#define SYS_MEM_DATA_START 0x02C00000
#define SYS_MEM_FB_START   0x40000000
#define SYS_MEM_CLB_START  0x40010000
#define SYS_MEM_DATA_START 0x400C0000

// Command List Base Addresses
#define P0_CLB_ADDRESS     0x00000000
#define P1_CLB_ADDRESS     0x00000400
#define P2_CLB_ADDRESS     0x00000800
#define P3_CLB_ADDRESS     0x00000C00
#define P4_CLB_ADDRESS     0x00001000
#define P5_CLB_ADDRESS     0x00001400
#define P6_CLB_ADDRESS     0x00001800
#define P7_CLB_ADDRESS     0x00001C00

// FIS Receive Base Addresses
#define P0_FB_ADDRESS      0x00000000
#define P1_FB_ADDRESS      0x00000100
#define P2_FB_ADDRESS      0x00000200
#define P3_FB_ADDRESS      0x00000280
#define P4_FB_ADDRESS      0x00000300
#define P5_FB_ADDRESS      0x00000380
#define P6_FB_ADDRESS      0x00000400
#define P7_FB_ADDRESS      0x00000480

//
// Port command table addresses
//
//#define P0_CTB_ADDR        0x02802000
//#define P1_CTB_ADDR        0x02804000
//#define P2_CTB_ADDR        0x02806000
//#define P3_CTB_ADDR        0x02808000
//#define P4_CTB_ADDR        0x0280A000
//#define P5_CTB_ADDR        0x0280C000
//#define P6_CTB_ADDR        0x0280E000
//#define P7_CTB_ADDR        0x02810000
#define P0_CTB_ADDR        0x40082000
#define P1_CTB_ADDR        0x40084000
#define P2_CTB_ADDR        0x40086000
#define P3_CTB_ADDR        0x40088000
#define P4_CTB_ADDR        0x4008A000
#define P5_CTB_ADDR        0x4008C000
#define P6_CTB_ADDR        0x4008E000
#define P7_CTB_ADDR        0x40090000

//
// Command slot offsets
//
#define CMD_SLOT_0    0x00000000
#define CMD_SLOT_1    0x00000020
#define CMD_SLOT_2    0x00000040
#define CMD_SLOT_3    0x00000060
#define CMD_SLOT_4    0x00000080
#define CMD_SLOT_5    0x000000A0
#define CMD_SLOT_6    0x000000C0
#define CMD_SLOT_7    0x000000E0
#define CMD_SLOT_8    0x00000100
#define CMD_SLOT_9    0x00000120
#define CMD_SLOT_10   0x00000140
#define CMD_SLOT_11   0x00000160
#define CMD_SLOT_12   0x00000180
#define CMD_SLOT_13   0x000001A0
#define CMD_SLOT_14   0x000001C0
#define CMD_SLOT_15   0x000001E0
#define CMD_SLOT_16   0x00000200
#define CMD_SLOT_17   0x00000220
#define CMD_SLOT_18   0x00000240
#define CMD_SLOT_19   0x00000260
#define CMD_SLOT_20   0x00000280
#define CMD_SLOT_21   0x000002A0
#define CMD_SLOT_22   0x000002C0
#define CMD_SLOT_23   0x000002E0
#define CMD_SLOT_24   0x00000300
#define CMD_SLOT_25   0x00000320
#define CMD_SLOT_26   0x00000340
#define CMD_SLOT_27   0x00000360
#define CMD_SLOT_28   0x00000380
#define CMD_SLOT_29   0x000003A0
#define CMD_SLOT_30   0x000003C0
#define CMD_SLOT_31   0x000003E0

//
// Command table offsets
//
#define CMD_TBL_CFIS_OFFSET  0x00000000
#define CMD_TBL_ACMD_OFFSET  0x00000040
#define CMD_TBL_PRD0_OFFSET  0x00000080
#define CMD_TBL_PRD1_OFFSET  0x00000084
#define CMD_TBL_PRD2_OFFSET  0x00000088
#define CMD_TBL_PRD3_OFFSET  0x0000008C
#define CMD_TBL_PRD4_OFFSET  0x00000090
#define CMD_TBL_PRD5_OFFSET  0x00000084
#define CMD_TBL_PRD6_OFFSET  0x00000088
#define CMD_TBL_PRD7_OFFSET  0x0000008C

//
// ATA Commands
//
#define CMD_CODE_SMART                                   0xB0
#define CMD_CODE_DEVICE_CONFIGURATION                    0xB1
#define CMD_CODE_EXECUTE_DEVICE_DIAGNOSTIC               0x90

//
// DMA Read Commands
//
#define CMD_CODE_READ_DMA_EXT                            0x25
#define CMD_CODE_READ_DMA                                0xC8

//
// DMA Write Commands
//
#define CMD_CODE_WRITE_DMA_EXT                           0x35
#define CMD_CODE_WRITE_DMA                               0xCA

//
// Read DMA Queued Commands
//
#define CMD_CODE_READ_DMA_QUEUED_EXT                     0x26
#define CMD_CODE_READ_DMA_QUEUED                         0xC7

//
// Write DMA Queued Commands
//
#define CMD_CODE_WRITE_DMA_QUEUED_EXT                    0x36
#define CMD_CODE_WRITE_DMA_QUEUED                        0xCC

//
// Device Reset Command
//
#define CMD_CODE_DEVICE_RESET                            0x08

//
// Non-Data Commands
//
#define CMD_CODE_CFA_ERASE_SECTORS                       0xC0
#define CMD_CODE_CFA_REQUEST_EXTENDED_ERROR              0x03
#define CMD_CODE_CHECK_MEDIA_CARD_TYPE                   0xD1
#define CMD_CODE_CHECK_POWER_MODE                        0xE5
#define CMD_CODE_DEVICE_CONFIGURATION_FREEZE_LOCK        #CMD_CODE_DEVICE_CONFIGURATION
#define CMD_FEAT_DEVICE_CONFIGURATION_FREEZE_LOCK        0xC1
#define CMD_CODE_DEVICE_CONFIGURATION_RESTORE            #CMD_CODE_DEVICE_CONFIGURATION
#define CMD_FEAT_DEVICE_CONFIGURATION_RESTORE            0xC0
#define CMD_CODE_FLUSH_CACHE_EXT                         0xEA
#define CMD_CODE_FLUSH_CACHE                             0xE7
#define CMD_CODE_GET_MEDIA_STATUS                        0xDA
#define CMD_CODE_IDLE_IMMEDIATE                          0xE1
#define CMD_CODE_IDLE                                    0xE3
#define CMD_CODE_MEDIA_EJECT                             0xED
#define CMD_CODE_MEDIA_LOCK                              0xDE
#define CMD_CODE_MEDIA_UNLOCK                            0xDF
#define CMD_CODE_NOP                                     0x00
#define CMD_CODE_READ_NATIVE_MAX_ADDRESS_EXT             0x27
#define CMD_CODE_READ_NATIVE_MAX_ADDRESS                 0xF8
#define CMD_CODE_READ_VERIFY_SECTOR_S_EXT                0x42
#define CMD_CODE_READ_VERIFY_SECTOR_S                    0x40
#define CMD_CODE_SECURITY_ERASE_PREPARE                  0xF3
#define CMD_CODE_SECURITY_FREEZE_LOCK                    0xF5
#define CMD_CODE_SEEK                                    0x70
#define CMD_CODE_SET_FEATURES                            0xEF
#define CMD_CODE_SET_MAX_ADDRESS_EXT                     0x37
#define CMD_CODE_SET_MAX_ADDRESS                         0xF9
#define CMD_CODE_SET_MULTIPLE_MODE                       0xC6
#define CMD_CODE_SLEEP                                   0xE6
#define CMD_CODE_SMART_DISABLE_OPERATIONS                #CMD_CODE_SMART
#define CMD_FEAT_SMART_DISABLE_OPERATIONS                0xD9
#define CMD_CODE_SMART_ENABLE_OPERATIONS                 #CMD_CODE_SMART
#define CMD_FEAT_SMART_ENABLE_OPERATIONS                 0xD8
#define CMD_CODE_SMART_ENABLE_DISABLE_AUTOSAVE           #CMD_CODE_SMART
#define CMD_FEAT_SMART_ENABLE_DISABLE_AUTOSAVE           0xD2
#define CMD_CODE_SMART_EXECUTE_OFF_LINE                  #CMD_CODE_SMART
#define CMD_FEAT_SMART_EXECUTE_OFF_LINE                  0xD4
#define CMD_CODE_SMART_RETURN_STATUS                     #CMD_CODE_SMART
#define CMD_FEAT_SMART_RETURN_STATUS                     0xDA
#define CMD_CODE_STANDBY_IMMEDIATE                       0xE0
#define CMD_CODE_STANDBY                                 0xE2

//
// Packet Command
//
#define CMD_CODE_PACKET                                  0xA0

#define CMD_CODE_SERVICE                                 0xA2

//
// PIO Read Commands
//
#define CMD_CODE_CFA_TRANSLATE_SECTOR                    0x87
#define CMD_CODE_DEVICE_CONFIGURATION_IDENTIFY           #CMD_CODE_DEVICE_CONFIGURATION
#define CMD_FEAT_DEVICE_CONFIGURATION_IDENTIFY           0xC2
#define CMD_CODE_IDENTIFY_DEVICE                         0xEC
#define CMD_CODE_IDENTIFY_PACKET_DEVICE                  0xA1
#define CMD_CODE_READ_BUFFER                             0xE4
#define CMD_CODE_READ_LOG_EXT                            0x2F
#define CMD_CODE_READ_MULTIPLE_EXT                       0x29
#define CMD_CODE_READ_MULTIPLE                           0xC4
#define CMD_CODE_READ_SECTOR_S_EXT                       0x24
#define CMD_CODE_READ_SECTOR_S                           0x20
#define CMD_CODE_SMART_READ_DATA                         #CMD_CODE_SMART
#define CMD_FEAT_SMART_READ_DATA                         0xD0
#define CMD_CODE_SMART_READ_LOG_SECTOR                   #CMD_CODE_SMART
#define CMD_FEAT_SMART_READ_LOG_SECTOR                   0xD5

//
// PIO Write Commands
//
#define CMD_CODE_CFA_WRITE_MULTIPLE_W_OUT_ERASE          0xCD
#define CMD_CODE_CFA_WRITE_SECTORS_W_OUT_ERASE           0x38
#define CMD_CODE_DEVICE_CONFIGURATION_SET                #CMD_CODE_DEVICE_CONFIGURATION
#define CMD_FEAT_DEVICE_CONFIGURATION_SET                0xC3
#define CMD_CODE_DOWNLOAD_MICROCODE                      0x92
#define CMD_CODE_SECURITY_DISABLE_PASSWORD               0xF6
#define CMD_CODE_SECURITY_ERASE_UNIT                     0xF4
#define CMD_CODE_SECURITY_SET_PASSWORD                   0xF1
#define CMD_CODE_SECURITY_UNLOCK                         0xF2
#define CMD_CODE_SMART_WRITE_LOG_SECTOR                  #CMD_CODE_SMART
#define CMD_FEAT_SMART_WRITE_LOG_SECTOR                  0xD6
#define CMD_CODE_WRITE_BUFFER                            0xE8
#define CMD_CODE_WRITE_LOG_EXT                           0x3F
#define CMD_CODE_WRITE_MULTIPLE_EXT                      0x39
#define CMD_CODE_WRITE_MULTIPLE                          0xC5
#define CMD_CODE_WRITE_SECTOR_S_EXT                      0x34
#define CMD_CODE_WRITE_SECTOR_S                          0x30

//
// FPDMA Commands
//
#define CMD_CODE_READ_FPDMA_QUEUED                       0x60
#define CMD_CODE_WRITE_FPDMA_QUEUED                      0x61

//
// PM Types
//
#define VTB_PM_PARTIAL  1
#define VTB_PM_SLUMBER  2

//
// BIST Types
//
#define VTB_BIST_FAR_END_RETIMED          1
#define VTB_BIST_FAR_END_ANALOG           2
#define VTB_BIST_FAR_END_TRANSMIT_ONLY    3

//
// Global stream port
//
#define NON_PORT 32
#define INTERRUPT_PORT 33

//
// Bits if Port Interrupt Status register
//
#define PORT_INT_DHRS       0
#define PORT_INT_PSS        1
#define PORT_INT_DSS        2
#define PORT_INT_SDBS       3
#define PORT_INT_UFS        4
#define PORT_INT_DPS        5
#define PORT_INT_PCS        6
#define PORT_INT_DMPS       7
#define PORT_INT_PRCS       22
#define PORT_INT_IPMS       23
#define PORT_INT_OFS        24
#define PORT_INT_INFS       26
#define PORT_INT_IFS        27

