#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#ifdef CONFIG_FB_TCC92XX_HVGA_LMS350DF01
#include <mach/dev_gpsb.h>
#include <mach/TCC_LCD_Interface.h>
#include <mach/TCC_LCD_DriverCtrl.h>

static int gDRV_SPI_Handle_LMS350DF01 = -1;

static sDRV_GPSB	gDRV_SPI_LMS350DF01 =
{
	0 
	| DRV_GPSB_MODE_OpMod_SPI			// 0 : spi
	| DRV_GPSB_MODE_MasterMode			// 0 : Master mode enable	
	| DRV_GPSB_MODE_BitWidth(23)		// 31 : Bit width selection, 32bits = 31+1
	| DRV_GPSB_MODE_SCKPolCtrl_High 	// 16
	| DRV_GPSB_MODE_EnableOperation 	// 3
	| DRV_GPSB_MODE_RcvRisingEdge		// 17
	| DRV_GPSB_MODE_TransRisingEdge 	// 18
	| DRV_GPSB_MODE_ClearRcvFifoCount	// 1 : Assert Rx FIFO Counter Clear
//	| DRV_GPSB_MODE_SCKPolCtrl_High 	// 1 : SCKI inverted. for SPI timing 1, 2	

//	, 90000 							// Clock Divider
	, 60000 							// Clock Divider
	, DRV_GPSB_CTRL_HW					// Hardware Control
	, DRV_GPSB_SDOSDI_ALLUSE				// SPI Input Only
	, 0
};


void LMS350DF01_GPSB_SETConfig(sDRV_GPSB * pDRV_GPSB, int iCH, int iPort)
{
	GPSB* pHwGPSB;
	U32  uMode, uSCKFreq, uSwCtrl, uSDOSDI, uDIV;
			// Get Channel

	pDRV_GPSB->CHSTS		= DRV_GPSB_STATUS_IDLE;
	pDRV_GPSB->CHANNEL		= iCH;
	pDRV_GPSB->PORT 		= iPort;
	pDRV_GPSB->pHwGPSB		= IO_GPSB_GetBaseAddr(iCH);

	pHwGPSB 	= pDRV_GPSB->pHwGPSB;

	uMode		= pDRV_GPSB->MODE;
	uSCKFreq	= pDRV_GPSB->SCKFREQ;
	uSwCtrl 	= pDRV_GPSB->SWCTRL;
	uSDOSDI 	= pDRV_GPSB->SDOSDI;


	//Initialize GPSB Channel
	IO_GPSB_InitChannel(pDRV_GPSB->CHANNEL, pDRV_GPSB->PORT, uMode, uSwCtrl, uSDOSDI);

	//Set the MODE Register
	pHwGPSB->MODE = uMode;

	//Set Handle Status
	pDRV_GPSB->CHSTS = DRV_GPSB_STATUS_ACTIVE;


}



volatile unsigned int cmd_reg = 0x740000;
volatile unsigned int data_reg = 0x760000;
void LCD_SPI_WRITE_CMDDATA(unsigned short cmd, unsigned short data)
{
	unsigned int value;

	value = (cmd_reg | (cmd & 0xFFFF));
	IO_GPSB_TxRxData(DRV_GPSB_CH1, &value, 0, 1);
	value = (data_reg | (data & 0xFFFF));
	IO_GPSB_TxRxData(DRV_GPSB_CH1, &value, 0, 1);
}

void LMS350DF01_initialize(void)
{
	// Install Interrupt Handler
	LMS350DF01_GPSB_SETConfig(&gDRV_SPI_LMS350DF01, DRV_GPSB_CH1, DRV_GPSB_PORT9);

	LCD_SPI_WRITE_CMDDATA(0x07, 0x0000);
		//wait moer than 6frames
	LCD_Driver_mdelay(16);
	
	LCD_SPI_WRITE_CMDDATA(0x11, 0x222F);
	LCD_SPI_WRITE_CMDDATA(0x12, 0x0F00);
	LCD_SPI_WRITE_CMDDATA(0x13, 0x7FE3);
	LCD_SPI_WRITE_CMDDATA(0x76, 0x2213);
	LCD_SPI_WRITE_CMDDATA(0x74, 0x0001);
	LCD_SPI_WRITE_CMDDATA(0x76, 0x0000);	
	LCD_SPI_WRITE_CMDDATA(0x10, 0x560C);
	//wait moer than 6frames
	LCD_Driver_mdelay(16);
	
	LCD_SPI_WRITE_CMDDATA(0x12, 0x0C63);
	//wait moer than 5frames	
//	LCD_Driver_mdelay(16 * 5);
	
	LCD_SPI_WRITE_CMDDATA(0x01, 0x0B3B);
	LCD_SPI_WRITE_CMDDATA(0x02, 0x0300);
	LCD_SPI_WRITE_CMDDATA(0x03, 0xC040);
	LCD_SPI_WRITE_CMDDATA(0x08, 0x0004);
	LCD_SPI_WRITE_CMDDATA(0x09, 0x000B);
	
	LCD_SPI_WRITE_CMDDATA(0x76, 0x2213);
	LCD_SPI_WRITE_CMDDATA(0x0B, 0x3340);
	LCD_SPI_WRITE_CMDDATA(0x0C, 0x0020);
	LCD_SPI_WRITE_CMDDATA(0x1C, 0x7770);
	LCD_SPI_WRITE_CMDDATA(0x76, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x0D, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x0E, 0x0500);
	LCD_SPI_WRITE_CMDDATA(0x14, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x15, 0x0803);
	LCD_SPI_WRITE_CMDDATA(0x16, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x30, 0x0005);
	LCD_SPI_WRITE_CMDDATA(0x31, 0x070F);
	LCD_SPI_WRITE_CMDDATA(0x32, 0x0300);
	LCD_SPI_WRITE_CMDDATA(0x33, 0x0003);
	LCD_SPI_WRITE_CMDDATA(0x34, 0x090C);
	LCD_SPI_WRITE_CMDDATA(0x35, 0x0001);
	LCD_SPI_WRITE_CMDDATA(0x36, 0x0001);
	LCD_SPI_WRITE_CMDDATA(0x37, 0x0303);
	LCD_SPI_WRITE_CMDDATA(0x38, 0x0F09);
	LCD_SPI_WRITE_CMDDATA(0x39, 0x0105);

	//Display ON Sequence
	LCD_SPI_WRITE_CMDDATA(0x07, 0x0001);
	//wait moer than 5frames		
//	LCD_Driver_mdelay(16 * 5);

	LCD_SPI_WRITE_CMDDATA(0x07, 0x0101);
	//wait moer than 5frames		
//	LCD_Driver_mdelay(16 * 5);

	LCD_SPI_WRITE_CMDDATA(0x07, 0x0103);

}

void LMS350DF01_PowerOff(void)
{
	// Install Interrupt Handler
	LMS350DF01_GPSB_SETConfig(&gDRV_SPI_LMS350DF01, DRV_GPSB_CH1, DRV_GPSB_PORT9);

	LCD_SPI_WRITE_CMDDATA(0x10, 0x0001);
	LCD_SPI_WRITE_CMDDATA(0x0B, 0x30E1);
	LCD_SPI_WRITE_CMDDATA(0x07, 0x0102);

	LCD_SPI_WRITE_CMDDATA(0x07, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x12, 0x0000);
	LCD_SPI_WRITE_CMDDATA(0x10, 0x0100);
}

// Unit 100hz
#define LCDC_CLK		(90000)
#define LCD_DIVIDER		2

#define LCD_BUS_WIDTH	24

#define LPW		0	// line pulse width
#define LPC		320 // line pulse count (active horizontal pixel - 1)
#define LSWC	9	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	6	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay

#define FPW1	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	480	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	4	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	2	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	0	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	480	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	4	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	2	// frame start wait cycle is the number of lines to insert at the begining each frame

#define SYNC_INVET (IV_INVERT_EN|IH_INVERT_EN)
	
void DEV_LCD_MODULE_SPEC(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	spec->lcd_width = 320;
	spec->lcd_height = 480;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}


void GPSB_Port_disable(void)
{
	#define LCD_GPSB_CH  1
	GPSB *pGPSB_CH1;

//	IO_GPSB_DisableChannel(DRV_GPSB_CH1);

	/* gpio F24 : SDO,  F25: SDI,  F26 : SCLK   F27 : SFRM */
	BITCSET(HwGPIOF->GPFN3, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK,
			HwPORTCFG_GPFN0(0) | HwPORTCFG_GPFN1(0) | HwPORTCFG_GPFN2(0) | HwPORTCFG_GPFN3(0) );

	BITCSET(HwGPIOF->GPEN, Hw24|Hw25|Hw26|Hw27, Hw24|Hw25|Hw26|Hw27);
	BITCLR(HwGPIOF->GPDAT, Hw24|Hw25|Hw26|Hw27);
}

 void DEV_LCD_Module (LCD_MODULE_CTRL uiControl)
{
	switch(uiControl)
	{
		case LCD_M_INITIALIZE:
			LMS350DF01_initialize();
			break;

		case LCD_M_ON:
			LMS350DF01_initialize();
			break;

		case LCD_M_OFF:
			LMS350DF01_PowerOff();
			break;
		default:
			break;
	}
	GPSB_Port_disable();
}
#endif//

