
/****************************************************************************
 *   FileName    : TCC_LCD_interfface.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif

#include <mach/dev_gpsb.h>
#include <mach/TCC_LCD_Interface.h>


#if 0
#define dprintk(msg...)	 { printk( "LCD_interface: " msg); }
#else
#define dprintk(msg...)	 
#endif


void LCD_Driver_mdelay(unsigned int msec)
{
	msleep(msec);	
}

void LCD_Driver_udelay(unsigned int usec)
{
	udelay(usec);	
}

// 5ms  
void LCD_OS_DELAY(unsigned int delay)
{
	while(delay--)
	{
		msleep(5);	
	}
}





void GPSB_SETConfig(sDRV_GPSB * pDRV_GPSB, int iCH, int iPort)
{

	GPSB*	pHwGPSB;		// Base Address

	U32	uMode, uSCKFreq, uSwCtrl, uSDOSDI, uDIV;
			// Get Channel

	pDRV_GPSB->CHSTS		= DRV_GPSB_STATUS_IDLE;
	pDRV_GPSB->CHANNEL		= iCH;
	pDRV_GPSB->PORT			= iPort;
	pDRV_GPSB->pHwGPSB		= IO_GPSB_GetBaseAddr(iCH);

	pHwGPSB		= pDRV_GPSB->pHwGPSB;

	uMode		= pDRV_GPSB->MODE;
	uSCKFreq	= pDRV_GPSB->SCKFREQ;
	uSwCtrl		= pDRV_GPSB->SWCTRL;
	uSDOSDI		= pDRV_GPSB->SDOSDI;

		//Get and Set the Divider value
#if 1
	if(uSCKFreq)
	{
		uDIV = 900000 / (2*uSCKFreq);
		if (uDIV > 0)
			uDIV--;
		uMode 				|= uDIV*Hw24;
		pDRV_GPSB->MODE 	= uMode;
	}
#endif//			
	//Initialize GPSB Channel
	IO_GPSB_InitChannel(pDRV_GPSB->CHANNEL, pDRV_GPSB->PORT, uMode, uSwCtrl, uSDOSDI);

	//Set the MODE Register
	pHwGPSB->MODE = uMode;

	//Set Handle Status
	pDRV_GPSB->CHSTS = DRV_GPSB_STATUS_ACTIVE;
}




static sDRV_GPSB gDRV_SPI_LCD;

/* -------------------------------------------------
clock : GPSB clock  //100hz .

---------------------------------------------------*/
void DEV_LCD_SPIO_Config(DEV_LCD_SPIO_Config_type *Lcd_gpsg)
{
	dprintk(KERN_INFO "%s :  \n", __FUNCTION__);

	gDRV_SPI_LCD.MODE = 0 
						| DRV_GPSB_MODE_OpMod_SPI 			// 0 : spi
						| DRV_GPSB_MODE_MasterMode			// 0 : Master mode enable	
						| DRV_GPSB_MODE_BitWidth(Lcd_gpsg->data_length - 1)		// 31 : Bit width selection, 32bits = 31+1
					//	| DRV_GPSB_MODE_SCKPolCtrl_High		// 16
						| DRV_GPSB_MODE_EnableOperation		// 3
					//	| DRV_GPSB_MODE_RcvRisingEdge		// 17
					//	| DRV_GPSB_MODE_TransRisingEdge		// 18
						| DRV_GPSB_MODE_ClearRcvFifoCount	// 1 : Assert Rx FIFO Counter Clear
					//	| DRV_GPSB_MODE_SCKPolCtrl_High		// 1 : SCKI inverted. for SPI timing 1, 2	
						; // GPSB MDOE SET
	
	gDRV_SPI_LCD.SCKFREQ = Lcd_gpsg->clock;
	gDRV_SPI_LCD.SWCTRL = DRV_GPSB_CTRL_HW;
	gDRV_SPI_LCD.SDOSDI = DRV_GPSB_SDOSDI_ALLUSE;

	GPSB_SETConfig(&gDRV_SPI_LCD, DRV_GPSB_CH1, Lcd_gpsg->port);

}

unsigned int DEV_LCD_SPIO_TxRxData(void *pTxBuf, void *pRxBuf, unsigned uLength)
{
	unsigned int ret;

	ret= IO_GPSB_TxRxData(DRV_GPSB_CH1, pTxBuf, pRxBuf, uLength);

	return ret;
}


/* -------------------------------------------------
I2C interface 
---------------------------------------------------*/


/* 
 * register structure
 */
#pragma pack(push, 4)
struct tcc_gpio {
	volatile unsigned long DAT, EN, SET, CLR, XOR, CD0, CD1, PD0, PD1, FN0, FN1, FN2, FN3;
};

struct tcc_i2c_regs {
	volatile unsigned long PRES, CTRL, TXR, CMD, RXR, SR, TR;
};

struct tcc_i2c {
	int ch;
	unsigned int clk;
	unsigned long IRQSTR;
	volatile struct tcc_i2c_regs *regs;
};


static struct tcc_i2c i2c[4] = {
	/* I2C Controller */
	[I2C_CH0] = {
		.ch = 0,
		.clk = 100,
		.IRQSTR = I2C_IRQSTR,
		.regs = (volatile struct tcc_i2c_regs *)I2C_CH0_BASE,
	},
	[I2C_CH1] = {
		.ch = 1,
		.clk = 100,
		.IRQSTR = I2C_IRQSTR,
		.regs = (volatile struct tcc_i2c_regs *)(I2C_CH0_BASE + CH1_BASE_OFFSET),
	},
	/* SMU_I2C Controller */
	[SMU_I2C_CH0] = {
		.ch = 2,
		.clk = 100,
		.IRQSTR = 0,
		.regs = (volatile struct tcc_i2c_regs *)SMU_I2C_CH0_BASE,
	},
	[SMU_I2C_CH1] = {
		.ch = 3,
		.clk = 100,
		.IRQSTR = 0,
		.regs = (volatile struct tcc_i2c_regs *)(SMU_I2C_CH0_BASE + CH1_BASE_OFFSET),
	},
};

#define read_reg(a)			(*(volatile unsigned long *)a)

static int wait_intr(ch)
{
	volatile unsigned long cnt = 0;

	if (ch == 0 || ch == 1) {
		while (!(read_reg(i2c[ch].IRQSTR) & (ch?Hw1:Hw0))) {
			cnt++;
			if (cnt > 100000) {
				return -1;
			}
		}
	} else {
		/* SMU_I2C wait */
		while (1) {
			cnt = i2c[ch].regs->SR;
			if (!(cnt & Hw1)) break;
		}
		for (cnt = 0; cnt <15; cnt++);
	}

	return 0;
}

int i2c_xfer(unsigned char slave_addr, 
		unsigned char out_len, unsigned char* out_buf, 
		unsigned char in_len, unsigned char* in_buf, 
		int ch)
{
	int ret;
	unsigned int i = 0;

#if 0
	if ((ch > 1)||(ch < 0))
		return -1;
#endif

	/* 
	 * WRITE 
	 */
	if (out_len > 0) {
		i2c[ch].regs->TXR = slave_addr | I2C_WR;
		i2c[ch].regs->CMD = Hw7 | Hw4;

		ret = wait_intr(ch);
		if (ret != 0) return ret;
		BITSET(i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt

		for (i = 0; i < out_len; i++) {
			i2c[ch].regs->TXR = out_buf[i];
			i2c[ch].regs->CMD = Hw4;

			ret = wait_intr(ch);
			if (ret != 0) return ret;
			BITSET(i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt
		}

		i2c[ch].regs->CMD = Hw6;

		ret = wait_intr(ch);
		if (ret != 0) return ret;
		BITSET(i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt
	}

	/* 
	 * READ
	 */
	if (in_len > 0) {
		i2c[ch].regs->TXR = slave_addr | I2C_RD;
		i2c[ch].regs->CMD = Hw7 | Hw4;

		ret = wait_intr(ch);
		if (ret != 0) return ret;
		BITSET(i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt

		for (i = 0; i < in_len; i++) {
			i2c[ch].regs->CMD = Hw5 | Hw3;

			ret = wait_intr(ch);
			if (ret != 0) return ret;
			BITSET( i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt

			in_buf[i] =(unsigned char)i2c[ch].regs->RXR;
		}

		i2c[ch].regs->CMD = Hw6;

		ret = wait_intr(ch);
		if (ret != 0) return ret;
		BITSET( i2c[ch].regs->CMD, Hw0); //Clear a pending interrupt
	}

	return 0;
}




void DEV_Lcd_PortSet(unsigned int bit_per_pixel)
{	
	BITCSET(HwGPIOC->GPEN, Hw24|Hw25|Hw26|Hw27|Hw28|Hw29|Hw30|Hw31, Hw24|Hw25|Hw26|Hw27|Hw28|Hw29|Hw30|Hw31);
	BITCSET(HwGPIOC->GPDAT, Hw24|Hw25|Hw26|Hw27|Hw28|Hw29|Hw30|Hw31, Hw24|Hw25|Hw26|Hw27|Hw28|Hw29|Hw30|Hw31);


	switch (bit_per_pixel) 
	{
		case 24:
			BITCSET(HwGPIOC->GPEN, Hw23|Hw22|Hw21|Hw20|Hw19|Hw18, Hw23|Hw22|Hw21|Hw20|Hw19|Hw18);
		case 18:
			BITCSET(HwGPIOC->GPEN, Hw17|Hw16, Hw17|Hw16);


		case 16:
			BITCSET(HwGPIOC->GPEN, 0x0000FFFF, 0x0000FFFF);
				break;
			
		default:
			// do nothing
			break;
	}
}

void LCDC_IO_Set (char lcdctrl_num, unsigned bit_per_pixel)
{
	char LCD_PORT_ENABLE;


	if(lcdctrl_num)
		LCD_PORT_ENABLE = 5;
	else
		LCD_PORT_ENABLE = 2;

#ifdef CONFIG_LCD_CPU_INTERFACE
	DEV_Lcd_PortSet(bit_per_pixel);
	LCD_PORT_ENABLE = 1;
#endif//_CONFIG_LCD_CPU_INTERFACE_	

	BITCSET(HwGPIOC->GPFN3, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK,
							HwPORTCFG_GPFN0(LCD_PORT_ENABLE) | HwPORTCFG_GPFN1(LCD_PORT_ENABLE) | HwPORTCFG_GPFN2(LCD_PORT_ENABLE) | 
							HwPORTCFG_GPFN3(LCD_PORT_ENABLE));

	switch (bit_per_pixel) {
		case 24:
			BITCSET(HwGPIOC->GPFN2, HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK |
									HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN2(LCD_PORT_ENABLE) | HwPORTCFG_GPFN3(LCD_PORT_ENABLE) | HwPORTCFG_GPFN4(LCD_PORT_ENABLE) | 
									HwPORTCFG_GPFN5(LCD_PORT_ENABLE) | HwPORTCFG_GPFN6(LCD_PORT_ENABLE) | HwPORTCFG_GPFN7(LCD_PORT_ENABLE));
		case 18:
			BITCSET(HwGPIOC->GPFN2, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_ENABLE) | HwPORTCFG_GPFN1(LCD_PORT_ENABLE));

		case 16:
			BITCSET(HwGPIOC->GPFN1, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | 
									HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK | HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_ENABLE) | HwPORTCFG_GPFN1(LCD_PORT_ENABLE) | HwPORTCFG_GPFN2(LCD_PORT_ENABLE) | 
									HwPORTCFG_GPFN3(LCD_PORT_ENABLE) | HwPORTCFG_GPFN4(LCD_PORT_ENABLE) | HwPORTCFG_GPFN5(LCD_PORT_ENABLE) | 
									HwPORTCFG_GPFN6(LCD_PORT_ENABLE) | HwPORTCFG_GPFN7(LCD_PORT_ENABLE));
		case 8:
			BITCSET(HwGPIOC->GPFN0, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | 
									HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK | HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_ENABLE) | HwPORTCFG_GPFN1(LCD_PORT_ENABLE) | HwPORTCFG_GPFN2(LCD_PORT_ENABLE) | 
									HwPORTCFG_GPFN3(LCD_PORT_ENABLE) | HwPORTCFG_GPFN4(LCD_PORT_ENABLE) | HwPORTCFG_GPFN5(LCD_PORT_ENABLE) | 
									HwPORTCFG_GPFN6(LCD_PORT_ENABLE) | HwPORTCFG_GPFN7(LCD_PORT_ENABLE));
			break;
			
		default:
			// do nothing
			break;
	}

}

#define LCD_PORT_DISABLE	0	
void LCDC_IO_Disable (char lcdctrl_num, unsigned bit_per_pixel)
{

	BITCSET(HwGPIOC->GPFN3, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK,
							HwPORTCFG_GPFN0(LCD_PORT_DISABLE) | HwPORTCFG_GPFN1(LCD_PORT_DISABLE) | HwPORTCFG_GPFN2(LCD_PORT_DISABLE) | 
							HwPORTCFG_GPFN3(LCD_PORT_DISABLE));

	BITCSET(HwGPIOC->GPEN, Hw24|Hw25|Hw26|Hw27, Hw24|Hw25|Hw26|Hw27);
	BITCLR(HwGPIOC->GPDAT, Hw24|Hw25|Hw26|Hw27);


	switch (bit_per_pixel) {
		case 24:
			BITCSET(HwGPIOC->GPFN2, HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK |
									HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN2(LCD_PORT_DISABLE) | HwPORTCFG_GPFN3(LCD_PORT_DISABLE) | HwPORTCFG_GPFN4(LCD_PORT_DISABLE) | 
									HwPORTCFG_GPFN5(LCD_PORT_DISABLE) | HwPORTCFG_GPFN6(LCD_PORT_DISABLE) | HwPORTCFG_GPFN7(LCD_PORT_DISABLE));

			BITCSET(HwGPIOC->GPEN, Hw23|Hw22|Hw21|Hw20|Hw19|Hw18, Hw23|Hw22|Hw21|Hw20|Hw19|Hw18);
			BITCLR(HwGPIOC->GPDAT, Hw23|Hw22|Hw21|Hw20|Hw19|Hw18);

		case 18:
			BITCSET(HwGPIOC->GPFN2, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_DISABLE) | HwPORTCFG_GPFN1(LCD_PORT_DISABLE));

			BITCSET(HwGPIOC->GPEN, Hw17|Hw16, Hw17|Hw16);
			BITCLR(HwGPIOC->GPDAT, Hw17|Hw16);
	
		case 16:
			BITCSET(HwGPIOC->GPFN1, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | 
									HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK | HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_DISABLE) | HwPORTCFG_GPFN1(LCD_PORT_DISABLE) | HwPORTCFG_GPFN2(LCD_PORT_DISABLE) | 
									HwPORTCFG_GPFN3(LCD_PORT_DISABLE) | HwPORTCFG_GPFN4(LCD_PORT_DISABLE) | HwPORTCFG_GPFN5(LCD_PORT_DISABLE) | 
									HwPORTCFG_GPFN6(LCD_PORT_DISABLE) | HwPORTCFG_GPFN7(LCD_PORT_DISABLE));
			BITCSET(HwGPIOC->GPFN0, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN1_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK | 
									HwPORTCFG_GPFN4_MASK | HwPORTCFG_GPFN5_MASK | HwPORTCFG_GPFN6_MASK | HwPORTCFG_GPFN7_MASK,
									HwPORTCFG_GPFN0(LCD_PORT_DISABLE) | HwPORTCFG_GPFN1(LCD_PORT_DISABLE) | HwPORTCFG_GPFN2(LCD_PORT_DISABLE) | 
									HwPORTCFG_GPFN3(LCD_PORT_DISABLE) | HwPORTCFG_GPFN4(LCD_PORT_DISABLE) | HwPORTCFG_GPFN5(LCD_PORT_DISABLE) | 
									HwPORTCFG_GPFN6(LCD_PORT_DISABLE) | HwPORTCFG_GPFN7(LCD_PORT_DISABLE));

			BITCSET(HwGPIOC->GPEN, 0x0000FFFF, 0x0000FFFF);
			BITCLR(HwGPIOC->GPDAT, 0x0000FFFF);
			break;

		default:
			// do nothing
			break;
	}
	
	
}


