/*
 * tcc7901_pwrbtn.c
 *
 * Power key driver for tsc2003
 *
 */
 
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/miscdevice.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>

#include <mach/tcc79x.h>

#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#define	IRQ_PWRBTN		IRQ_EI1

static void PowerOnPeripherals(void);

static void pwr_bh(unsigned long tmp);
DECLARE_TASKLET(pwr_tasklet,pwr_bh,0);


typedef void (*lpfunc)(void);
static lpfunc lpSelfRefresh;

#define DEVICE_NAME	"pwr"
#define MAJOR_ID	201

static struct proc_dir_entry *pwr_entry = NULL;
static int pwr_status = 1;

#define SRAM_ADDR_STANDBY   0xF8000000 //FIXME: change SRAM_BASE_ADDR
#define SRAM_ADDR_VAR       0xF8001000
#define SRAM_FUNC_SIZE      0x400

void EnterStandbyMode2(void)
{
	volatile unsigned int *bakMODECTR;
    
	bakMODECTR = (volatile unsigned int *)SRAM_ADDR_VAR;
	//*bakMODECTR = HwMODECTR;
	HwMODECTR = *bakMODECTR;
}

#define DEBUG_PWRBTN
#ifdef DEBUG_PWRBTN
#define LED_ON	HwGPFDAT |= (1<<17)
#define LED_OFF	HwGPFDAT &= ~(1<<17)
#endif

void EnterStandbyMode(void)
{
	// Declare Pointers to backup CKC Registers
	unsigned long	*BAKCLKCTRL,		*BAKPLL0CFG, 	*BAKPLL1CFG,	*BAKCLKDIVC,	*BAKMODECTR;
	unsigned long	*BAKBCLKCTR0,	*BAKBCLKCTR1,	*BAKSWRESET0,	*BAKSWRESET1;
	unsigned long	*BAKBCLKMSKE0,	*BAKBCLKMSKE1,	*BAKPCK, *i, *j, *pPCK;

#ifdef DEBUG_PWRBTN
	LED_ON; // LED ON, for testing
#endif
	
	// Let CPU Speed Lower, Low Clock Operation
	// Assign Registers to Pointers
	BAKMODECTR		= (unsigned long*)(SRAM_ADDR_VAR);
	BAKPLL0CFG		= (unsigned long*)(SRAM_ADDR_VAR+0x04);
	BAKPLL1CFG		= (unsigned long*)(SRAM_ADDR_VAR+0x08);
	BAKCLKCTRL		= (unsigned long*)(SRAM_ADDR_VAR+0x0C);
	BAKCLKDIVC		= (unsigned long*)(SRAM_ADDR_VAR+0x10);
	BAKBCLKMSKE0	= (unsigned long*)(SRAM_ADDR_VAR+0x14);
	BAKBCLKMSKE1	= (unsigned long*)(SRAM_ADDR_VAR+0x18);
	BAKBCLKCTR0	= (unsigned long*)(SRAM_ADDR_VAR+0x1C);
	BAKBCLKCTR1	= (unsigned long*)(SRAM_ADDR_VAR+0x20);
	BAKSWRESET0	= (unsigned long*)(SRAM_ADDR_VAR+0x24);
	BAKSWRESET1	= (unsigned long*)(SRAM_ADDR_VAR+0x28);

	i = (unsigned long*)(SRAM_ADDR_VAR+0x30);
	j = (unsigned long*)(SRAM_ADDR_VAR+0x34);

	BAKPCK = (unsigned long*)(SRAM_ADDR_VAR+0x38);

	// Backup Related Registers
	*BAKMODECTR		=	HwMODECTR;
	*BAKPLL0CFG		=	HwPLL0CFG;
	*BAKPLL1CFG		=	HwPLL1CFG;
	*BAKCLKCTRL		=	HwCLKCTRL;
	*BAKCLKDIVC		=	HwCLKDIVC;
	*BAKBCLKMSKE0	=	HwBCLKMASKE;
	*BAKBCLKMSKE1	=	HwBCLKMASKE1;
	*BAKSWRESET0	=	HwSWRESET;
	*BAKSWRESET1	=	HwSWRESET1;

	// SDRAM into Self Refresh
	HwSDCFG	|=	0x1;
	while ((HwMCFG & 0xA0000) != 0xA0000);
	
	// Disable SDR CLK
	HwMCFG &= ~((1<<6) | (1<<1));

	// Save BCLKCTR and enable all of bus clock
	*BAKBCLKCTR0	=	HwBCLKCTR;
	*BAKBCLKCTR1	=	HwBCLKCTR1;

	HwBCLKCTR = 0xFFFFFFFF;
	HwBCLKCTR1 = 0xFFFFFFFF;
	
	// Save All of PCK_XXX Register
	pPCK	=	(unsigned long*)(&(HwPCK_USB11H));

	for( *i=0; *i<24; (*i)++ )
	{
		BAKPCK[*i] = *pPCK;
		if(((BAKPCK[*i] & 0x1f000000) != 0x16000000) &&
			((BAKPCK[*i] & 0x1f000000) != 0x17000000)){
				*pPCK = (BAKPCK[*i] & ~0x10000000);
		}
		pPCK ++;
	}

	// Clock Divided by Maximum value
	HwCLKDIVC |= 0xBF00;

	// Change CPU clock Source to XIN
	HwCLKCTRL = (HwCLKCTRL & ~0xF) | HwCLKCTRL_CKSEL_Fxindiv;
	
	// Disable PLL
	HwPLL0CFG |= HwPLL0CFG_PD_DIS;
	HwPLL1CFG |= HwPLL1CFG_PD_DIS;
	
	// IRQ all Clear
	HwCLR = 0xFFFFFFFF;
	
	// Enter PowerDown mode, Set MODECTR to PWDN mode.
	HwMODECTR |= (Hw8|Hw9);

	for( (*i)=10; (*i)>0 ; (*i)--)
		(*j) += (*i)/10;
		
	// Restore MODECTR register
	HwMODECTR	=	*BAKMODECTR;

	// Restore PLL
	HwPLL0CFG	=	*BAKPLL0CFG;
	HwPLL1CFG	=	*BAKPLL1CFG;

	for( (*i)=2; (*i)>0 ; (*i)--)
		(*j) += (*i)/10; // wait for PLL normal

	// Restore CLKCTR
	HwCLKCTRL	=	*BAKCLKCTRL;
	HwCLKDIVC	=	*BAKCLKDIVC;

	// Restore PCK_XXX
	pPCK = (unsigned long*)(&HwPCK_USB11H);

	for( (*i)=0; (*i)<24; (*i)++)
		*pPCK++ = BAKPCK[*i];

	for( (*i)=2; (*i)>0 ; (*i)--)
		(*j) += (*i)/10; // wait for PCLK normal

	// Restore BCLKCTR
	HwSWRESET	=	*BAKSWRESET0;
	HwSWRESET1	=	*BAKSWRESET1;
	
	HwBCLKMASKE	=	*BAKBCLKMSKE0;
	HwBCLKMASKE1	=	*BAKBCLKMSKE1;

	HwBCLKCTR	=	*BAKBCLKCTR0;
	HwBCLKCTR1 	=	*BAKBCLKCTR1;

	// SDRAM exit Self Refresh
	HwMCFG		|= ((1<<6) | (1<<1));
	HwSDCFG	&= ~0x1;   // exit self refresh mode

	// Wait until SDRAM stable
	while ((HwMCFG & 0xA0000) != 0xA0000);

	for( (*i)=2; (*i)>0 ; (*i)--)
		(*j) += (*i)/10; // wait for SDRAM normal

	(*i) = 0;
	// Clear External Interrupt
	HwCLR	=	0xF;

#ifdef DEBUG_PWRBTN
	LED_OFF; // LED ON, for testing
#endif
}

void CopyFuncsToSRAM(void)
{
    volatile unsigned int *fPtr;
    unsigned int *p;
    int i;

    // copy function EnterSelfrefreshMode to SRAM
	fPtr = (unsigned int*)EnterStandbyMode;
	lpSelfRefresh = (lpfunc)SRAM_ADDR_STANDBY;

	p = (unsigned int*)SRAM_ADDR_STANDBY;
	for (i = 0;i < SRAM_FUNC_SIZE;i++)
	{
		*p = *fPtr;
		p++;
		fPtr++;
	}
	
	// Jump to SRAM excute self-refresh mode
	lpSelfRefresh();
}

#if CONFIG_PM
static void PowerOffPeripherals(void)
{
	// make all devices suspend
	device_suspend(PMSG_SUSPEND);
	//device_power_down(PMSG_SUSPEND);
}

static void PowerOnPeripherals(void)
{
	// make all devices wakeup
	device_resume(PMSG_RESUME);
}
#endif

static void PowerOffHandler(void)
{
	unsigned long uMODE, uMODEA, uPOL, uIEN, uEINTSEL0, uEINTSEL1, uIRQMSK;
	volatile unsigned long* ptr =(volatile unsigned long *)0xF3000014;


	*ptr = 0;

	printk("Power OFF\n");

#if CONFIG_PM
	// Power off External Peripherals.
	PowerOffPeripherals(); 
	printk("PowerOffPeripherals completed\n");
#endif

	uIEN = HwIEN;
	uIRQMSK = HwIRQMSK;
	HwIEN = 0;
	HwIRQMSK = 0;
	HwCLR = 0xFFFFFFFF;
	
	uMODE = HwMODE;
	uMODEA = HwMODEA;
	uPOL = HwPOL;
	uEINTSEL0 = HwEINTSEL0;
	uEINTSEL1 = HwEINTSEL1;

	udelay(100);

	//HwPOL |= (1<<IRQ_PWRBTN);		// Active Low
	HwPOL = uPOL & (~(1<<IRQ_PWRBTN));
	//HwMODE |= (1<<IRQ_PWRBTN);		// EINT1 level-triggered
	HwMODE  = uMODE & (~(1<<IRQ_PWRBTN));        // IRQ_PWRBTN edge-triggered
	//HwMODEA &= ~(1<<IRQ_PWRBTN);		// EINT1 level-triggered is only single edge
	HwMODEA = uMODEA & (~(1<<IRQ_PWRBTN));      // IRQ_PWRBTN level-triggered is only single edge

	HwIRQMSK = (1<<IRQ_PWRBTN);
	HwIEN = (1<<IRQ_PWRBTN);

	//------------EnterStandbyMode-------------
	HwWKEN &= ~(1<<IRQ_PWRBTN); //EINT1 wakeup system, TCC79X datasheet is wrong
	CopyFuncsToSRAM();
	HwWKEN = 0xFFFFFFFF;	//Wakeup system disable, TCC79X datasheet is wrong

	HwPOL = uPOL;
	HwMODE = uMODE;
	HwMODEA = uMODEA;
	HwEINTSEL0 = uEINTSEL0;
	HwEINTSEL1 = uEINTSEL1;
	HwIRQMSK = uIRQMSK;
	HwIEN = uIEN;

#if CONFIG_PM
	// Power On External Peripherals
	PowerOnPeripherals(); 
#endif

	while(1){
		udelay(100);
		if(HwGPADAT & (1<<3))
			break;
	}
	while(1){
		udelay(100);
		if(HwGPADAT & (1<<3))
			break;
	}

	printk("Power ON\n");

	HwCLR = (1<<IRQ_PWRBTN);
	
	pwr_status = 1;
}

static void pwr_bh(unsigned long tmp)
{
	printk("POWER KEY BH\n");
}

// interrupt handler
static irqreturn_t pwrbtn_handler(int irq, void *dev_id)
{
	int i;
	
	// get rid of noise
	while (!(HwGPADAT & (1<<3)));
	for (i = 0; i < 10; i++)
	{
		mdelay(2);
		while (!(HwGPADAT & (1<<3)));
	}
	
	//printk("Power key OFF, PLL1:%x\n", HwPLL1CFG);
	
	pwr_status = 0;
	//PowerOffHandler();
	//device_suspend(PMSG_SUSPEND);
	//device_resume();
	
	//printk("Power key ON, PLL1:%x\n", HwPLL1CFG);

	//tasklet_schedule(&pwr_tasklet);
	return IRQ_HANDLED;
}

// enable PWRBTN interrupt
static void pwrbtn_enable_int(void)
{
#ifdef DEBUG_PWRBTN
	HwPORTCFG2 = (HwPORTCFG2 & (~(0xf<<12))) | (1<<12);
	HwGPFEN |= (1<<17);
#endif

	// Clear IRQ_PWRBTN
	HwCLR = (1<<IRQ_PWRBTN);

	HwPORTCFG4 &= ~0xF000;
	HwCPUD4 = (HwCPUD4 & (~0xc0000000)) | 0x40000000;
	HwEINTSEL0 = (HwEINTSEL0 & (~(0x1f<<8))) | HwEINTSEL0_EINT1(SEL_GPIOA3);

	HwSEL |= (1<<IRQ_PWRBTN);
	HwIRQMSK |= (1<<IRQ_PWRBTN);
	HwIEN &= ~(1<<IRQ_PWRBTN);
	
	// GPA3 input mode
	HwGPAEN &= ~(1<<3);
	
	// IRQ_PWRBTN to be single edge-triggered, rising edge active
	HwMODE |= (1<<IRQ_PWRBTN);
	HwMODEA &= ~(1<<IRQ_PWRBTN);
	HwPOL |= (1<<IRQ_PWRBTN);
	
	HwWKEN = 0xffffffff;	// IRQ_PWRBTN wakeup system
	
}

static int pwr_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static ssize_t pwr_read(struct file *filp, char *buffer, size_t count, loff_t *ppos)
{
	copy_to_user(buffer, &pwr_status, sizeof(pwr_status));	
	return 0;
}

static ssize_t pwr_write(struct file *filp, const char *buffer, size_t count, loff_t *ppos)
{
	PowerOffHandler();
	return 0;
}

static int pwr_release(struct inode *inode, struct file *filp)
{
	return 0;
}

static struct file_operations tcc7901_fops = {
	owner:	THIS_MODULE,
	open:	pwr_open,
	read:	pwr_read,	
	write:	pwr_write,
	release:	pwr_release,
};

static int pwr_read_proc(char *page, char **start, off_t off, int count,
			  int *eof, void *data_unused)
{
	char len;
	
	len = sprintf(page, "%d\n", pwr_status);

	*eof = 1; 
	return len;
}


static int __init pwrbtn_init(void)
{
	int ret;
		

	ret = register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc7901_fops);
		
	pwrbtn_enable_int();
	
	ret = request_irq(IRQ_PWRBTN, pwrbtn_handler, IRQF_DISABLED, //IRQF_SHARED, 
			  "pwrbtn", NULL);
	if(ret < 0) {
		printk("pwrbtn reuqest irq error\n");
        return -1;
    }
	
	// create PROC file
	pwr_entry = create_proc_entry(DEVICE_NAME, 0, NULL);
	if (pwr_entry)
	{
		pwr_entry->read_proc = pwr_read_proc;
		//pwr_entry->write_proc = pwr_write_proc;
	}
	
	printk("Power button initialized\n");
    return 0;
}

static void __exit pwrbtn_exit(void)
{	
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);
	free_irq(IRQ_PWRBTN, pwrbtn_handler);
	remove_proc_entry(DEVICE_NAME, NULL);
}

module_init(pwrbtn_init);
module_exit(pwrbtn_exit);

