/*
 * File:        drivers/char/tcc_bt_dev.c
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/ioctl.h>
#include <linux/device.h>

#include <bsp.h>
#include <asm/io.h>
#include <linux/delay.h>

#include <linux/tcc_bt_dev.h>
#include <mach/tcc_pca953x.h>

#include <linux/gpio.h>
#include <mach/gpio.h>

#ifndef ON
#define ON 		1
#endif

#ifndef OFF
#define OFF 	0
#endif

#define DEV_NAME "tcc_bt_dev"
static struct class *bt_dev_class;

typedef struct {
	int module;  // 0x12:CSR, 0x34:Broadcom
	int TMP1;
	int TMP2;
} tcc_bt_info_t;

static int tcc_bt_dev_open(struct inode *inode, struct file *file)
{
	printk("[## BT ##] tcc_bt_dev_open\n");
    return 0;
}

static int tcc_bt_dev_release(struct inode *inode, struct file *file)
{
	printk("[## BT ##] tcc_bt_dev_release\n");
    return 0;
}

int tcc_bt_power_control(int on_off)
{
	volatile PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
    
	printk("[## BT ##] tcc_bt_power_control input[%d]\n", on_off);
	
	if(on_off == BT_DEV_ON)
	{	    
		#if defined (CONFIG_MACH_TCC9200) || defined (CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)

			// CSR Only
			#if defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)
            // power on
            pGPIO->GPBFN3 &= 0xFFFFFF0F;
			pGPIO->GPBEN |= Hw25;
			pGPIO->GPBDAT |= Hw25;
			msleep(500);

            // reset
			#if defined(CONFIG_MACH_TCC8900)
			tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, BT_HWAKE, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
			msleep(500);
			tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, BT_HWAKE, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
			#else
			pGPIO->GPCFN3 &= 0xFF0FFFFF;
			pGPIO->GPCEN  |= Hw29;
			pGPIO->GPCDAT |= Hw29;

			msleep(100);
			pGPIO->GPCDAT &= ~Hw29;
			msleep(500);
			pGPIO->GPCDAT |= Hw29;
			#endif
			msleep(300);
			#endif

		#elif defined (CONFIG_MACH_TCC9200S)
			
			#if defined (CONFIG_TCC_BRCM_BCM4325D0_MODULE_SUPPORT)
			pGPIO->GPEEN |= Hw4;
			pGPIO->GPEDAT &= ~Hw4;
			msleep(100);
			pGPIO->GPDEN |= Hw11;
			pGPIO->GPDDAT &= ~Hw11;
			msleep(100);
			
			pGPIO->GPEEN |= Hw4;
			pGPIO->GPEDAT |= Hw4;
			msleep(100);
			pGPIO->GPDEN |= Hw11;
			pGPIO->GPDDAT |= Hw11;
			msleep(100);

			#elif defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)			
            // power on
			pGPIO->GPEFN0 &= 0xFFF0FFFF;
			pGPIO->GPEEN  |= Hw4;
			pGPIO->GPEDAT |= Hw4;
			msleep(500);

            // reset
			pGPIO->GPDFN1 &= 0xFFFF0FFF;
			pGPIO->GPDEN  |= Hw11;
			pGPIO->GPDDAT |= Hw11;

			msleep(100);
			pGPIO->GPDDAT &= ~Hw11;
			msleep(500);
			pGPIO->GPDDAT |= Hw11;
			msleep(300);
			#endif
		#elif defined (CONFIG_MACH_TCC9300)
			gpio_set_value(TCC_GPEXT1(7), 1);   /* BT-ON Enable */
			gpio_set_value(TCC_GPEXT3(2), 1);   /* BT_WAKE Enable , pwr_ctrl*/
			gpio_set_value(TCC_GPEXT3(3), 0);   /* BT_HWAKE Enable , /reset */
			msleep(500);
			gpio_set_value(TCC_GPEXT3(3), 1);   /* BT_HWAKE Enable , /reset */
		#elif defined (CONFIG_MACH_TCC9300ST)
			// power on
			pGPIO->GPFFN0	&= 0xFFFFFF00;	/* pwr_ctrl, reset*/
			pGPIO->GPFEN	|= Hw1|Hw0;
			pGPIO->GPFDAT	|= Hw1|Hw0;		/* BT-On Enbale, reset high*/
			msleep(100);
			pGPIO->GPFDAT	&= ~Hw1;		/* reset low */
			msleep(500);
			pGPIO->GPFDAT	|= Hw1;			/* reset high */
		#endif	
	}
	else if(on_off == BT_DEV_OFF)
	{
		#if defined (CONFIG_MACH_TCC9200) || defined (CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)  
		
			// CSR Only
			#if defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)		
            // power off
			pGPIO->GPBFN3 &= 0xFFFFFF0F;
			pGPIO->GPBEN  |= Hw25;
			pGPIO->GPBDAT &= ~Hw25;
			msleep(500);

            // reset
			#if defined(CONFIG_MACH_TCC8900)
			tcc_pca953x_setup(PCA9539_U2_SLAVE_ADDR, BT_HWAKE, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
			#else
			pGPIO->GPCFN3 &= 0xFF0FFFFF;
			pGPIO->GPCEN  |= Hw29;
			pGPIO->GPCDAT &= ~Hw29;
			#endif
			msleep(200);
			#endif

		#elif defined (CONFIG_MACH_TCC9200S)
		
			#if defined (CONFIG_TCC_BRCM_BCM4325D0_MODULE_SUPPORT)
			pGPIO->GPEEN |= Hw4;
			pGPIO->GPEDAT &= ~Hw4;
			msleep(100);
			pGPIO->GPDEN |= Hw11;
			pGPIO->GPDDAT &= ~Hw11;
			msleep(100);
			
		#elif defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)				
            // power off
			pGPIO->GPEFN0 &= 0xFFF0FFFF;
			pGPIO->GPEEN  |= Hw4;
			pGPIO->GPEDAT &= ~Hw4;
			
            // reset low
			pGPIO->GPDFN1 &= 0xFFFF0FFF;
			pGPIO->GPDEN  |= Hw11;
			//pGPIO->GPDDAT |= Hw11;
			//usleep(100000);
			pGPIO->GPDDAT &= ~Hw11;
			msleep(200);
			#endif
		#elif defined (CONFIG_MACH_TCC9300)
			gpio_set_value(TCC_GPEXT1(7), 0);   /* BT-ON Disable */
			gpio_set_value(TCC_GPEXT3(2), 0);   /* BT_WAKE Disable */
			gpio_set_value(TCC_GPEXT3(3), 0);   /* BT_HWAKE Disable */
		#elif defined (CONFIG_MACH_TCC9300ST)
			// power on
			pGPIO->GPFFN0	&= 0xFFFFFF00;	/* pwr_ctrl, reset*/
			pGPIO->GPFEN	|= Hw1|Hw0;
			pGPIO->GPFDAT	&= ~Hw0;		/* power off */
			pGPIO->GPFDAT	&= ~Hw1;		/* reset low */
			msleep(200);
		#endif
	}
	else
	{
		printk("[## BT_ERR ##] input_error On[%d] Off[%d]\n", BT_DEV_ON, BT_DEV_OFF);
	}

	return 0;
}


static int tcc_bt_get_info(tcc_bt_info_t* arg)
{
	tcc_bt_info_t *info_t;
	int module_t;
	
	info_t = (tcc_bt_info_t *)arg;
	copy_from_user(info_t, (tcc_bt_info_t *)arg, sizeof(tcc_bt_info_t));

	module_t = 0;

	#if defined (CONFIG_MACH_TCC9200) || defined (CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)	|| defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300ST)
		// CSR Only
		#if defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)		
        module_t = 0x12;
		#endif
	#elif defined (CONFIG_MACH_TCC9200S)	
		#if defined (CONFIG_TCC_BRCM_BCM4325D0_MODULE_SUPPORT)
		module_t = 0x34;
		#elif defined (CONFIG_TCC_CSR_BC0406_MODULE_SUPPORT)				
        module_t = 0x12;
		#endif		
	#endif
		
	printk("[## BT ##] module[0x%x]\n", module_t);

	info_t->module = module_t;

	copy_to_user((tcc_bt_info_t *)arg, info_t, sizeof(tcc_bt_info_t));

	return 0;
}


int tcc_bt_dev_ioctl(struct inode *inode, struct file *file,
                 unsigned int cmd, void *arg)
{
    int *parm1;
    
    memset(&parm1, 0, sizeof(int));
	printk("[## BT ##] tcc_bt_dev_ioctl cmd[%d] arg[%d]\n", cmd, arg);
	
    switch(cmd)
    {
	    case IOCTL_BT_DEV_POWER:			
    		parm1 = (int*)arg;
    		printk("[## BT ##] IOCTL_BT_DEV_POWER cmd[%d] parm1[%d]\n", cmd, *parm1);
			tcc_bt_power_control(*parm1);
	        // GPIO Control
	        break;
	        
		case IOCTL_BT_DEV_SPECIFIC:
			printk("[## BT ##] IOCTL_BT_DEV_SPECIFIC cmd[%d]\n", cmd);
			tcc_bt_get_info((tcc_bt_info_t*)arg);
			break;
        
	    default :
    		printk("[## BT ##] tcc_bt_dev_ioctl cmd[%d]\n", cmd);
	        break;
    }

    return 0;
}

struct file_operations tcc_bt_dev_ops = {
    .owner      = THIS_MODULE,
    .ioctl      = tcc_bt_dev_ioctl,
    .open       = tcc_bt_dev_open,
    .release    = tcc_bt_dev_release,
};


int init_module(void)
{
    int ret;

	printk("[## BT ##] init_module\n");
    ret = register_chrdev(BT_DEV_MAJOR_NUM, DEV_NAME, &tcc_bt_dev_ops);

	bt_dev_class = class_create(THIS_MODULE, DEV_NAME);
	device_create(bt_dev_class, NULL, MKDEV(BT_DEV_MAJOR_NUM, BT_DEV_MINOR_NUM), NULL, DEV_NAME);

    if(ret < 0){
        printk("[## BT ##] [%d]fail to register the character device\n", ret);
        return ret;
    }

    return 0;
}

void cleanup_module(void)
{
	printk("[## BT ##] cleanup_module\n");
    unregister_chrdev(BT_DEV_MAJOR_NUM, DEV_NAME);
}


module_init(init_module);
module_exit(cleanup_module);


MODULE_AUTHOR("Telechips Inc. linux@telechips.com");
MODULE_DESCRIPTION("TCC_BT_DEV");
MODULE_LICENSE("GPL");


