/*
 * drivers/char/Tcc83xx_scaler.c
 *
 * Copyright (C) 2004 Texas Instruments, Inc. 
 *
 * Video-for-Linux (Version 2) graphic capture driver for
 * the OMAP H2 and H3 camera controller.
 *
 * Adapted from omap24xx driver written by Andy Lowe (source@mvista.com)
 * Copyright (C) 2003-2004 MontaVista Software, Inc.
 * 
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 *
 * History:
 *   27/03/05   Vladimir Barinov - Added support for power management
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>

#include <mach/bsp.h>
#include <linux/poll.h>

#include "tcc_scaler.h"
#include <mach/tcc_scaler_ctrl.h>

#if 0
static int debug	   = 1;
#else
static int debug	   = 0;
#endif

volatile PM2MSCALER pM2MSCALER;

#define dprintk(msg...)	if (debug) { printk( "tcc_scaler: " msg); }

typedef struct _intr_data_t {
	//wait for Poll!!  
	wait_queue_head_t poll_wq;
	spinlock_t poll_lock;
	unsigned int poll_count;

	//wait for Ioctl command!!
	wait_queue_head_t cmd_wq;
	spinlock_t cmd_lock;
	unsigned int cmd_count;

	struct mutex io_mutex;
	unsigned char block_operating;
	unsigned char block_waiting;
	unsigned char irq_reged;
	unsigned int  dev_opened;
} intr_data_t;
static intr_data_t sc_data;

#define DEVICE_NAME			"scaler"
#define MAJOR_ID			200
#define MINOR_ID			1

static struct clk *m2m0_clk;
static struct clk *m2m0_ddi_cache;

static int tccxxx_scaler_mmap(struct file *file, struct vm_area_struct *vma)
{
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
	if(remap_pfn_range(vma,vma->vm_start, vma->vm_pgoff , vma->vm_end - vma->vm_start, vma->vm_page_prot))
	{
		return -EAGAIN;
	}

	vma->vm_ops		= NULL;
	vma->vm_flags 	|= VM_IO;
	vma->vm_flags 	|= VM_RESERVED;
	
	return 0;
}

char M2M_Scaler0_Ctrl_External(SCALER_TYPE *scale_img)
{
	unsigned int pSrcBase0 = 0, pSrcBase1 = 0, pSrcBase2 = 0;
	unsigned int pDstBase0 = 0, pDstBase1 = 0, pDstBase2 = 0;
	unsigned int SrcHsize, SrcVsize;
	unsigned int DstHsize, DstVsize;
	unsigned int SrcImgType, DstImgType;
	unsigned int ScaleRatV, ScaleRatH;
	unsigned int SrcOff_Hsize, DstOff_Hsize;
	int src_bpp, dst_bpp;
	int ret = 0;
	unsigned char interleaved_mode = 0; //0:Cr/Cb, 1:Cb/Cr
	unsigned char interleaved_src = 0;
	unsigned char interleaved_dst = 0;
	unsigned int crop_width;
	
	dprintk(" M2M_Scaler_Ctrl_Detail response[%d] \n", scale_img->responsetype);

	if(scale_img->src_fmt == SCALER_YUV422_inter || scale_img->src_fmt == SCALER_YUV420_inter)
		interleaved_src = 1;

	if(scale_img->dest_fmt == SCALER_YUV422_inter || scale_img->dest_fmt == SCALER_YUV420_inter)
		interleaved_dst = 1;
	
	if(!scaler_image_converter(scale_img->src_fmt, &SrcImgType, &src_bpp))
	{
		dprintk("source format error \n");
		return -1;
	}
	if(!scaler_image_converter(scale_img->dest_fmt, &DstImgType, &dst_bpp))
	{
		 dprintk("destination format error \n");		
		 return -1;
	}

	pSrcBase0 = (unsigned int)scale_img->src_Yaddr;
	pSrcBase1 = (unsigned int)scale_img->src_Uaddr;
	pSrcBase2 = (unsigned int)scale_img->src_Vaddr;

	// address limitation!!
	{
		crop_width	 = scale_img->src_winRight - scale_img->src_winLeft;
		scale_img->src_winLeft	= (scale_img->src_winLeft>>3)<<3; 
		scale_img->src_winRight = scale_img->src_winLeft + crop_width;
	}
	scale_get_address(scale_img->src_fmt, (unsigned int)scale_img->src_Yaddr, 
						scale_img->src_ImgWidth, scale_img->src_ImgHeight, 
						scale_img->src_winLeft, scale_img->src_winTop,
						&pSrcBase0, &pSrcBase1, &pSrcBase2);
	
	
	pDstBase0 = (unsigned int)scale_img->dest_Yaddr;
	pDstBase1 = (unsigned int)scale_img->dest_Uaddr;
	pDstBase2 = (unsigned int)scale_img->dest_Vaddr;

	// address limitation!!
	{
		crop_width	 = scale_img->dest_winRight - scale_img->dest_winLeft;
		scale_img->dest_winLeft	= (scale_img->dest_winLeft>>3)<<3; 
		scale_img->dest_winRight = scale_img->dest_winLeft + crop_width;
	}
	scale_get_address(scale_img->dest_fmt, (unsigned int)scale_img->dest_Yaddr, 
						scale_img->dest_ImgWidth, scale_img->dest_ImgHeight, 
						scale_img->dest_winLeft, scale_img->dest_winTop,
						&pDstBase0, &pDstBase1, &pDstBase2);
	
	// m2m scaler setting
	SrcOff_Hsize = scale_img->src_ImgWidth;
	DstOff_Hsize = scale_img->dest_ImgWidth;
	SrcHsize = scale_img->src_winRight - scale_img->src_winLeft;
	SrcVsize = scale_img->src_winBottom - scale_img->src_winTop;
	DstHsize = scale_img->dest_winRight - scale_img->dest_winLeft;
	DstVsize = scale_img->dest_winBottom - scale_img->dest_winTop;
	
	ScaleRatH = (SrcHsize * 256)/DstHsize;
	ScaleRatV = (SrcVsize * 256)/DstVsize;

	dprintk("src fmt[%x] width[%d] height[%d] \n", SrcImgType, SrcHsize, SrcVsize);
	dprintk("src Address [0x%x] [0x%x] [0x%x] \n", pSrcBase0, pSrcBase1, pSrcBase2);
	
	dprintk("dest fmt[%x] width[%d] height[%d] \n", DstImgType, DstHsize, DstVsize);
	dprintk("dest Address [0x%x] [0x%x] [0x%x] \n", pDstBase0, pDstBase1, pDstBase2);


	M2M_Scaler_SW_Reset(M2M_SCALER0);

	if(scale_img->responsetype == SCALER_INTERRUPT)
		MEMSCL_SET_INTERRUPT(M2M_SCALER0, SET_M2MSCL_INT_ENABLE);
	else
		MEMSCL_SET_INTERRUPT(M2M_SCALER0, SET_M2MSCL_INT_DISABLE);

	MEM_SCALER_CTRL(MSC_CTRL_RDY_INT_EN);
					
	MEM_SCALER_SRC_ADDR(pSrcBase0, pSrcBase1, pSrcBase2);
	
	MEM_SCALER_SRC_SIZE(SrcHsize, SrcVsize);
	MEM_SCALER_SRC_CONFIG(MSC_CFG_ALL, interleaved_mode, interleaved_src, SrcImgType);
	
	if ((SrcImgType == HwMSC_SRC_CFG_YUV422_SEP) || (SrcImgType == HwMSC_SRC_CFG_YUV420_SEP))
	{
		if(interleaved_src)
			MEM_SCALER_SRC_OFFSET_YC(SrcOff_Hsize, SrcOff_Hsize);
		else
			MEM_SCALER_SRC_OFFSET_YC(SrcOff_Hsize, (SrcOff_Hsize/2));
	}
	else
	{
		MEM_SCALER_SRC_OFFSET_YC((SrcOff_Hsize*2), 0);
	}

	if(DstImgType >= HwMSC_SRC_CFG_RGB565)
		pDstBase1 = pDstBase2 = 0;
	
	MEM_SCALER_DST_ADDR(pDstBase0, pDstBase1, pDstBase2);
	MEM_SCALER_RATIO(ScaleRatH, ScaleRatV);
	MEM_SCALER_DST_SIZE(DstHsize, DstVsize);

	if ((DstImgType == HwMSC_SRC_CFG_YUV422_SEP) || (DstImgType == HwMSC_SRC_CFG_YUV420_SEP))
	{
		if(interleaved_dst)
			MEM_SCALER_DST_OFFSET_YC(DstOff_Hsize, DstOff_Hsize);
		else
		MEM_SCALER_DST_OFFSET_YC(DstOff_Hsize, (DstOff_Hsize/2));
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, 1, 1, interleaved_mode, 0, interleaved_dst, DstImgType);
	}
	else
	{
		MEM_SCALER_DST_OFFSET_YC((DstOff_Hsize*2), 0);
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, 1, 1, 0, 0, 0, DstImgType);
	}

	MEM_SCALER_CTRL(MSC_CTRL_EN);
	MEM_SCALER_CTRL(MSC_CTRL_DIS);
	MEM_SCALER_CTRL(MSC_CTRL_RDY_INT_EN);

	if(scale_img->responsetype == SCALER_POLLING)
	{
		ret = MEM_SCALER_POLLING_CHECK_External();
		M2M_Scaler_SW_Reset(M2M_SCALER1);
 	}

	return ret;
}

extern unsigned int scaler_ended;
char M2M_Scaler_Ctrl_Detail(SCALER_TYPE *scale_img)
{
	unsigned int pSrcBase0 = 0, pSrcBase1 = 0, pSrcBase2 = 0;
	unsigned int pDstBase0 = 0, pDstBase1 = 0, pDstBase2 = 0;
	unsigned int SrcHsize, SrcVsize;
	unsigned int DstHsize, DstVsize;
	unsigned int SrcImgType, DstImgType;
	unsigned int ScaleRatV, ScaleRatH;
	unsigned int SrcOff_Hsize, DstOff_Hsize;
	int src_bpp, dst_bpp;
	int ret = 0;
	unsigned char interleaved_mode = 0; //0:Cr/Cb, 1:Cb/Cr
	unsigned char interleaved_src = 0;
	unsigned char interleaved_dst = 0;
	unsigned int crop_width;
	
	// Ҷ reset ϵ   ..pjj
	M2M_Scaler_SW_Reset(M2M_SCALER0);

	dprintk(" M2M_Scaler_Ctrl_Detail response[%d] \n", scale_img->responsetype);

	if(scale_img->src_fmt == SCALER_YUV422_inter || scale_img->src_fmt == SCALER_YUV420_inter)
		interleaved_src = 1;

	if(scale_img->dest_fmt == SCALER_YUV422_inter || scale_img->dest_fmt == SCALER_YUV420_inter)
		interleaved_dst = 1;
	
	if(!scaler_image_converter(scale_img->src_fmt, &SrcImgType, &src_bpp))
	{
		dprintk("source format error \n");
		return -1;
	}
	if(!scaler_image_converter(scale_img->dest_fmt, &DstImgType, &dst_bpp))
	{
		 dprintk("destination format error \n");		
		 return -1;
	}

	pSrcBase0 = (unsigned int)scale_img->src_Yaddr;
	pSrcBase1 = (unsigned int)scale_img->src_Uaddr;
	pSrcBase2 = (unsigned int)scale_img->src_Vaddr;

	// address limitation!!
	{
		crop_width	 = scale_img->src_winRight - scale_img->src_winLeft;		
		scale_img->src_winLeft	= (scale_img->src_winLeft>>3)<<3; 
		scale_img->src_winRight = scale_img->src_winLeft + crop_width;
	}
	scale_get_address(scale_img->src_fmt, (unsigned int)scale_img->src_Yaddr, 
						scale_img->src_ImgWidth, scale_img->src_ImgHeight, 
						scale_img->src_winLeft, scale_img->src_winTop,
						&pSrcBase0, &pSrcBase1, &pSrcBase2);
	
	
	pDstBase0 = (unsigned int)scale_img->dest_Yaddr;
	pDstBase1 = (unsigned int)scale_img->dest_Uaddr;
	pDstBase2 = (unsigned int)scale_img->dest_Vaddr;

	// address limitation!!
	{
		crop_width	 = scale_img->dest_winRight - scale_img->dest_winLeft;		
		scale_img->dest_winLeft	= (scale_img->dest_winLeft>>3)<<3; 
		scale_img->dest_winRight = scale_img->dest_winLeft + crop_width;
	}
	scale_get_address(scale_img->dest_fmt, (unsigned int)scale_img->dest_Yaddr, 
						scale_img->dest_ImgWidth, scale_img->dest_ImgHeight, 
						scale_img->dest_winLeft, scale_img->dest_winTop,
						&pDstBase0, &pDstBase1, &pDstBase2);
	
	// m2m scaler setting
	SrcOff_Hsize = scale_img->src_ImgWidth;
	DstOff_Hsize = scale_img->dest_ImgWidth;
	SrcHsize = scale_img->src_winRight - scale_img->src_winLeft;
	SrcVsize = scale_img->src_winBottom - scale_img->src_winTop;
	DstHsize = scale_img->dest_winRight - scale_img->dest_winLeft;
	DstVsize = scale_img->dest_winBottom - scale_img->dest_winTop;
	
	ScaleRatH = (SrcHsize * 256)/DstHsize;
	ScaleRatV = (SrcVsize * 256)/DstVsize;

	dprintk("src fmt[%x] width[%d] height[%d] \n", SrcImgType, SrcHsize, SrcVsize);
	dprintk("src Address [0x%x] [0x%x] [0x%x] \n", pSrcBase0, pSrcBase1, pSrcBase2);
	
	dprintk("dest fmt[%x] width[%d] height[%d] \n", DstImgType, DstHsize, DstVsize);
	dprintk("dest Address [0x%x] [0x%x] [0x%x] \n", pDstBase0, pDstBase1, pDstBase2);
					
	MEM_SCALER_SRC_ADDR(pSrcBase0, pSrcBase1, pSrcBase2);
	
	MEM_SCALER_SRC_SIZE(SrcHsize, SrcVsize);
	MEM_SCALER_SRC_CONFIG(MSC_CFG_ALL, interleaved_mode, interleaved_src, SrcImgType);
	
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	if ((SrcImgType == HwMSC_SRC_CFG_YUV422_SEP) || (SrcImgType == HwMSC_SRC_CFG_YUV420_SEP))
#else
	if ((SrcImgType == HwSRCCFG_422SEP) || (SrcImgType == HwSRCCFG_420SEP))
#endif
	{
		if(interleaved_src)
			MEM_SCALER_SRC_OFFSET_YC(SrcOff_Hsize, SrcOff_Hsize);
		else
			MEM_SCALER_SRC_OFFSET_YC(SrcOff_Hsize, (SrcOff_Hsize/2));
	}
	else
	{
		MEM_SCALER_SRC_OFFSET_YC((SrcOff_Hsize*2), 0);
	}

	//interlaced frame process ex) MPEG2
	if(scale_img->interlaced)
	{
		unsigned int validSrcVSize = (( SrcVsize/2) >>4) <<4 ;
		
		MEM_SCALER_SRC_SIZE(SrcHsize,validSrcVSize);
		
		if(interleaved_src)
			MEM_SCALER_SRC_OFFSET_YC((unsigned int)SrcOff_Hsize*2, SrcOff_Hsize*2);
		else
			MEM_SCALER_SRC_OFFSET_YC((unsigned int)SrcOff_Hsize*2, SrcOff_Hsize);
		
		ScaleRatV = (validSrcVSize * 256)/DstVsize;
	}
	
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	if(DstImgType >= HwMSC_SRC_CFG_RGB565)
#else
	if(DstImgType >= HwSRCCFG_565)
#endif
		pDstBase1 = pDstBase2 = 0;
	
	MEM_SCALER_DST_ADDR(pDstBase0, pDstBase1, pDstBase2);
	MEM_SCALER_RATIO(ScaleRatH, ScaleRatV);
	MEM_SCALER_DST_SIZE(DstHsize, DstVsize);

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	if ((DstImgType == HwMSC_SRC_CFG_YUV422_SEP) || (DstImgType == HwMSC_SRC_CFG_YUV420_SEP))
#else
	if ((DstImgType == HwSRCCFG_422SEP) || (DstImgType == HwSRCCFG_420SEP))
#endif
	{
		if(interleaved_dst)
			MEM_SCALER_DST_OFFSET_YC(DstOff_Hsize, DstOff_Hsize);
		else
		MEM_SCALER_DST_OFFSET_YC(DstOff_Hsize, (DstOff_Hsize/2));
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, 0, 0, interleaved_mode, 0, interleaved_dst, DstImgType);
#else
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 1, 1, 0, DstImgType);
#endif
	}
	else
	{
		MEM_SCALER_DST_OFFSET_YC((DstOff_Hsize*2), 0);
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, 1, 1, 0, 0, 0, DstImgType);
#else
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 1, 1, 0, DstImgType);
#endif
	}

//	printk("\n%8x, %8x, %8x, %8x \n",pM2MSCALER->SRCBASEY, pM2MSCALER->SRCBASEU, pM2MSCALER->SRCBASEV, pM2MSCALER->SRCSIZE);
//	printk("%8x, %8x, 00000000, 00000000 \n",pM2MSCALER->SRCOFF, pM2MSCALER->SRCCFG);
//	printk("%8x, %8x, %8x, %8x \n",pM2MSCALER->DSTBASEY, pM2MSCALER->DSTBASEU, pM2MSCALER->DSTBASEV, pM2MSCALER->DSTSIZE);
//	printk("%8x, %8x, 00000000, 00000000 \n\n",pM2MSCALER->DSTOFF, pM2MSCALER->DSTCFG);
	
	scaler_ended = 0;
	BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x80000000);
	BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x00000000);
//	BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x00000003);
	
	if(scale_img->viqe_onthefly)
	{
		BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x01000003);
	}
	else
	{
		BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x00000003);
	}

	if(scale_img->responsetype  == SCALER_POLLING)
	{
		ret = MEM_SCALER_POLLING_CHECK();
	}

	return ret;
}

static unsigned int tccxxx_internal_wq_release(intr_data_t *msc_data, unsigned char timeout)
{
	if(!timeout)
	{
		spin_lock_irq(&(msc_data->poll_lock));
		msc_data->poll_count--;
		spin_unlock_irq(&(msc_data->poll_lock));
	}

/////////////////////
	if(msc_data->block_waiting)
	{
		spin_lock_irq(&(msc_data->cmd_lock));
		msc_data->cmd_count++;
		spin_unlock_irq(&(msc_data->cmd_lock));

		msc_data->block_waiting = 0;
		wake_up_interruptible(&(msc_data->cmd_wq));
	}
//	printk("block_operating free \n");
	msc_data->block_operating = 0;
/////////////////////

	if(!timeout)
	{
		if(scaler_ended == 1)
			return (POLLIN | POLLRDNORM);
		else
			return POLLERR;
	}
	else
		return 0;

}

static unsigned int tccxxx_scaler_poll(struct file *filp, poll_table *wait)
{
	intr_data_t *msc_data = (intr_data_t *)filp->private_data;

	if (msc_data == NULL)
		return -EFAULT;

	if (msc_data->poll_count > 0) 
	{
		return tccxxx_internal_wq_release(msc_data, 0);
	}
	
	poll_wait(filp, &(msc_data->poll_wq), wait);

	if (msc_data->poll_count > 0) 
	{	
		return tccxxx_internal_wq_release(msc_data, 0);
	} 
	else 
	{
		return 0;
	}
}


// interrupt handler
static irqreturn_t tccxxx_scaler_handler(int irq, void *client_data/*, struct pt_regs *reg*/)
{  	
	unsigned long IFlag;

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	intr_data_t *msc_data = client_data;

	spin_lock_irq(&(msc_data->poll_lock));
	msc_data->poll_count++;
	spin_unlock_irq(&(msc_data->poll_lock));
	
	IFlag = pM2MSCALER->MSCSTR;

	if (IFlag & HwMSC_STATUS_IRDY)
	{
		scaler_ended = 1;
	}
	else if (IFlag & HwMSC_STATUS_IBUSY)
	{
		scaler_ended = 2;
		printk("interrupt:: scaler busy!!!\n");
	}
	else
	{
		printk("unknown interrupt:: 0x%x!!!\n", IFlag);
	}

	BITCSET(pM2MSCALER->MSCCTR, 0xffffffff, 0x00000000);
	pM2MSCALER->MSCSTR = IFlag; // clear interrupt flag
	
	wake_up_interruptible(&(msc_data->poll_wq));
#else // CONFIG_ARCH_TCC92X
	sHwINT	*pHwINT	= &HwIEN;

	dprintk("handler ISR \n");

	IFlag = HwMSCSTR;
	if (IFlag & HwMSCSTR_IRDY)
	{
		dprintk("handler finish \n");
		//scaler disable
		MEMSCL_SET_INTERRUPT(SET_M2MSCL_INT_DISABLE);
		M2M_Scaler_SW_Reset();
		IO_CKC_DisableBUS_MSCL();

		wake_up_interruptible(&scaler_wait);
	}
	HwMSCSTR = IFlag;	
	
	pHwINT->ICLR = HwINT_SC;	
#endif

	return IRQ_HANDLED;
}
		
static int tccxxx_scaler_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int ret;
	SCALER_TYPE scaler_v;
	intr_data_t *msc_data = (intr_data_t *)file->private_data;
	
	switch(cmd)
	{
		case TCC_SCALER_IOCTRL:			
			mutex_lock(&msc_data->io_mutex);
			
			if(msc_data->block_operating)
			{
				msc_data->block_waiting = 1;
				ret = wait_event_interruptible_timeout(msc_data->cmd_wq, msc_data->cmd_count > 0, msecs_to_jiffies(200));
				spin_lock_irq(&(msc_data->cmd_lock));
				msc_data->cmd_count = 0;
				spin_unlock_irq(&(msc_data->cmd_lock));
				
				if(!ret)
				{
					msc_data->block_waiting = 0;
					printk("[%d]: scaler timed_out!! \n", ret);
				}
			}

			msc_data->block_operating = 1;	
			mutex_unlock(&msc_data->io_mutex);

			if(copy_from_user(&scaler_v,(SCALER_TYPE*)arg,sizeof(scaler_v)))
			{
				msc_data->block_operating = 0;
				return -EFAULT;
			}
			
			spin_lock_irq(&(msc_data->poll_lock));
			msc_data->poll_count = 0;
			spin_unlock_irq(&(msc_data->poll_lock));

			ret = M2M_Scaler_Ctrl_Detail(&scaler_v);
			
			return ret;

		default:
			printk(KERN_ALERT "Not Supported SCALER_IOCTL(%d)\n", cmd);
			break;			
	}

	return 0;
}

static int tccxxx_scaler_release(struct inode *inode, struct file *filp)
{
	dprintk("scaler_release In!! %d'th, block(%d/%d), cmd(%d), irq(%d)  \n", sc_data.dev_opened, sc_data.block_operating, sc_data.block_waiting, sc_data.cmd_count, sc_data.irq_reged);

	if(sc_data.dev_opened > 0)
		sc_data.dev_opened--;

	if(sc_data.dev_opened == 0)
	{
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
		if(sc_data.irq_reged)
		{
			free_irq(INT_SC0, &sc_data);	
			sc_data.irq_reged = 0;
		}

		M2M_Scaler_SW_Reset(M2M_SCALER0);
#else
		free_irq(IRQ_SC, NULL);
#endif
		sc_data.block_operating = sc_data.block_waiting = 0;
		sc_data.poll_count = sc_data.cmd_count = 0;
	}
	clk_disable(m2m0_clk);
	clk_disable(m2m0_ddi_cache);

	dprintk("scaler_release Out!! %d'th \n", sc_data.dev_opened);

	return 0;
}

static int tccxxx_scaler_open(struct inode *inode, struct file *filp)
{	
	int ret = 0;

	dprintk("scaler_open In!! %d'th, block(%d/%d), cmd(%d), irq(%d) \n", sc_data.dev_opened, sc_data.block_operating, sc_data.block_waiting, sc_data.cmd_count, sc_data.irq_reged);

	clk_enable(m2m0_clk);
	clk_enable(m2m0_ddi_cache);

	if(!sc_data.irq_reged)
	{
		M2M_Scaler_SW_Reset(M2M_SCALER0);
		MEMSCL_SET_INTERRUPT_INIT(M2M_SCALER0);
		MEMSCL_SET_INTERRUPT(M2M_SCALER0, SET_M2MSCL_INT_ENABLE);
	
		ret = request_irq(INT_SC0, tccxxx_scaler_handler, IRQF_DISABLED, "scaler", &sc_data);
		if (ret) {
			clk_disable(m2m0_clk);
			clk_disable(m2m0_ddi_cache);
			printk("FAILED to aquire scaler0-irq\n");
			return -EFAULT;
		}
		sc_data.irq_reged = 1;
	}

	sc_data.dev_opened++;
	filp->private_data = &sc_data;
	
	dprintk("scaler_open Out!! %d'th \n", sc_data.dev_opened);
	
	return ret;	
}


static struct file_operations tcc_scaler_fops = 
{
	.owner			= THIS_MODULE,
	.ioctl				= tccxxx_scaler_ioctl,
	.mmap			= tccxxx_scaler_mmap,
	.open			= tccxxx_scaler_open,
	.release			= tccxxx_scaler_release,
	.poll				= tccxxx_scaler_poll,
};

void __exit
tccxxx_scaler_cleanup(void)
{
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);
	clk_put(m2m0_clk);

	return;
}

static char banner[] __initdata = KERN_INFO "TCC scaler driver initializing\n";

static struct class *scaler_class;

int __init 
tccxxx_scaler_init(void)
{
	printk(banner);
	
	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_scaler_fops);
	scaler_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(scaler_class, NULL, MKDEV(MAJOR_ID, MINOR_ID), NULL, DEVICE_NAME);

	memset(&sc_data, 0, sizeof(intr_data_t));
	spin_lock_init(&(sc_data.poll_lock));
	init_waitqueue_head(&(sc_data.poll_wq));
	spin_lock_init(&(sc_data.cmd_lock));
	init_waitqueue_head(&(sc_data.cmd_wq));

	mutex_init(&sc_data.io_mutex);

	m2m0_clk = clk_get(NULL, "m2m0");
	BUG_ON(m2m0_clk == NULL);

	m2m0_ddi_cache = clk_get(NULL, "ddi_cache");
	BUG_ON(m2m0_ddi_cache == NULL);

	pM2MSCALER = (volatile PM2MSCALER)tcc_p2v(HwM2MSCALER0_BASE);

	return 0;
}


MODULE_AUTHOR("Telechips.");
MODULE_DESCRIPTION("TCC scaler driver");
MODULE_LICENSE("GPL");


module_init(tccxxx_scaler_init);
module_exit(tccxxx_scaler_cleanup);


