/*
 * drivers/char/tcc_tdmbtuner.c
 *
 * Copyright (C) 2009 Telechips, Inc.
 *
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>

#include <mach/bsp.h>

#define DEVICE_NAME			"tdmb_tuner"
#define MAJOR_ID			197
#define MINOR_ID			1

static void init_device(void)
{
	volatile GPIO *RGPIO = (volatile GPIO *) &HwGPIO_BASE;

	/*
         * Initialize GPIO ports
         */
	// External interrupt
	BITCLR(RGPIO->GPAFN1, (Hw16 - Hw12)); // use to GPIO
	BITCLR(RGPIO->GPAEN, Hw11);

#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) 
	// PD
        BITCLR(RGPIO->GPEFN0, (Hw16 - Hw12)); // use to GPIO
        BITSET(RGPIO->GPEEN, Hw3);
        BITCLR(RGPIO->GPEDAT, Hw3);

	// RST
        BITCLR(RGPIO->GPEFN1, (Hw16 - Hw12)); // use to GPIO
        BITSET(RGPIO->GPEEN, Hw11);
        BITCLR(RGPIO->GPEDAT, Hw11);

	// Power on
	BITSET(RGPIO->GPEDAT, Hw3);

	// Reset
        BITCLR(RGPIO->GPEDAT, Hw11);
        msleep(20);
        BITSET(RGPIO->GPEDAT, Hw11);
        
#elif defined(CONFIG_MACH_TCC9200S)
	// PD
        BITCLR(RGPIO->GPEFN0, HwPORTCFG_GPFN5(0));
        BITSET(RGPIO->GPEEN, Hw5);
        BITCLR(RGPIO->GPEDAT, Hw5);

	// RST
        BITCLR(RGPIO->GPDFN1, Hw4 - Hw0);
        BITSET(RGPIO->GPDEN, Hw8);
        BITCLR(RGPIO->GPDDAT, Hw8);

	/* Power on */
	BITSET(RGPIO->GPESET, Hw5);

        /* Reset */
	BITCLR(RGPIO->GPDDAT, Hw8);
	msleep(20);
	BITSET(RGPIO->GPDDAT, Hw8);
#endif
}

static void deinit_device(void)
{
	volatile GPIO *RGPIO = (volatile GPIO *) &HwGPIO_BASE;

#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
	BITCLR(RGPIO->GPEDAT, Hw3);
#elif defined(CONFIG_MACH_TCC9200S)
	BITSET(RGPIO->GPECLR, Hw5);
#endif
}

static int tcc_tdmbtuner_release(struct inode *inode, struct file *filp)
{
	deinit_device();
	return 0;
}

static int tcc_tdmbtuner_open(struct inode *inode, struct file *filp)
{	
	init_device();	
	return 0;
}


static struct file_operations tcc_tdmbtuner_fops = 
{
	.owner			= THIS_MODULE,
	.open			= tcc_tdmbtuner_open,
	.release		= tcc_tdmbtuner_release,
};

void __exit
tcc_tdmbtuner_cleanup(void)
{
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);
	
	return;
}

static char banner[] __initdata = KERN_INFO "TCC T-DMB tuner driver initializing\n";

static struct class *tdmbtuner_class;

int __init 
tcc_tdmbtuner_init(void)
{
	printk(banner);
	
	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_tdmbtuner_fops);
	tdmbtuner_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(tdmbtuner_class, NULL, MKDEV(MAJOR_ID, MINOR_ID), NULL, DEVICE_NAME);

	return 0;
}

MODULE_AUTHOR("Telechips.");
MODULE_DESCRIPTION("TCC T-DMB tuner driver");
MODULE_LICENSE("GPL");

module_init(tcc_tdmbtuner_init);
module_exit(tcc_tdmbtuner_cleanup);

