
/****************************************************************************
 *   FileName    : tca_keypad.c
 *   Description : 
 ****************************************************************************
*
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
*
 ****************************************************************************/
#include "tca_keypad.h"

typedef struct {
	unsigned int	StartVal;
	unsigned int	EndVal;
	unsigned char	KeyCode;
}SCANCODE;

#ifdef __KERNEL__
const SCANCODE	stScanCodeMapping[] =
{
#if defined(_TCC79x_)

	{0x14A, 0x168, KEY_ESC},
	{0x1A4, 0x1C2, KEY_1 },
	{0x0  , 0x28 , KEY_2 },
	{0x50 , 0x6E , KEY_3 },
	{0x10E, 0x12C, KEY_4 },
	{0x3A2, 0x3B6, KEY_5 },
	{0xE1 , 0xF5 , KEY_6 },
	{0x32F, 0x343, KEY_7 },
	{0x2B2, 0x2C6, KEY_8 },
	{0x91 , 0xA5 , KEY_9 },
	{0x235, 0x249, KEY_0 },

#elif defined(_TCC83x_)	
    { 0x154, 0x15c, KEY_ESC },  
    { 0x1B2, 0x1BA, KEY_1 },  
    { 0x16 , 0x1E , KEY_2 }, 
    { 0x57 , 0x5F , KEY_3 },
    { 0x11B, 0x123, KEY_4 },
    { 0x3AC, 0x3B4, KEY_5 },
    { 0xE7 , 0xEF , KEY_6 },
    { 0x33B, 0x343, KEY_7 },
    { 0x2BD, 0x2C5, KEY_8 },
    { 0x97 , 0x9F , KEY_9 },
    { 0x23E, 0x246, KEY_0 },
#elif defined(CONFIG_ARCH_TCC92X)
#if 0
	// Grater than 4050 -> IDLE
	// {ADC MinValue, MaxValue, Virtual Key Value}
	{1370, 1410, KEY_ESC},	// VK_LWIN
	{1730, 1780, KEY_1},	// VK_MENU
	{90  , 130 , KEY_2},	// VK_SPACE
	{350 , 390 , KEY_3},	// VK_UP
	{1120, 1170, KEY_4},	// VK_TAB
	{3730, 4100, KEY_5},	// VK_ESCAPE
	{920 , 970 , KEY_6},	// VK_RBUTTON
	{3280, 3340, KEY_7},	// VK_LEFT
	{2780, 2840, KEY_8},	// VK_RIGHT
	{600 , 660 , KEY_9},	// VK_DOWN
	{2280, 2350, KEY_0},	// VK_RETURN
#endif
#if 1
	{0x23F, 0x248, KEY_BACK},	// VK_LWIN
	{0x11E, 0x121, KEY_7},		// VK_MENU
	{0x9A , 0x9F , KEY_DOWN},	// VK_SPACE
	{0x5A , 0x5F , KEY_UP},		// VK_UP
	{0x2BA, 0x2C1, KEY_RIGHT},	// VK_TAB
	{0x8  , 0x1E , KEY_ENTER},	// VK_ESCAPE
	{0x338, 0x340, KEY_LEFT},	// VK_RBUTTON
	{0x1B4, 0x1B9, KEY_MENU},	// VK_LEFT
	{0xEA , 0xEF , KEY_8},		// VK_RIGHT
	{0x158, 0x15C, KEY_HOME},	// VK_DOWN
	{0x384, 0x3CB, KEY_CAMERA},	// VK_RETURN
#else
	{575, 584, KEY_0},		// VK_LWIN
	{286, 289, KEY_7},		// VK_MENU
	{154, 159, KEY_9},		// VK_SPACE
	{90 , 95 , KEY_3},		// VK_UP
	{698, 705, KEY_4},		// VK_TAB
	{8  , 30 , KEY_5},		// VK_ESCAPE
	{824, 832, KEY_6},		// VK_RBUTTON
	{436, 441, KEY_1},		// VK_LEFT
	{234, 239, KEY_8},		// VK_RIGHT
	{344, 348, KEY_2},		// VK_DOWN
	{900, 971, KEY_ESC},	// VK_RETURN
#endif
#elif defined(CONFIG_MACH_TCC9300CM)
  {0xE00 , 0xF18 , KEY_MENU},
	{0xA00, 0xB40, KEY_ENTER},
	{0x600 , 0x720 , KEY_LEFT},
	{0x400, 0x520, KEY_RIGHT},
	{0x200 , 0x320 , KEY_HOME},
	{0x10 , 0x120 , KEY_BACK},
#endif

};	
#else
const SCANCODE	stScanCodeMapping[] =
{
	// 0x3E8, greater than 1000 -> IDLE
#if defined(_TCC79x_)
	{0x14A, 0x168, 0x5B},	//VK_LWIN },		// A1
	{0x1A4, 0x1C2, 0x12},	//VK_MENU },		// A2
	{0x0  , 0x28 , 0x20},	//VK_SPACE },		// A3
	{0x50 , 0x6E , 0x26},	//VK_UP },			// A4
	{0x10E, 0x12C, 0x09},	//VK_TAB },			// A5
	{0x3A2, 0x3B6, 0x1B},	//VK_ESCAPE },		// B0
	{0xE1 , 0xF5 , 0x02},	//VK_RBUTTON },		// B1
	{0x32F, 0x343, 0x25},	//VK_LEFT },		// B2
	{0x2B2, 0x2C6, 0x27},	//VK_RIGHT },		// B3
	{0x91 , 0xA5 , 0x28},	//VK_DOWN },		// B4
	{0x235, 0x249, 0x0D},	//VK_RETURN }		// B5

#elif defined(_TCC83x_)	
	{0x50 , 0x6E , 0x12},	//VK_MENU },		// A1
	{0x1A4, 0x1C2, 0x1B},	//VK_ESCAPE },		// A2
	{0x14A, 0x168, 0x09},	//VK_TAB},			// B0
	{0x91 , 0xA5 , 0x20},	//VK_SPACE },		// B1
	{0x10E, 0x12C, 0x0D},	//VK_RETURN },		// B2
	{0x3A2, 0x3B6, 0x5B},	//VK_LWIN },		// C0
	{0x0  , 0x28 , 0x26},	//VK_UP },			// C1
	{0x235, 0x249, 0x02},	//VK_RBUTTON }		// C2
	{0x32F, 0x343, 0x25},	//VK_LEFT },		// D0
	{0xE1 , 0xF5 , 0x28},	//VK_DOWN },		// D1
	{0x2B2, 0x2C6, 0x27},	//VK_RIGHT },		// D2
	
#elif defined(CONFIG_ARCH_TCC92X)
	{0x14A, 0x168, 0x5B},	//VK_LWIN },		// A1
	{0x1A4, 0x1C2, 0x12},	//VK_MENU },		// A2
	{0x0  , 0x28 , 0x20},	//VK_SPACE },		// A3
	{0x50 , 0x6E , 0x26},	//VK_UP },			// A4
	{0x10E, 0x12C, 0x09},	//VK_TAB }, 		// A5
	{0x3A2, 0x3B6, 0x1B},	//VK_ESCAPE },		// B0
	{0xE1 , 0xF5 , 0x02},	//VK_RBUTTON }, 	// B1
	{0x32F, 0x343, 0x25},	//VK_LEFT },		// B2
	{0x2B2, 0x2C6, 0x27},	//VK_RIGHT },		// B3
	{0x91 , 0xA5 , 0x28},	//VK_DOWN },		// B4
	{0x235, 0x249, 0x0D},	//VK_RETURN }		// B5
#endif	
};	
#endif

/*****************************************************************************
* Function Name : tca_keypad_getscancode(void)
******************************************************************************
* Desription  	: 
* Parameter   	: 
* Return      	: 
* Note          : 
******************************************************************************/
unsigned int tca_keypad_getscancode(void)
{
	unsigned int	adcdata;


	int adc_ch = 0;	// Keypad-> ADC0
#if 1
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)

    volatile PTSADC pTSADC = (volatile PTSADC)tcc_p2v(HwTSADC_BASE);

	BITSET(pTSADC->ADCCON,Hw2| adc_ch<<3);
	BITCLR(pTSADC->ADCCON,Hw2);		
	BITSET(pTSADC->ADCCON,Hw0);			

	do{
		adcdata = pTSADC->ADCDAT0 & 0x3ff;
	}while(!(pTSADC->ADCCON & Hw15));

	BITSET(pTSADC->ADCCON,Hw2);
	return (adcdata);	
#else
	BITSET(HwADCCON, Hw4|adc_ch);

	do{
		BITCLR(HwADCCON, Hw4);
	#if defined(_TCC80x_)
		adcdata = HwADCDATA;
	#else
	
		adcdata = HwADDATA;
	#endif
	
	}while(!(adcdata & 1));
	// OS_TimeDelay();
	BITCSET(HwADCCON, Hw4|(Hw3-Hw0), Hw4);	// ADC Standby.. & ADC Input Channel Select Clear
	
	return (adcdata>>1);
#endif	
#endif


}

/*****************************************************************************
* Function Name : tca_keypad_getkeycodebyscancode()
******************************************************************************
* Desription  	:
* Parameter   	:
* Return      	:
* Note          : 
******************************************************************************/
int tca_keypad_getkeycodebyscancode(unsigned int adcdata)
{
	int i;
	int key = -1;
	
	if(adcdata > 4050)
		key = -1;
	else
	{	
		//printk("Out Key Value :: %d\n", adcdata);
		for (i = 0; i < sizeof(stScanCodeMapping)/sizeof(SCANCODE); i++)
		{
			if ((adcdata >= stScanCodeMapping[i].StartVal) && (adcdata <= stScanCodeMapping[i].EndVal))
				key = stScanCodeMapping[i].KeyCode;
		}
	}
	
	return key;
}

/*****************************************************************************
* Function Name : tca_keypad_getkeycodebyscancode()
******************************************************************************
* Desription  	:
* Parameter   	:
* Return      	:
* Note          : 
******************************************************************************/
void tca_keypad_adccontrollerinit(void)
{
	//    ADC Controller initialize here for Keypad..
#if defined(CONFIG_ARCH_TCC92X)
//	volatile PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
	volatile PTSADC pTSADC = (volatile PTSADC)tcc_p2v(HwTSADC_BASE);

	tca_ckc_setiobus(RB_TSADCCONTROLLER,1); 

	BITCSET(HwGPIOE->GPFN3,HwPORTCFG_GPFN0_MASK,HwPORTCFG_GPFN0(1));

	BITCSET(pTSADC->ADCCON,Hw15-Hw6, Hw14|(0xB<<6));	// ADC Prescaler	// prescale 12 (11+1),  PCK (12MHz)
	BITCSET(pTSADC->ADCCON,Hw2|(Hw6-Hw3), Hw2);	// ADC Standby.. & ADC Input Channel Select Clear	
#elif defined(CONFIG_ARCH_TCC93XX)
	volatile PTSADC pTSADC = (volatile PTSADC)tcc_p2v(HwTSADC_BASE);
	volatile PGPION pGPIO  = (volatile PGPION)tcc_p2v(HwGPIOG_BASE);

	tca_ckc_setiobus(RB_TSADCCONTROLLER,1); 

    BITCSET(pGPIO->GPFN2 , 0x0F000000, 0x01000000);  // Function AIN[4] key
    BITCLR(pGPIO->GPEN, Hw22); // Port as Input

	BITCSET(pTSADC->ADCCON,Hw15-Hw6, Hw14|(0xB<<6));	// ADC Prescaler	// prescale 12 (11+1),  PCK (12MHz)
	BITCSET(pTSADC->ADCCON,Hw2|(Hw6-Hw3), Hw2);	// ADC Standby.. & ADC Input Channel Select Clear	
#else
	// Enable Bus Clock for ADC
	BITSET(HwBCLKCTR, HwBCLKCTR_ADC_ON);
	
#if defined(_TCC79x_)
	BITCLR(HwPORTCFG8,0xF000); // Port Configure as ADC, Channel 0
#elif defined(_TCC80x_)
	BITSET(HwGPIOAD_FS0, Hw0);	// Port confiture as ADIN0 -> Keypad
#endif
	BITCSET(HwADCCON, Hw4|(Hw3-Hw0), Hw4);	// ADC Standby.. & ADC Input Channel Select Clear

#ifdef __KERNEL__
#ifdef _TCC83x_
    HwPCLKCFG6 = (1 << 15) | 0x40;
#endif
#endif
	// 0x|ClockDivisorValue|DelayCnt|FIFOThreshold|IEN,R8,PD,SM|
	BITSET(HwADCCFG, 0xFF00);
#endif
}
EXPORT_SYMBOL(tca_keypad_adccontrollerinit);

/*****************************************************************************
* Function Name : tca_keypad_adcpowerdown()
******************************************************************************
* Desription  	:
* Parameter   	:
* Return      	:
* Note          : 
******************************************************************************/
void tca_keypad_adcpowerdown(void)
{
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
	tca_ckc_setperi(PERI_ADC,DISABLE,120000, PCDIRECTXIN);
	tca_ckc_setiobus(RB_TSADCCONTROLLER,0); 
#else
	BITSET(HwADCCFG, Hw0|Hw1);
#if defined(_TCC79x_)
	BITCLR(HwPCK_ADC, Hw28);
#elif defined(_TCC80x_)
	BITCLR(HwACLKADC, Hw28);
#endif
	BITCLR(HwBCLKCTR, HwBCLKCTR_ADC_ON);
#endif
}

/*****************************************************************************
* Function Name : tca_keypad_adcpowerup()
******************************************************************************
* Desription  	:
* Parameter   	:
* Return      	:
* Note          : 
******************************************************************************/
void tca_keypad_adcpowerup(void)
{
#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
    volatile PTSADC pTSADC = (volatile PTSADC)tcc_p2v(HwTSADC_BASE);	

	tca_ckc_setiobus(RB_TSADCCONTROLLER,1); 
	tca_ckc_setperi(PERI_ADC,ENABLE,120000, PCDIRECTXIN);
#else
	BITSET(HwBCLKCTR, HwBCLKCTR_ADC_ON);
#if defined(_TCC79x_)
	BITSET(HwPCK_ADC, Hw28);
#elif defined(_TCC80x_)
	BITSET(HwACLKADC, Hw28);
#endif
	BITCLR(HwADCCFG, Hw0|Hw1);
#endif
}

