
/****************************************************************************
 *   FileName    : tca_remocon.h
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

//*******************************************
//	Remote controller define
//*******************************************

//Low Bit
#if (defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX))
#define LOW_MIN			0x0700
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1800
#define HIGH_MAX		0x1E00
//Start Bit
#define START_MIN		0x4000
#define START_MAX		0x5000
//Repeat Bit
#define REPEAT_MIN		0x2000
#define REPEAT_MAX		0x2800
#else
//Low Bit
#define LOW_MIN			0x0500
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1000
#define HIGH_MAX		0x1D00
//Start Bit
#define START_MIN		0x2500
#define START_MAX		0x5000
#endif


/*****************************************************************************
*
* IR remote controller scancode
*
******************************************************************************/
#define REM_ID_ERROR	0xFF

	#define REMOCON_ID			0x106F
	#define REMOCON_REPEAT		0x10000000
	#define SCAN_PWR			0x0001

	#define SCAN_POWER			0x8877

	#define SCAN_NUM_1			0xC43B
	#define SCAN_NUM_2			0xA45B
	#define SCAN_NUM_3			0xE41B
	#define SCAN_NUM_4			0x946B
	#define SCAN_NUM_5			0xD42B
	#define SCAN_NUM_6			0xB44B
	#define SCAN_NUM_7			0xF40B
	#define SCAN_NUM_8			0x8C73
	#define SCAN_NUM_9			0xCC33
	#define SCAN_NUM_0			0x847B
	#define SCAN_NUM_MINUS		0x9966
	#define SCAN_NUM_PREV		0xAC53

	#define SCAN_MUTE			0xC837
	#define SCAN_INPUT			0xE817

	#define SCAN_VOLUP			0xA05F
	#define SCAN_VOLDN			0xE01F

	#define SCAN_CH_UP			0x807F
	#define SCAN_CH_DN			0xC03F

	#define SCAN_SLEEP			0xB847
	#define SCAN_CANCEL			0xED12
	#define SCAN_MENU			0xE11E
	#define SCAN_GUIDE			0xCAB5

	#define SCAN_UP				0x817E
	#define SCAN_DOWN			0xC13E
	#define SCAN_LEFT			0xF00F
	#define SCAN_RIGHT			0xB04F
	#define SCAN_CENTER			0x916E

	#define SCAN_AUTO_CHANNEL	0x956A
	#define SCAN_ADD_DELETE		0xD52A
	#define SCAN_SIZE			0xF708
	#define SCAN_MULTI_SOUND	0xA857

	#define SCAN_STOP			0xC6B9
	#define SCAN_PLAY			0x86F9
	#define SCAN_PAUSE			0xAED1
	#define SCAN_REC			0xDEA1
	#define SCAN_FB				0xB8C7
	#define SCAN_FF				0xF887
	#define SCAN_CURRENT			0xBCC3
	

/*****************************************************************************
*
* External Variables
*
******************************************************************************/
#define NO_KEY		0

static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_POWER, 		107},

	{SCAN_NUM_1,		2},		// 1
	{SCAN_NUM_2,		3},		// 2
	{SCAN_NUM_3,		4},		// 3
	{SCAN_NUM_4,		5},		// 4
	{SCAN_NUM_5,		6},		// 5
	{SCAN_NUM_6,		7},		// 6
	{SCAN_NUM_7,		8},		// 7
	{SCAN_NUM_8,		9},		// 8
	{SCAN_NUM_9,		10},	// 9
	{SCAN_NUM_0,		11},	// 0
	{SCAN_NUM_MINUS,	12},	// MINUS
	{SCAN_NUM_PREV,		52},	// PERIOD

	{SCAN_MUTE,			200},	// MUTE
	{SCAN_INPUT,		235},	// F1

	{SCAN_VOLUP,		115},	// VOLUME_UP
	{SCAN_VOLDN,		114},	// VOLUME_DOWN

	{SCAN_CH_UP,		205},	// PAGE_UP
	{SCAN_CH_DN,		206},	// PAGE_DOWN

	{SCAN_SLEEP,		242},	// F10
	{SCAN_CANCEL,		158},	// BACK
	{SCAN_MENU,			139},	// MENU
	{SCAN_GUIDE,		102},	// HOME

	{SCAN_UP,			103},	// DPAD_UP
	{SCAN_DOWN,		108},	// DPAD_DOWN
	{SCAN_LEFT,			105},	// DPAD_LEFT
	{SCAN_RIGHT,		106},	// DPAD_RIGHT
#if 1
	{SCAN_CENTER,		232},	// DPAD_CENTER
#else
	{SCAN_CENTER,		28},	// ENTER
#endif

	{SCAN_AUTO_CHANNEL,	234},	// F2
	{SCAN_ADD_DELETE,	14},	// DEL
	{SCAN_SIZE,			243},	// F11
	{SCAN_MULTI_SOUND,	244},	// F12

	{SCAN_STOP, 		199},	// MEDIA_STOP
	{SCAN_PLAY,			198},	// MEDIA_PLAY_PAUSE
	{SCAN_PAUSE,		198},	// MEDIA_PLAY_PAUSE
	{SCAN_REC,			207},	// NOTIFICATION
#if 1
	{SCAN_FB,			201},	// MEDIA_REWIND
	{SCAN_FF,			202},	// MEDIA_FAST_FORWARD
#else
	{SCAN_FB,			204},	// MEDIA_PREVIOUS
	{SCAN_FF,			203},	// MEDIA_NEXT
#endif
	{SCAN_CURRENT,		15},	// TAB
};
