/*
 * linux/drivers/serial/tcc_ts_tsc.c
 *
 * Author: <linux@telechips.com>
 * Created: June 10, 2008
 * Description: Touchscreen driver for TSC2003 on Telechips TCC Series
 *
 * Copyright (C) 2008-2009 Telechips 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/device.h>

#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/sched.h>
#include <linux/clk.h>

#include <linux/slab.h>
#include <linux/irq.h>

#include <linux/spinlock.h>

#include <mach/hardware.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include <asm/io.h>
#ifdef CONFIG_ARCH_TCC79X
#include <mach/tcc79x.h>
//#include "TCC79x_Virtual.h"
#elif defined(CONFIG_ARCH_TCC83X)
#include <mach/tcc83x_virt.h>
#include <mach/irqs.h>
//#include "TCC83x_Virtual.h"
#elif defined(CONFIG_ARCH_TCC92X)
#include <mach/irqs.h>
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC93XX) //kch
#include <mach/bsp.h>
#include <mach/bsp.h>
#include <mach/io.h>
#endif

#include "tcc_ts_tsc.h"
#if defined(_TOUCH_AK4183_)
#include "tca_ak4183tchctl.h"
#endif


#ifdef CONFIG_I2C
#   include <linux/i2c.h>
#else
#   include "tcc83xx_i2c.h"
#endif



// =============================================================
// tca_tchhwctl.c
#if defined(CONFIG_ARCH_TCC92X)
#define TSC_EINT_PIN		Hw4
#define TSC_EINT_PORT_EN	HwGPIOA->GPEN
#define TSC_EINT_PORT_DAT	HwGPIOA->GPDAT
#define TSC_EINT_SEL		HwEINTSEL2_EINT11(SEL_GPIOA4)

#define TSC_EINT_SEL_MASK	HwEINTSEL2_EINT11_MASK
#define TSC_EINT_SEL_ID 	HwINT0_EI11
#endif

#if defined(CONFIG_MACH_TCC9300) //kch
#define TSC_EINT_PIN		Hw5
#define TSC_EINT_PORT_EN		((PGPION)tcc_p2v(HwGPIOA_BASE))->GPEN
#define TSC_EINT_PORT_DAT	((PGPION)tcc_p2v(HwGPIOA_BASE))->GPDAT
#define TSC_EINT_SEL		HwEINTSEL2_EINT11(SEL_GPIOA5)

#define TSC_EINT_SEL_MASK	HwEINTSEL2_EINT11_MASK
#define TSC_EINT_SEL_ID 	HwINT0_EI11
#elif defined(CONFIG_MACH_TCC9300CM)
#define TSC_EINT_PIN		Hw4
#define TSC_EINT_PORT_EN		((PGPION)tcc_p2v(HwGPIOA_BASE))->GPEN
#define TSC_EINT_PORT_DAT	((PGPION)tcc_p2v(HwGPIOA_BASE))->GPDAT
#define TSC_EINT_SEL		HwEINTSEL2_EINT11(SEL_GPIOA4)

#define TSC_EINT_SEL_MASK	HwEINTSEL2_EINT11_MASK
#define TSC_EINT_SEL_ID 	HwINT0_EI11
#endif
// =============================================================





// =============================================================
// tca_tsc2003tchctl.c
#define SC2003_CMD_MEASURE_X	(12<<4)
#define SC2003_CMD_MEASURE_Y	(13<<4)

#define SC2003_CMD_MEASURE_Z1	(14<<4)
#define SC2003_CMD_MEASURE_Z2	(15<<4)

#define SC2003_CMD_XY_OFF		(0<<4)

#define SC2003_CMD_POWER_0		(0<<2)
#define SC2003_CMD_POWER_1		(1<<2)
#define SC2003_CMD_POWER_2		(2<<2)
#define SC2003_CMD_POWER_3		(3<<2)
// =============================================================



// =============================================================
// tca_tchcontrol.c


#define MAX_X  	3870
#define MIN_X  	170
#define MAX_Y 	3800
#define MIN_Y  	290


//for the digi_int_handle
#if defined(_TCC80x_)
#define TOUCH_COLLECT_NR		 6  // (TOUCH_COLLECT_NR-TOUCH_VALID_VALUE) must be even
#define TOUCH_VALID_VALUE		 2  // among COLLECT_NR samples, OK should be over 
#define TOUCH_PRESSURE_LIMIT  4000//	8000
#else
#define TOUCH_COLLECT_NR		6  // 4 // (TOUCH_COLLECT_NR-TOUCH_VALID_VALUE) must be even
#define TOUCH_VALID_VALUE		2  // among COLLECT_NR samples, OK should be over 
#define TOUCH_PRESSURE_LIMIT	8000
#endif


#define TCA_CAL_TOUCH_POS
#if defined(TCA_CAL_TOUCH_POS)
#define	TCA_ABS(x,y)	(((x)>(y))?(x)-(y):(y)-(x))
unsigned int tca_old_pos_x[3], tca_old_pos_y[3];
unsigned int tca_old_pos_idx;
#endif
// =============================================================





//==============================================================
#define DEVICE_NAME	"tsc2003_ts"

#define CMD_SLEEP_NO		0x80
#define CMD_ADC_ON		0x04

#define PEN_DOWN 1
#define PEN_RELEASE 0

#if defined(CONFIG_MACH_TCC9300) //kch
#define READ_PENIRQ() ((((PGPIO)tcc_p2v(HwGPIOA_BASE))->GPADAT) & Hw5)
#elif defined(CONFIG_MACH_TCC9300CM)
#define READ_PENIRQ() ((((PGPIO)tcc_p2v(HwGPIOA_BASE))->GPADAT) & Hw4)
#else
#define READ_PENIRQ() (HwGPIOA->GPDAT & Hw4)
#endif

#if defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01)
#define X_AXIS_MIN	10
#define X_AXIS_MAX	2000
#define Y_AXIS_MIN	10
#define Y_AXIS_MAX	2000
#elif defined(CONFIG_FB_TCC92XX_WVGA_LMS480KF)|| defined(CONFIG_FB_TCC92XX_WVGA_LB070WV6)
#define X_AXIS_MIN	10
#define X_AXIS_MAX	2000
#define Y_AXIS_MIN	140
#define Y_AXIS_MAX	1887
#elif defined(CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW)
#define X_AXIS_MIN	10
#define X_AXIS_MAX	2000
#define Y_AXIS_MIN	140
#define Y_AXIS_MAX	1887
#elif defined(CONFIG_FB_TCC89XX_WXGA_HT121WX2)
#define X_AXIS_MIN	10
#define X_AXIS_MAX	2000
#define Y_AXIS_MIN	140
#define Y_AXIS_MAX	1887
#endif



struct ts_event {
	unsigned short pressure;
	unsigned short x;
	unsigned short y;
};

struct tsc2003_ts {
    struct input_dev *input;
    unsigned int x, y, pressure, valid;
    int pen_status, opened, running;
    int irq;
    struct mutex mutex;
	struct work_struct work_q;
#ifdef CONFIG_HAS_EARLYSUSPEND
    struct early_suspend early_suspend;
#endif
	struct clk *clk;
};

struct tsc2003_ts *tsc2003_ts;
static struct timer_list ts_timer;
static int irq_no;

#ifdef CONFIG_I2C
#if defined(CONFIG_FB_TCC7901_DEMOBOARD)
static unsigned short probe[] = { 1, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_FB_TCC7901_LMS350DF01)
static unsigned short probe[] = { 0, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_MACH_TCC9300) //kch
static unsigned short probe[] = { 4, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_MACH_TCC9300CM)
static unsigned short probe[] = { 1, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_ARCH_TCC92X)
#if defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01)
static unsigned short probe[] = { 0, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_FB_TCC92XX_WVGA_LMS480KF)|| defined(CONFIG_FB_TCC92XX_WVGA_LB070WV6)
static unsigned short probe[] = { 1, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW)
static unsigned short probe[] = { 1, 0x48, I2C_CLIENT_END };
#elif defined(CONFIG_FB_TCC89XX_WXGA_HT121WX2)
static unsigned short probe[] = { 1, 0x48, I2C_CLIENT_END };
#endif
#endif


static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe      = probe,
	.ignore     = dummy,
};

static struct i2c_driver ak_i2c_driver;
static struct i2c_client *ak_i2c_client;

#if 0
static void TOUCH_SEND_CMD(unsigned char reg, unsigned char val)
{
	unsigned char cmd[1];
	cmd[0] = val;
	i2c_master_send(ak_i2c_client, cmd, 1);
}
#endif





void tca_tchhw_enableirq(void)
{
#if defined(_TCC79x_)
	HwINTMSK |= HwINT_EI2; 
#elif defined(_TCC83x_)
	BITSET(HwCLR_B , HwIEN_EI2); 
	BITSET(HwIEN_B , HwIEN_EI2); 
	BITSET(HwIEN , Hw0); 
#elif defined(_TCC80x_)
	BITSET(HwCREQ_B , Hw7);  // PK[7] External Request 7 
	BITSET(HwIEN_B , Hw7);  // PK[7] External Request 7 
	BITSET(HwIRQSEL_B , Hw7);  // PK[7] 1, it means that the corresponding interrupt is considered as IRQ interrupt
#elif defined(CONFIG_ARCH_TCC92X)
	HwPIC->INTMSK0 |= TSC_EINT_SEL_ID;
#elif defined(CONFIG_ARCH_TCC93XX) //kch
	((PPIC)tcc_p2v(HwPIC_BASE))->INTMSK0 |= TSC_EINT_SEL_ID;

#endif
}

void tca_tchhw_disableirq(void)
{
#if defined(_TCC79x_)
	HwINTMSK &= ~HwINT_EI2; 
#elif defined(_TCC83x_)
	BITCLR(HwIEN_B , HwIEN_EI2);
#elif defined(_TCC80x_)
	BITCLR(HwIEN_B , Hw7);  // PK[7] External Request 7 
#elif defined(CONFIG_ARCH_TCC92X)
	HwPIC->INTMSK0 &= ~TSC_EINT_SEL_ID;
#elif defined(CONFIG_ARCH_TCC93XX) //kch
	((PPIC)tcc_p2v(HwPIC_BASE))->INTMSK0 &= ~TSC_EINT_SEL_ID;
#endif
}
unsigned int tca_tchhw_getirqnum(void)
{
#if defined(_TCC83x_)
#ifdef __KERNEL__
    return INT_EI2;
#endif
#elif defined(_TCC79x_)
	return IRQ_EI2;
#elif defined(_TCC80x_)
	return IRQ_EI7;
#elif defined(CONFIG_ARCH_TCC92X)
	return INT_EI11;
#elif defined(CONFIG_ARCH_TCC93XX) // by kch
	return INT_EI11;
#endif
}
void tca_tchhw_initport(void)
{
#if defined(_TCC79x_)
#ifdef CONFIG_MACH_TCC7901S
	BITCSET(HwPORTCFG9, Hw20 - Hw16, Hw16);	
	BITCLR(HwGPEEN, Hw31);
	BITCSET(HwEINTSEL0, Hw21 - Hw16, Hw17);	
	BITSET(HwGPEDAT, Hw31);
#else
	//EINT_2 init
	BITCLR(HwPORTCFG4, Hw20 - Hw16);
	BITCLR(HwGPAEN, Hw2);
	BITCSET(HwEINTSEL0, Hw21 - Hw16, Hw20 | Hw17);	
#endif
    BITSET(HwSEL, HwINT_EI2);
    BITSET(HwIRQMSK, HwINT_EI2);

    BITSET(HwPOL, HwINT_EI2);
#ifdef __KERNEL__
    BITCLR(HwMODE, HwINT_EI2);
#else
    BITSET(HwMODE, HwINT_EI2);
#endif

#elif defined(_TCC83x_)
#ifdef __KERNEL__
//#       if defined(_AK4183_)
    BITCLR(HwGIOCON_B, Hw6);
    //EINT_2 init, GPIO_B6
    HwNGSEL_B &= ~(1<<6);
    HwGPIOB_PE |= (1<<6);
    HwGPIOB_PS |= (1<<6);
    HwGSEL_B |= HwGSEL_B_N1_EI2;

    BITCLR(HwICFG, Hw11);
    BITCLR(HwICFG, Hw10 | Hw9);

    //set IRQ as level sensitive; low active
    HwSEL_B |= HwSEL_EI2;
    HwIEN_B &= ~HwIEN_EI2;
    HwMODE_B |= HwMODE_EI2;
    HwSYNC_B |= HwSYNC_EI2;
#else
	//EINT_2 init
	BITSET(HwGSEL_B,    Hw4);  //GPIO_B6 setting GPIO 
	BITCLR(HwGIOCON_B,  Hw6);  //GPIO_B6 input mode setting
	BITSET(HwSEL_B,		Hw2);
	BITSET(HwMSTS_B,	Hw2);
	BITCLR(HwMODE_B,	Hw2);
	BITSET(HwCLR_B,		Hw2); 

	//set low level trigger
	BITCLR(HwICFG, Hw11);	   
	BITSET(HwICFG, Hw10|Hw9);      
#endif

#elif defined(_TCC80x_)
	// PK7 --> EXT7
	BITSET(HwPMIO_FS0, Hw7); // 1 : PK pin controlled by GPIOPK_DAT/DOE/FS0~2 
	BITSET(HwGPIOPK_FS0,Hw7);// External Interrupt Inputs [7:0] to Interrupt Controller 
	BITCLR(HwGPIOPK_DOE,Hw7);// GPIO_PK7 Input mode Setting 
	BITSET(HwGPIOPK_IRQST,Hw7);// Interrupt Status for PK[31:0] 
	BITSET(HwGPIOPK_IRQTM0,Hw7);// Level Trigger

	BITSET(HwGPIOPK_IRQEN,Hw7);// Interrupt Enable for PK[31:0] (0 = Disable, 1 = Enable)

	BITSET(HwPOL_B,Hw7);	// Low Active
	BITSET(HwTMODE_B,Hw7);	// Level
	BITSET(HwSYNC_B,Hw7);	// Sync Disable
#elif defined(CONFIG_ARCH_TCC92X)
	//power control
	BITCSET(HwGPIOE->GPFN3, HwPORTCFG_GPFN7_MASK, HwPORTCFG_GPFN7(0));
	HwGPIOE->GPEN	|= Hw31;
	HwGPIOE->GPDAT	|= Hw31;	

	// Interrupt Control
	BITCLR(TSC_EINT_PORT_EN, TSC_EINT_PIN);  //Input mode
	BITCSET(HwEINTSEL->EINTSEL2, TSC_EINT_SEL_MASK, TSC_EINT_SEL);

	HwPIC->INTMSK0 |= TSC_EINT_SEL_ID;	   

	
	HwPIC->IEN0 	&= ~(TSC_EINT_SEL_ID);						/* disable I2C */
	HwPIC->MODEA0	&= ~(TSC_EINT_SEL_ID) ; 					/* signle edge */

	HwPIC->MODE0	&= ~(TSC_EINT_SEL_ID);						/* set edge trigger mode */	

	HwPIC->POL0 	|= (TSC_EINT_SEL_ID);						/* active-high */
	HwPIC->CLR0 	=  TSC_EINT_SEL_ID; 						/* clear pending status */
	HwPIC->IEN0 	|= (TSC_EINT_SEL_ID);						/* Enable Interrupt */


//  BITSET(HwPOL, HwINT_EI2);
//	BITCLR(HwMODE, HwINT_EI2);

#elif defined(CONFIG_MACH_TCC9300) //kch
	// Interrupt Control

	BITCLR((TSC_EINT_PORT_EN) , TSC_EINT_PIN);  //Input mode
	BITCSET(((PGPIOINT)tcc_p2v(HwEINTSEL_BASE))->EINTSEL2, TSC_EINT_SEL_MASK, TSC_EINT_SEL);

	((PPIC)tcc_p2v(HwPIC_BASE))->INTMSK0 |= TSC_EINT_SEL_ID;	   

	
	((PPIC)tcc_p2v(HwPIC_BASE))->IEN0 	&= ~(TSC_EINT_SEL_ID);						/* disable I2C */
	((PPIC)tcc_p2v(HwPIC_BASE))->MODEA0	&= ~(TSC_EINT_SEL_ID) ; 					/* signle edge */

	((PPIC)tcc_p2v(HwPIC_BASE))->MODE0	&= ~(TSC_EINT_SEL_ID);						/* set edge trigger mode */	

	((PPIC)tcc_p2v(HwPIC_BASE))->POL0 	|= (TSC_EINT_SEL_ID);						/* active-high */
	((PPIC)tcc_p2v(HwPIC_BASE))->CLR0 	=  TSC_EINT_SEL_ID; 						/* clear pending status */
	((PPIC)tcc_p2v(HwPIC_BASE))->IEN0 	|= (TSC_EINT_SEL_ID);						/* Enable Interrupt */

#elif defined(CONFIG_MACH_TCC9300CM)
	// Interrupt Control

	BITCLR((TSC_EINT_PORT_EN) , TSC_EINT_PIN);  //Input mode
	BITCSET(((PGPIOINT)tcc_p2v(HwEINTSEL_BASE))->EINTSEL2, TSC_EINT_SEL_MASK, TSC_EINT_SEL);

	((PPIC)tcc_p2v(HwPIC_BASE))->INTMSK0 |= TSC_EINT_SEL_ID;	   

	
	((PPIC)tcc_p2v(HwPIC_BASE))->IEN0 	&= ~(TSC_EINT_SEL_ID);						/* disable I2C */
	((PPIC)tcc_p2v(HwPIC_BASE))->MODEA0	&= ~(TSC_EINT_SEL_ID) ; 					/* signle edge */

	((PPIC)tcc_p2v(HwPIC_BASE))->MODE0	&= ~(TSC_EINT_SEL_ID);						/* set edge trigger mode */	

	((PPIC)tcc_p2v(HwPIC_BASE))->POL0 	|= (TSC_EINT_SEL_ID);						/* active-high */
	((PPIC)tcc_p2v(HwPIC_BASE))->CLR0 	=  TSC_EINT_SEL_ID; 						/* clear pending status */
	((PPIC)tcc_p2v(HwPIC_BASE))->IEN0 	|= (TSC_EINT_SEL_ID);						/* Enable Interrupt */
#endif

}




// Initialize related registers
void tca_tch_init(void)
{

	tea_tch_openi2c();
	
	tca_tchhw_initport();

#if	defined(_TOUCH_AK4183_)
	tca_ak4183_init();
	tca_ak4183_init();
#else
	tca_tsc2003_init();
	tca_tsc2003_init();
#endif

}

void tca_tch_poweroff(void)
{
#if	defined(_TOUCH_AK4183_)
#ifdef __KERNEL__
    tca_ak4183_poweroff();
#endif
#else
	tca_tsc2003_poweroff();
#endif
}


void tca_touchbubblesort(unsigned int Number[],unsigned int num) 
{
	int i,j;
	unsigned int temp;
	for(i=0 ; i<(int)(num-1) ; i++)   
	{    
		for(j=i+1;j<(int)num;j++)
		{ 
			if(Number[i]>Number[j]) 
			{ 
				temp   = Number[i]; 
				Number[i] = Number[j]; 
				Number[j] = temp;
			}
		} 
	}
}

int tca_getrawdata2(int * x, int * y)
{
	int i;
	unsigned int nDatax, nDatay,nDataz1,nDataz2;
	for(i=0; i < 1; i++)
	{
		nDatax = tea_tch_readi2c(READ_X);
		nDatay = tea_tch_readi2c(READ_Y);
		nDataz1 = tea_tch_readi2c(READ_Z1);

		if(nDataz1!=0)
		{
			nDataz2 = tea_tch_readi2c(READ_Z2);
			if((nDatax*nDataz2/nDataz1-nDatax)<TOUCH_PRESSURE_LIMIT)
			{
				*x = nDatax;
				*y = nDatay;
				return 0;
			}
		}
		else
		{
		}
			
	}	
	return -1;
}

unsigned int tca_touch_getvaliddata(unsigned int Number[], unsigned int num)
{
	unsigned int CurrNum, CurrCnt, OldNum, OldCnt;
	int i, InvalidCnt;

	OldNum=OldCnt=0;
	InvalidCnt = 0;
	CurrNum = Number[0];
	CurrCnt = 1;
	
	for (i=1 ; i<num ; i++)
	{
		if (CurrNum != Number[i])
		{
			if (OldCnt < CurrCnt)
			{
				OldCnt = CurrCnt;
				OldNum = CurrNum;
				InvalidCnt = 0;
			}
			else if (OldCnt == CurrCnt)
			{
				InvalidCnt++;
			}
			CurrNum = Number[i];
			CurrCnt = 1;
		}
		else
			CurrCnt++;
	}

	if (OldCnt < CurrCnt)
	{
		OldCnt = CurrCnt;
		OldNum = CurrNum;
		InvalidCnt = 0;
	}
	else if (OldCnt == CurrCnt)
	{
		InvalidCnt++;
	}

	if (InvalidCnt)
		return 0;

	return OldNum;
}


int tca_getrawdata(int * x, int * y)
{
	unsigned int  i, ValidNum;
	unsigned int r_x[TOUCH_COLLECT_NR], r_y[TOUCH_COLLECT_NR];
	unsigned long  x_tol, y_tol;
	unsigned int m_pos_x, m_pos_y;
	unsigned int z1,z2;
	unsigned int index;
	
	m_pos_x = m_pos_y = 0;
	ValidNum = 0; 
	for(i = 0; i < TOUCH_COLLECT_NR; i ++)
	{
		r_x[ValidNum] = tea_tch_readi2c(READ_X);
		r_y[ValidNum] = tea_tch_readi2c(READ_Y);

		if((r_x[ValidNum] != 0)&&(r_y[ValidNum] != 0)) 
		{		
			z1 = tea_tch_readi2c(READ_Z1);
			if(z1!=0)
			{
				z2 = tea_tch_readi2c(READ_Z2);
				if((r_x[ValidNum]*z2/z1-r_x[ValidNum])<TOUCH_PRESSURE_LIMIT)
					ValidNum++;
			}				
		}
		
	}
	if(ValidNum<TOUCH_VALID_VALUE)
	{
		goto digi_int_exit;
	}
    
	tca_touchbubblesort(r_x,ValidNum);	
	tca_touchbubblesort(r_y,ValidNum);

	x_tol = y_tol = 0; //sum the coordinate values
	m_pos_x = tca_touch_getvaliddata(r_x, ValidNum);
	m_pos_y = tca_touch_getvaliddata(r_y, ValidNum);
	if (!m_pos_x || !m_pos_y)
	{
		index = (ValidNum-TOUCH_VALID_VALUE)>>1;
		
		for(i=index;i<(index+TOUCH_VALID_VALUE);++i)
		{
			x_tol += r_x[i];
			y_tol += r_y[i];		
		}
		m_pos_x = x_tol/TOUCH_VALID_VALUE;
		m_pos_y = y_tol/TOUCH_VALID_VALUE;
	}

#if defined(TCA_CAL_TOUCH_POS)
	// 1.   ǥ 1st_Ago ǥ ٸ.
	if ((tca_old_pos_x[tca_old_pos_idx]!= m_pos_x) || (tca_old_pos_y[tca_old_pos_idx]!= m_pos_y))
	{
		// 2.   ǥ 1st_Ago ǥ ̰ 16 ۴.
		if ((TCA_ABS(tca_old_pos_x[tca_old_pos_idx], m_pos_x) <= 16) && (TCA_ABS(tca_old_pos_y[tca_old_pos_idx], m_pos_y) <= 16))
		{
			// 3. 1st_Ago  2nd_Ago ǥ .
			if ((tca_old_pos_x[tca_old_pos_idx] == tca_old_pos_x[(tca_old_pos_idx+2)%3]) &&
				(tca_old_pos_y[tca_old_pos_idx] == tca_old_pos_y[(tca_old_pos_idx+2)%3]))
			{
				*x = tca_old_pos_x[(tca_old_pos_idx+1)%3];
				*y = tca_old_pos_y[(tca_old_pos_idx+1)%3];

				tca_old_pos_idx++;
				if (tca_old_pos_idx > 2)
					tca_old_pos_idx = 0;

				tca_old_pos_x[tca_old_pos_idx] = m_pos_x;
				tca_old_pos_y[tca_old_pos_idx] = m_pos_y;
	
				if(*x && *y)
				{
					return 0;
				}
				else
				{
					return -1;
				}
			}
		}
	}

	tca_old_pos_idx++;
	if (tca_old_pos_idx > 2)
		tca_old_pos_idx = 0;

	tca_old_pos_x[tca_old_pos_idx] = m_pos_x;
	tca_old_pos_y[tca_old_pos_idx] = m_pos_y;
#endif

	//printk("ValidNum : %d, StartIdx : %d\n", ValidNum, index);
	//printk("(%d): %04d,%04d,%04d,%04d,%04d,%04d,%04d,%04d\n", m_pos_x, r_x[0],r_x[1],r_x[2],r_x[3],r_x[4],r_x[5],r_x[6],r_x[7]);
	//printk("(%d): %04d,%04d,%04d,%04d,%04d,%04d,%04d,%04d\n", m_pos_y, r_y[0],r_y[1],r_y[2],r_y[3],r_y[4],r_y[5],r_y[6],r_y[7]);
    
digi_int_exit:
	*x = m_pos_x;
	*y = m_pos_y;
	if(m_pos_x && m_pos_y)
	{
		return 0;
	}
	else
	{
		return -1;
	}


 }

void tca_tsc2003_setreadcmd(int channel, unsigned char *nBfrTXD)
{	
	if(channel == READ_X)		
		nBfrTXD[0] = SC2003_CMD_MEASURE_X | SC2003_CMD_POWER_3;	
	else if(channel == READ_Y)		
		nBfrTXD[0] = SC2003_CMD_MEASURE_Y | SC2003_CMD_POWER_3;	
	else if(channel == READ_Z1)		
		nBfrTXD[0] = SC2003_CMD_MEASURE_Z1 | SC2003_CMD_POWER_3;	
	else if(channel == READ_Z2)		
		nBfrTXD[0] = SC2003_CMD_MEASURE_Z2 | SC2003_CMD_POWER_3;
}

void tca_tsc2003_poweroff(void)
{	
	tea_tch_writei2c(SC2003_CMD_XY_OFF | SC2003_CMD_POWER_0);	
}

void tca_tsc2003_init(void)
{	
	tca_tchhw_enableirq();		
	tca_tsc2003_poweroff();
}



void tca_rawtoscreen(int *x, int *y, unsigned int iScreenWidth, unsigned int iScreenHeight)
{
    unsigned int posX, posY;
    

	if(*x>MAX_X)
		*x=MAX_X;
	else if(*x<MIN_X)
		*x=MIN_X;

	if(*y>MAX_Y)
		*y=MAX_Y;
	else if(*y<MIN_Y)
		*y=MIN_Y;

	posX = (MAX_X-*x)*iScreenWidth/(MAX_X-MIN_X);
	posY = (*y-MIN_Y)*iScreenHeight/(MAX_Y-MIN_Y);

    *x = posX<<2;
    *y = posY<<2;
}



static void TOUCH_READ_DAT(unsigned char reg, unsigned int val)
{
	unsigned char *buf;
	buf = (unsigned char *)val;
	i2c_master_recv(ak_i2c_client, buf, 2);
}

static int ak_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
    struct i2c_client *i2c;
    int ret;

    i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
    if (!i2c)
        return -ENOMEM;

    strcpy(i2c->name, "TSC2003");
    i2c->flags = 0;
    i2c->addr = addr;
    i2c->adapter = adap;
    i2c->driver = &ak_i2c_driver;

    ak_i2c_client = i2c;

    /* attach i2c client device */
    ret = i2c_attach_client(i2c);
    if (ret < 0) {
        printk("%s: failed to attach codec at addr %x\n", __func__, addr);
        goto err;
    }

	
    return ret;
err:
    kfree(i2c);
    return ret;
}

static int ak_i2c_detach(struct i2c_client *client)
{
    i2c_detach_client(client);
    kfree(ak_i2c_client);
    return 0;
}

static int ak_i2c_attach(struct i2c_adapter *adap)
{
    return i2c_probe(adap, &addr_data, ak_i2c_probe);
}

/* tsc2003 i2c codec control layer */
static struct i2c_driver ak_i2c_driver = {
    .driver = {
        .name   = "tcc-tsc2003-ts",
        .owner  = THIS_MODULE,
    },
    .id             = I2C_DRIVERID_TSC2003,
    .attach_adapter = ak_i2c_attach,
    .detach_client  = ak_i2c_detach,
    .command        = NULL,
};
#endif

/*
static void write_cmd_ak(unsigned char cmd)
{
	TOUCH_SEND_CMD(0, cmd);
	
#ifndef CONFIG_I2C
	udelay(1);
#endif
}
*/

static short read_ak(int cmd)
{
	unsigned char data[]={0,0};
	short value;
	
	cmd |= (CMD_SLEEP_NO | CMD_ADC_ON);
    tea_tch_writei2c(cmd);
	
	TOUCH_READ_DAT(0, (unsigned int)(&data[0]));
	
	value = (data[0] & 0x00ff) << 4;
	value |= ((data[1] & 0x00ff) >> 4);
	value &= 0xfff;
	
#ifndef CONFIG_I2C
	udelay(1);
#endif
	return(value>>1);
}

void tea_tch_openi2c(void)
{
    int ret;
#ifdef CONFIG_I2C
	ret = i2c_add_driver(&ak_i2c_driver);
	if (ret != 0)
		printk("%s: can't add i2c driver", __func__);
#else
	TOUCH_IIC_INIT();
#endif
}

short tea_tch_readi2c(int channel)
{
    short ret;
    unsigned char buf[2];
    int cmd;
    tca_tsc2003_setreadcmd(channel, buf);
    //printk("tea_tch_readi2c, channel = (%x), buf = (%x)\n", channel, buf[0]);
    cmd = (int) buf[0];
    ret = read_ak(cmd);
    return ret;
}

int tea_tch_writei2c(unsigned char command)
{
#ifdef CONFIG_I2C
    i2c_master_send(ak_i2c_client, &command, 1);
#else
    TOUCH_SEND_CMD(0, (unsigned int)(&cmd));
#endif
    udelay(1);

    return 0;
}

static inline void tsc_pen_release(struct tsc2003_ts *ts_data, struct input_dev *dev)
{
    if (ts_data->pen_status != PEN_RELEASE) {
//        printk("PENUP\n");
        ts_data->pen_status = PEN_RELEASE;

        input_report_key(dev, BTN_TOUCH, PEN_RELEASE);
        input_sync(dev);


    }
}

static inline void tsc_pen_pressure(struct tsc2003_ts *ts_data, struct input_dev *dev)
{
    ts_data->pen_status = PEN_DOWN;

    input_report_key(dev, BTN_TOUCH, PEN_DOWN);

#if defined(CONFIG_FB_TCC7901_DEMOBOARD)
        input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
        input_report_abs(dev, ABS_Y, tsc2003_ts->y);
#elif defined(CONFIG_FB_TCC7901_LMS350DF01)
        input_report_abs(dev, ABS_X, Y_AXIS_MAX - tsc2003_ts->y);
        input_report_abs(dev, ABS_Y, X_AXIS_MAX - tsc2003_ts->x);
#elif defined(CONFIG_ARCH_TCC92X)
#if defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01)		
		input_report_abs(dev, ABS_X, Y_AXIS_MAX - tsc2003_ts->y);
		input_report_abs(dev, ABS_Y, X_AXIS_MAX - tsc2003_ts->x);
#elif defined(CONFIG_FB_TCC92XX_WVGA_LMS480KF) || defined(CONFIG_FB_TCC92XX_WVGA_LB070WV6)
		input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
		input_report_abs(dev, ABS_Y, tsc2003_ts->y);	
#elif defined(CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW)		
		input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
		input_report_abs(dev, ABS_Y, tsc2003_ts->y);	
#elif defined(CONFIG_FB_TCC89XX_WXGA_HT121WX2)		
		input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
		input_report_abs(dev, ABS_Y, tsc2003_ts->y);	
#endif		
#endif

#if defined(CONFIG_ARCH_TCC93XX) // for test
        input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
        input_report_abs(dev, ABS_Y, tsc2003_ts->y);
#endif

		
    input_report_key(dev, ABS_PRESSURE, 1);
    input_sync(dev);
    msleep_interruptible(1);   //MCC
//    printk("PEN DOWN %u, %u, %u\n", ts_data->x, ts_data->y, ts_data->pressure);
}
static void ts_fetch_thread(struct work_struct *work)
{
    struct tsc2003_ts* ts_data = container_of(work, struct tsc2003_ts, work_q);
    struct input_dev *dev = tsc2003_ts->input;
    int flag, valid;
   
    ts_data->running = 1;

    tca_tchhw_disableirq();
    if(READ_PENIRQ()){
        tsc_pen_release(ts_data, dev);
        ts_data->running = 0;
        tca_tchhw_enableirq();
    } else {

        flag = tca_getrawdata(&(ts_data->x), &(ts_data->y));
        tca_tch_poweroff();

        valid = ts_data->x | ts_data->y;
        if ((flag == 0) && valid) {
            tsc_pen_pressure(ts_data, dev);
            ts_timer.expires = jiffies + msecs_to_jiffies(50);	  // It is adjustable. //MCC
            add_timer(&ts_timer);
        } else {
            tsc_pen_release(ts_data, dev);
            ts_data->running = 0;
            tca_tchhw_enableirq();
        }
    }

#if 0
    flag = tca_getrawdata(&(ts_data->x), &(ts_data->y));

	tca_tch_poweroff();

    valid = ts_data->x | ts_data->y;
//   printk("valid = %d, flag =%d\n", valid, flag);

    if((flag == 0) && valid){
        ts_data->pen_status = PEN_DOWN;

#if defined(CONFIG_FB_TCC7901_DEMOBOARD)
        input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
        input_report_abs(dev, ABS_Y, tsc2003_ts->y);
#elif defined(CONFIG_FB_TCC7901_LMS350DF01)
        input_report_abs(dev, ABS_X, Y_AXIS_MAX - tsc2003_ts->y);
        input_report_abs(dev, ABS_Y, X_AXIS_MAX - tsc2003_ts->x);
#elif defined(CONFIG_ARCH_TCC92X)
#if defined(CONFIG_FB_TCC92XX_HVGA)		
		input_report_abs(dev, ABS_X, Y_AXIS_MAX - tsc2003_ts->y);
		input_report_abs(dev, ABS_Y, X_AXIS_MAX - tsc2003_ts->x);
#elif defined(CONFIG_FB_TCC92XX_WVGA)		
		input_report_abs(dev, ABS_X, X_AXIS_MAX - tsc2003_ts->x);
		input_report_abs(dev, ABS_Y, tsc2003_ts->y);	
#endif
#endif 
        //input_report_key(dev, ABS_PRESSURE, 1);
        input_report_key(dev, BTN_TOUCH, PEN_DOWN);
        input_sync(dev);

        wake_up_interruptible(&ts_data->wait_q);
//        printk("PEN DOWN %u, %u, %u\n", ts_data->x, ts_data->y, ts_data->pressure);
    }else{
        if (ts_data->pen_status != PEN_RELEASE) {
            wake_up_interruptible(&ts_data->wait_q);
	    //        printk("PENUP\n");
	    ts_data->pen_status = PEN_RELEASE;

	    input_report_key(dev, BTN_TOUCH, PEN_RELEASE);
	    input_sync(dev);
        }

        ts_data->running = 0;
    }
    
    ts_timer.expires = jiffies + 5;
    add_timer(&ts_timer);
#endif

}


static irqreturn_t tsc2003_irq_handler(int irq, void *dev_id)
{
    struct tsc2003_ts *ts_data = (struct tsc2003_ts *)dev_id;

    if (ts_data->running == 0) {
        ts_data->running = 1;
        ts_timer.expires = jiffies + msecs_to_jiffies(50);		 // It is adjustable. //MCC
        add_timer(&ts_timer);
    }

	return IRQ_HANDLED;
}

static void ts_timer_handler(unsigned long data)
{
    if(tsc2003_ts->opened){
        if (schedule_work(&(tsc2003_ts->work_q)) == 0) {
            printk("cannot schedule work !!!\n");
            tsc2003_ts->running = 0;
        }
    }else {
        tca_tchhw_disableirq();
    }
}



#ifdef CONFIG_PM
static int tsc2003ts_suspend(struct platform_device *dev, pm_message_t state)
{
#if 0
	tca_tchhw_disableirq();	
	tca_tsc2003_poweroff();
#endif
    return 0;
}

static int tsc2003ts_resume(struct platform_device *dev)
{
#if 0
	tca_tchhw_enableirq();	
	tca_tsc2003_poweroff();
#endif
    return 0;
}
#else
#define tsc2003ts_suspend     NULL
#define tsc2003ts_resume      NULL
#endif



static void tcc92xx_tsc_early_suspend(struct early_suspend *h);
static void tcc92xx_tsc_late_resume(struct early_suspend *h);

static int __devinit tsc2003ts_probe(struct platform_device *pdev)
{
    struct input_dev *input_dev;
    int err = -ENOMEM;
	struct clk *clk;

    printk("%s\n", __func__);

    tca_tch_init();

    /* INPUT DEVICE */
    tsc2003_ts = kzalloc(sizeof(struct tsc2003_ts), GFP_KERNEL);

    input_dev = input_allocate_device();

    if (!tsc2003_ts || !input_dev)
    {
        printk("ERROR 1\n");
        goto fail1;
    }

    platform_set_drvdata(pdev, tsc2003_ts);


/*
	clk = clk_get(NULL, "i2c2");
	if (clk)
		clk_enable(clk);
	else
		pr_err("tsc2003: can't get i2c2 clock\n");
	tsc2003_ts->clk = clk;
*/

    tsc2003_ts->running = 0;
    tsc2003_ts->opened = 1;
    tsc2003_ts->pen_status = PEN_RELEASE;

    tsc2003_ts->input = input_dev;

    input_dev->name = "tcc-tsc2003-ts";
    input_dev->phys = "tsc2003/input0";
    input_dev->id.version = 0x0001;
    input_dev->dev.parent = &pdev->dev;

    input_dev->evbit[0] = BIT_MASK(EV_KEY) | BIT_MASK(EV_ABS);
    input_dev->keybit[BIT_WORD(BTN_TOUCH)] = BIT_MASK(BTN_TOUCH);
    input_set_abs_params(input_dev, ABS_X, X_AXIS_MIN, X_AXIS_MAX, 0, 0);
    input_set_abs_params(input_dev, ABS_Y, Y_AXIS_MIN, Y_AXIS_MAX, 0, 0);
    /* TODO */
    input_set_abs_params(input_dev, ABS_PRESSURE, 0, 0, 0, 0);

    INIT_WORK(&(tsc2003_ts->work_q), ts_fetch_thread);
    //init_waitqueue_head(&(tsc2003_ts->wait_q));
    mutex_init(&(tsc2003_ts->mutex));

    init_timer(&ts_timer);
    ts_timer.data = (unsigned long)tsc2003_ts;
    ts_timer.function = ts_timer_handler;

    irq_no = tca_tchhw_getirqnum();
	/* Enable touch interrupt */
	if (request_irq(irq_no, tsc2003_irq_handler, IRQF_DISABLED, DEVICE_NAME, tsc2003_ts))
    {
        err = -EBUSY;
        goto fail1;
    }

	tca_tch_poweroff();

    err = input_register_device(tsc2003_ts->input);
    if (err)
        goto fail2;
        
#ifdef CONFIG_HAS_EARLYSUSPEND
	tsc2003_ts->early_suspend.suspend 	= tcc92xx_tsc_early_suspend;
	tsc2003_ts->early_suspend.resume 	= tcc92xx_tsc_late_resume;
	tsc2003_ts->early_suspend.level 	= EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&tsc2003_ts->early_suspend);
#endif

    printk("tsc2003 driver is successfully loaded\n");
	return 0;

fail2: free_irq(irq_no, tsc2003_ts);
fail1: input_free_device(input_dev);
       kfree(tsc2003_ts);
       return err;
}

static int __devexit tsc2003ts_remove(struct platform_device *pdev)
{
    tsc2003_ts->opened = 0;
    tca_tchhw_disableirq();
	del_timer_sync(&ts_timer);
    flush_scheduled_work();
	free_irq(irq_no, tsc2003_ts);
    input_unregister_device(tsc2003_ts->input);
    kfree(tsc2003_ts);

#ifdef CONFIG_I2C
    i2c_del_driver(&ak_i2c_driver);
    kfree(ak_i2c_client);
#endif
	clk_put(tsc2003_ts->clk);
	tsc2003_ts->clk = NULL;

    return 0;
}

static struct platform_driver tsc2003_driver = {
    .driver     = {
        .name   = "tcc-tsc2003-ts",
    },
    .probe      = tsc2003ts_probe,
    .remove     = tsc2003ts_remove,
    .suspend    = tsc2003ts_suspend,
    .resume     = tsc2003ts_resume,
};


#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_tsc_early_suspend(struct early_suspend *h)
{
	tca_tchhw_disableirq();	
	tca_tsc2003_poweroff();
	//tsc2003_driver.suspend;

}
static void tcc92xx_tsc_late_resume(struct early_suspend *h)
{
	tca_tchhw_enableirq();	
	tca_tsc2003_poweroff();
	//tsc2003_driver.resume;
}

#endif


static int __init tsc2003ts_init(void)
{

    int res;
    printk("Telechips TSC2003 Touch Driver\n");
    res =  platform_driver_register(&tsc2003_driver);
    if(res)
    {
        printk("fail : platform driver %s (%d) \n", tsc2003_driver.driver.name, res);
        return res;
    }

    return 0;
}


void __exit tsc2003ts_exit(void)
{
    platform_driver_unregister(&tsc2003_driver);
}

module_init(tsc2003ts_init);
module_exit(tsc2003ts_exit);

MODULE_AUTHOR("linux@telechips.com");
MODULE_DESCRIPTION("Telechips tsc2003 touch driver");
MODULE_LICENSE("GPL");
