
/****************************************************************************
 *   FileName    : tca_rtc.C
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifdef __KERNEL__
#   if defined(_TCC79x_)
#       include <mach/tcc79x.h>
#   elif defined(_TCC83x_)
#       include <mach/tcc83x_virt.h>
#   endif
#else
#	include <windows.h>
#	include <ceddk.h>
#	include <nkintr.h>
#	include <oal.h>
#   if defined(_TCC79x_)
#       include "TCC79x_Virtual.h"
#   elif defined(_TCC83x_)
#       include "TCC83x_Virtual.h"
#   endif
#endif

/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include "tca_rtc.h"

/*****************************************************************************
*
* Defines
*
******************************************************************************/

/*****************************************************************************
*
* structures
*
******************************************************************************/


/*****************************************************************************
*
* Variables
*
******************************************************************************/



/*****************************************************************************
*
* Functions
*
******************************************************************************/


/*****************************************************************************
* Function Name : tca_rtcgettime()
******************************************************************************/
void tca_rtc_gettime(rtctime *pTime)
{
	
	unsigned int data;
	unsigned short seconds;

   // Enable RTC control.
    BITSET(HwRTCCON, Hw1); // RTC Register write enabled
	BITSET(HwINTCON, Hw0); // Interrupt Block Write Enable  

    do {
			data = (HwBCDSEC & 0x7f);
			seconds = FROM_BCD(data);

			data = HwBCDYEAR;
			pTime->wYear = ((data >> 8 ) & 0xFF)*100 + (data & 0xFF); // Start from 1980

			data = (HwBCDMON & 0x1f);
			pTime->wMonth = FROM_BCD(data);

			data = (HwBCDDATE & 0x3f);
			pTime->wDay = FROM_BCD(data);

			data = (HwBCDDAY & 0x07);
			pTime->wDayOfWeek = FROM_BCD(data);

			data = (HwBCDHOUR & 0x3f);
			pTime->wHour = FROM_BCD(data);

			data = (HwBCDMIN & 0x7f);
			pTime->wMinute = FROM_BCD(data);

			data = (HwBCDSEC & 0x7f);
			pTime->wSecond = FROM_BCD(data);

			pTime->wMilliseconds = 0;
    } while (pTime->wSecond != seconds); 

}

/*****************************************************************************
* Function Name : tca_rtcsettime()
******************************************************************************/
void tca_rtc_settime(rtctime *pTime)
{
     // Enable RTC control.
    BITSET(HwRTCCON, Hw0);	//RTC Start Halt
    BITSET(HwRTCCON, Hw4);	//RTC Clock Count Reset
	BITCLR(HwRTCCON, Hw4);	//RTC Clock Count No Reset
	BITSET(HwRTCCON, Hw1);	//RTC Write Enable

	HwBCDSEC	=	TO_BCD(pTime->wSecond);
	HwBCDMIN	=	TO_BCD(pTime->wMinute);
	HwBCDHOUR	=	TO_BCD(pTime->wHour);
	HwBCDDAY	=	TO_BCD(pTime->wDayOfWeek);
	HwBCDDATE	=	TO_BCD(pTime->wDay);
	HwBCDMON	=	TO_BCD(pTime->wMonth);
	HwBCDYEAR	=	TO_BCD(pTime->wYear);
	
    // Disable RTC control.
    BITCLR(HwRTCCON, Hw0);	//RTC Start Run
	BITCLR(HwRTCCON, Hw1);	//RTC Write Disable
}


/*****************************************************************************
* Function Name : tca_rtcsettime()
******************************************************************************/
unsigned int	tca_rtc_checkvalidtime(void)
{

	unsigned	IsNeedMending;
	rtctime		pTimeTest;

	tca_rtc_gettime(&pTimeTest);

	//-------------------------------------------
	// Conversion to Dec and Check validity
	//-------------------------------------------
	IsNeedMending = 0;

	/* Second */
	if (pTimeTest.wSecond > 59)
	{
		IsNeedMending	= 1;
	}

	/* Minute */
	if ( pTimeTest.wMinute > 59 )
	{
		IsNeedMending	= 1;
	}

	/* Hour */
	if ( pTimeTest.wHour > 23 )
	{
		IsNeedMending	= 1;
	}

	/* Date */ // 1~31
	if ( ( pTimeTest.wDay < 1 ) || ( pTimeTest.wDay > 31 ) )
	{
		IsNeedMending	= 1;
	}

	/* Day */ // Sunday ~ Saturday
	if ( pTimeTest.wDayOfWeek > 6 )
	{
		IsNeedMending	= 1;
	}

	/* Month */
	if ( ( pTimeTest.wMonth < 1 ) || ( pTimeTest.wMonth > 12 ) )
	{
		IsNeedMending	= 1;
	}

	/* Year */
	if ( ( pTimeTest.wYear < RTC_YEAR_DATUM ) || ( (pTimeTest.wYear ) > 2030) )
	{
		IsNeedMending	= 1;
	}

	return	IsNeedMending;

}

