/*
 * linux/drivers/serial/tcc_serial.c
 *
 * Author:  <linux@telechips.com>
 * Created: Feb 10, 2009
 * Description: Driver for onboard UARTs on the Telechips TCC Series, CORE = 16C550
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/sysrq.h>
#include <linux/console.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial_core.h>
#include <linux/serial.h>
#include <linux/delay.h>
#include <linux/clk.h>
#include <linux/serial_reg.h>
#include <linux/interrupt.h>


#include <asm/io.h>
#include <asm/irq.h>

#include <mach/hardware.h>
#include <mach/irqs.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#if CONFIG_ARCH_TCC92X
#include <mach/bsp.h>
/*
#include <mach/TCC89x_Physical.h>
#include <mach/TCC89x_Structures.h>
#include <mach/globals.h>
#include <mach/tca_ckc.h>
*/
#define DRV_NAME "tcc9200-uart"
#define CONSOLE_PORT 0
#define	CONSOLE_BASE	*(volatile unsigned long *)0xF0532000
#endif

#define BAUDRATE  115200

#include <mach/tca_ckc.h>
//#include "tca_tccserial.h"

#if 0
#ifdef CONFIG_SERIAL_TCC_DMA
#include <linux/dma-mapping.h>

#define DMA_PORT	1
#define DMA_NUM		6
#define DMA_BUF_SIZE	FIFOSIZE
#endif
#endif

#define TIMEOUT     jiffies_to_msecs(5000)


/* UARTx_IER Masks */
#define UART_IER_ERXI   UART_IER_RDI 
#define UART_IER_ETXI   UART_IER_THRI 
#define UART_IER_ELSI   UART_IER_RLSI 

#define UART_FCR_FE     UART_FCR_ENABLE_FIFO	/* Enable the FIFO */
#define UART_FCR_RXFR   UART_FCR_CLEAR_RCVR	/* Clear the RCVR FIFO */
#define UART_FCR_TXFR   UART_FCR_CLEAR_XMIT	/* Clear the XMIT FIFO */
#define UART_FCR_DRTE   UART_FCR_DMA_SELECT	/* For DMA applications */


#define UART_FCR_TXT1   0x30
#define UART_FCR_TXT4   0x20
#define UART_FCR_TXT8   0x10
#define UART_FCR_TXT16  0x00
#define UART_FCR_RXT1   0x00
#define UART_FCR_RXT4   0x40
#define UART_FCR_RXT8   0x80
#define UART_FCR_RXT14  0xC0

#define OFFSET_THR    0x00	/* Transmit Holding register            */
#define OFFSET_RBR    0x00	/* Receive Buffer register              */
#define OFFSET_DLL    0x00	/* Divisor Latch (Low-Byte)             */
#define OFFSET_IER    0x04	/* Interrupt Enable Register            */
#define OFFSET_DLM    0x04	/* Divisor Latch (High-Byte)            */
#define OFFSET_IIR    0x08	/* Interrupt Identification Register    */
#define OFFSET_FCR    0x08	/* FIFO Control Register                */
#define OFFSET_LCR    0x0C	/* Line Control Register                */
#define OFFSET_MCR    0x10	/* Modem Control Register               */
#define OFFSET_LSR    0x14	/* Line Status Register                 */
#define OFFSET_MSR    0x18	/* Modem Status Register                */
#define OFFSET_SCR    0x1C	/* SCR Scratch Register                 */
#define OFFSET_AFT    0x20	/* AFC Trigger Level Register           */
#define OFFSET_UCR    0x24	/* UART Control Register                */

#define portaddr(port, reg) ((port)->membase + (reg))

#define rd_regb(port, reg) (__raw_readb(portaddr(port, reg)))
#define rd_regl(port, reg) (__raw_readl(portaddr(port, reg)))

#define wr_regl(port, reg, val) \
    do { __raw_writel(val, portaddr(port, reg)); } while(0)
#define wr_regb(port, reg, val) \
    do { __raw_writeb(val, portaddr(port, reg)); } while(0)


//static struct resource *tcc_uart_mem;
//static void __iomem *tcc_uart_base;


/* structures */
/*
struct tcc_uart_info {
	char			*name;
//	unsigned int	type;
//	unsigned int	fifosize;
};
*/

struct tcc_reg_info {
	unsigned long	bDLL;
	unsigned long	bIER;
	unsigned long	bDLM;
	unsigned long	bLCR;
	unsigned long	bMCR;
	unsigned long	bSCR;
	unsigned long	bFCR;
	unsigned long	bMSR;
	unsigned long	bAFT;
	unsigned long	bUCR;
};

struct tcc_uart_port {
    unsigned int            lsr_break_flag;

//	struct tcc_uart_info	*info;
	struct uart_port		port;
    wait_queue_head_t wait_q;
    int                     fifosize;
    char                    *name;
	struct tcc_reg_info		reg;	// for suspend/resume
    unsigned int            ier;
    unsigned int            lcr;
    unsigned int            mcr;
#ifdef CONFIG_HAS_EARLYSUSPEND
    struct early_suspend early_suspend;
#endif	
 
};

/* configuration defines */

#if 1
//#define dbg(x...) printk(KERN_DEBUG "tcc uart: ");
#define dbg printk
#else /* no debug */
#define dbg(x...) do {} while(0)
#endif

/* UART name and device definitions */

#define TCC_SERIAL_NAME	"ttySAC"
#define TCC_SERIAL_MAJOR	204
#define TCC_SERIAL_MINOR	64

/* conversion functions */

#define tcc_dev_to_port(__dev) (struct uart_port *)dev_get_drvdata(__dev)

/* Number of serial ports */
#define NR_PORTS (6)

#define FIFOSIZE (16) /* defend on FCR */

#define tx_enabled(port)	((port)->unused[0])
#define rx_enabled(port)	((port)->unused[1])
#define port_used(port)		((port)->unused1)

static void tcc_serial_console_putchar(struct uart_port *port, int ch);
extern void tcc_serial_uart_putchar(struct uart_port *port, int ch);


int tcc_serial_init_ports(struct tcc_uart_port *ourport, struct platform_device *pdev);

static struct uart_ops tcc_serial_ops;

static struct tcc_uart_port tcc_serial_ports[NR_PORTS] = {
	[0] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[0].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART0,
			.uartclk	= 0, 
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 0,
		},
        .name = "uart0"
	},
	[1] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[1].port.lock),
			.iotype		= UPIO_MEM,
			.uartclk	= 0,
			.irq		= INT_UART1,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 1,
		},
        .name = "uart1"
	},
	[2] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[2].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART2,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 2,
		},
        .name = "uart2"
	},
	[3] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[3].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART3,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 3,
		},
        .name = "uart3"
	},
	[4] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[4].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART4,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 4,
		},
        .name = "uart4"
	},
	[5] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[5].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART5,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 5,
		},
        .name = "uart5"
	},

};

/*
static inline struct tcc_uart_port *to_ourport(struct uart_port *port)
{
	return container_of(port, struct tcc_uart_port, port);
}
*/

/* translate a port to the device name */
/*
static inline const char *tcc_serial_portname(struct uart_port *port)
{
	return to_platform_device(port->dev)->name;
}
*/


static void tcc_serial_stop_tx(struct uart_port *port)
{
    //struct circ_buf *xmit = &port->info->xmit;

    //while(!(rd_regl(port, OFFSET_LSR)) & UART_LSR_TEMT)
    //    cpu_relax();
//        continue;

    //wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & ~UART_IER_ETXI));
    if(tcc_serial_ports[port->line].ier & UART_IER_ETXI){
        tcc_serial_ports[port->line].ier &= ~UART_IER_ETXI;
        wr_regl(port, OFFSET_IER, tcc_serial_ports[port->line].ier);
    }

    /*
    if(tx_enabled(port)){
        disable_irq(port->irq);
        tx_enabled(port) = 0;
    }
    */
}


static void tcc_serial_start_tx(struct uart_port *port)
{
//	dbg("%s\n", __func__);
	unsigned long flags;
	struct circ_buf *xmit = &port->info->xmit;	

	//local_irq_save(flags);
	spin_lock_irqsave(port->lock, flags);
		
    if (!(tcc_serial_ports[port->line].ier & UART_IER_ETXI)) 
	{		
		// soon__,
		#if 1
		if(port->line == 0 || port->line == 1)
		{			
			//if(port->line == 0)
			//	udelay(100);
			tcc_serial_uart_putchar(port, xmit->buf[xmit->tail]);
		    //wr_regb(port, OFFSET_THR, xmit->buf[xmit->tail]);
		    xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
		    port->icount.tx++;
			
		    if(uart_circ_empty(xmit))
		    {
        		tcc_serial_stop_tx(port);
		    }
			else
			{
        		tcc_serial_ports[port->line].ier |= UART_IER_ETXI;
        		wr_regl(port, OFFSET_IER, tcc_serial_ports[port->line].ier);
			}			
		}
		else
		#endif
		{
        	tcc_serial_ports[port->line].ier |= UART_IER_ETXI;
        	wr_regl(port, OFFSET_IER, tcc_serial_ports[port->line].ier);
		}
		
	}
	
	//local_irq_restore(flags);
	spin_unlock_irqrestore(&port->lock, flags);
}


static void tcc_serial_stop_rx(struct uart_port *port)
{
//	dbg("%s\n", __func__);

    tcc_serial_ports[port->line].ier &= ~UART_IER_ELSI;
    port->read_status_mask &= ~UART_LSR_DR;
    wr_regl(port, OFFSET_IER,tcc_serial_ports[port->line].ier);
}

static void tcc_serial_enable_ms(struct uart_port *port)
{
    /*
    tcc_serial_ports[port->line].ier |= UART_IER_MSI;
    wr_regl(port, OFFSET_IER, tcc_serial_ports[port->line].ier);
    */
}


#if 1
static inline void receive_chars(struct uart_port *port, int *status)
{
//	dbg("%s\n", __func__);
	struct tty_struct *tty;
	unsigned int ch = 0;
	unsigned flag;
    int max_count = 256;

    tty = port->info->port.tty;

    do {
        ch = rd_regl(port, OFFSET_RBR); 
        flag = TTY_NORMAL;
        port->icount.rx++;
    
        if (unlikely(*status & (UART_LSR_BI | UART_LSR_PE |
                                UART_LSR_FE | UART_LSR_OE))) {
            /*
             * For statistics only
             */

            if (*status & UART_LSR_BI) {
                dbg("break!\n");
                *status &= ~(UART_LSR_FE | UART_LSR_PE);
                port->icount.brk++;
                /*
                 * We do the SysRQ and SAK checking
                 * here because otherwise the break
                 * may get masked by ignore_status_mask
                 * or read_status_mask.
                 */
                if (uart_handle_break(port)) 
                    goto ignore_char;
            } else if (*status & UART_LSR_PE)
                port->icount.parity++;
            else if (*status & UART_LSR_FE)
                port->icount.frame++;
            if (*status & UART_LSR_OE)
                port->icount.overrun++;

            /*
             * Mask off conditions which should be ignored.
             */
            *status &= port->read_status_mask;

#ifdef CONFIG_SERIAL_TCC_CONSOLE
            if (port->line == port->cons->index) {
                /* Recover the break flag from console xmit */
                *status |= tcc_serial_ports[port->line].lsr_break_flag;
                tcc_serial_ports[port->line].lsr_break_flag = 0;
            }
#endif

            if (*status & UART_LSR_BI) {
                flag = TTY_BREAK;
            } else if (*status & UART_LSR_PE)
                flag = TTY_PARITY;
            else if (*status & UART_LSR_FE) 
                flag = TTY_FRAME;
        }

        if (uart_handle_sysrq_char(port, ch))
            goto ignore_char;

        /* put the received char into UART buffer */
        uart_insert_char(port, *status, UART_LSR_OE, ch, flag);
ignore_char:
        *status = rd_regl(port, OFFSET_LSR);
    } while ((*status & UART_LSR_DR) && (max_count-- > 0));
    tty_flip_buffer_push(tty);
}
#endif

static void transmit_chars(struct uart_port *port)
{
	struct circ_buf *xmit = &port->info->xmit;
    int count;
	//dbg("%s\n", __func__);

    /*
	while (!(rd_regl(port, OFFSET_LSR) & UART_LSR_THRE))
		barrier();
        */

	if (port->x_char) {
        wr_regb(port, OFFSET_THR, port->x_char); 
        port->icount.tx++;
        port->x_char = 0;
        return;
	}
    
    if (uart_circ_empty(xmit) || uart_tx_stopped(port)) {
        tcc_serial_stop_tx(port);
        return;
    }

    count = port->fifosize;
    do {
        //printk("count = %d, THR = %c \n", count, xmit->buf[xmit->tail]);
        //udelay(200);
       	//tcc_serial_uart_putchar(port, xmit->buf[xmit->tail]);
        wr_regb(port, OFFSET_THR, xmit->buf[xmit->tail]);
        xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
        port->icount.tx++;
        if (uart_circ_empty(xmit))
            break;
    } while (--count > 0);

    if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS)
        uart_write_wakeup(port);

    if (uart_circ_empty(xmit))
        tcc_serial_stop_tx(port);
}

// soon__,
void debug_call_temp(void)
{
	struct uart_port *port;
	struct circ_buf *xmit = &port->info->xmit;
	int temp;
	
	uart_circ_empty(xmit);
	receive_chars(port, &temp); 
	transmit_chars(port);
	tcc_serial_stop_tx(port);
	tcc_serial_start_tx(port);
}

static inline void check_modem_status(struct uart_port *port)
{
    int status;

    status = rd_regl(port, OFFSET_MSR);

/*
    if ((status & UART_MSR_ANY_DELTA) == 0)
        return;

    if (status & UART_MSR_TERI)
        up->port.icount.rng++;
    if (status & UART_MSR_DDSR)
        up->port.icount.dsr++;
    if (status & UART_MSR_DDCD)
        uart_handle_dcd_change(&up->port, status & UART_MSR_DCD);
*/
    if (status & UART_MSR_DCTS)
        uart_handle_cts_change(port, status & UART_MSR_CTS);

    wake_up_interruptible(&port->info->delta_msr_wait);
}


static irqreturn_t tcc_serial_interrupt(int irq, void *dev_id)
{
	struct uart_port *port = dev_id;
    unsigned int iir, lsr;
	unsigned long flags;

	spin_lock_irqsave(port->lock, flags);
	//spin_lock(port->lock);	// soon__,
	
    iir = rd_regl(port, OFFSET_IIR);
    if (iir & UART_IIR_NO_INT)
    {
		spin_unlock_irqrestore(&port->lock, flags);
		//spin_unlock(port->lock);	// soon__,
        return IRQ_NONE;
    }

    lsr = rd_regl(port, OFFSET_LSR);
    if (lsr & UART_LSR_DR)
        receive_chars(port, &lsr);

	if(port->line != 1 && port->line != 0)	// soon__,
    	check_modem_status(port);

    if (lsr & UART_LSR_THRE)
		transmit_chars(port);

	spin_unlock_irqrestore(&port->lock, flags);
	//spin_unlock(port->lock);	// soon__,
	
    return IRQ_HANDLED;
/*
    if (iir_data & 0x02 || iir_data & 0x06) {
	
    spin_lock(port->lock);

    iir_data = (iir_data & 0x0E) >> 1;
    if (iir_data & 0x02 || iir_data & 0x06) {
        ch = rd_regl(port, OFFSET_RBR);
        rx_flag = 1;
    } else if (iir_data & 0x01) {
        t_ier = rd_regl(port, OFFSET_IER);
        wr_regl(port, OFFSET_IER, (t_ier & ~UART_IER_ETXI));
        wake_up(&(tcc_serial_ports[port->line].wait_q));
    }
*/
}


static unsigned int tcc_serial_tx_empty(struct uart_port *port)
{
	unsigned int lsr;
//	dbg("%s\n", __func__);

	lsr = rd_regl(port, OFFSET_LSR);
	if (lsr & UART_LSR_TEMT)
		return TIOCSER_TEMT;
	else
		return 0;

}

static unsigned int tcc_serial_get_mctrl(struct uart_port *port)
{
#if 0
    unsigned char status;
    unsigned int ret;

	dbg("%s\n", __func__);

    status = rd_regb(port, OFFSET_MSR);

    ret = 0;
    /*
    if (status & UART_MSR_DCD)
        ret |= TIOCM_CAR;
    if (status & UART_MSR_RI)
        ret |= TIOCM_RNG;
    if (status & UART_MSR_DSR)
        ret |= TIOCM_DSR;
        */
    if (status & UART_MSR_CTS)
        ret |= TIOCM_CTS;
    return ret;
#else

    return TIOCM_CTS | TIOCM_DSR | TIOCM_CAR;
#endif
}

static void tcc_serial_set_mctrl(struct uart_port *port, unsigned int mctrl)
{
//	dbg("%s\n", __func__);
#if 0
    unsigned char mcr = 0;

	/* todo - possibly remove AFC and do manual CTS */

    if (mctrl & TIOCM_RTS)
        mcr |= UART_MCR_RTS;

    if (mctrl & TIOCM_LOOP)
        mcr |= UART_MCR_LOOP;

    mcr |= tcc_serial_ports[port->line].mcr;

    wr_regl(port, OFFSET_MCR, mcr);
#endif
}

static void tcc_serial_break_ctl(struct uart_port *port, int break_state)
{
    unsigned long flags;
	//dbg("%s\n", __func__);

#if 0
	spin_lock_irqsave(&port->lock, flags);
    if (break_state == -1)
        tcc_serial_ports[port->line].lcr |= UART_LCR_SBC;
    else
        tcc_serial_ports[port->line].lcr &= ~UART_LCR_SBC;
    wr_regl(port, OFFSET_LCR, tcc_serial_ports[port->line].lcr);
    spin_unlock_irqrestore(&port->lock, flags);
#endif
}

static void tcc_serial_shutdown(struct uart_port *port)
{
    unsigned long flags;

    wr_regl(port, OFFSET_IER, 0x0);
    free_irq(port->irq, port);

	spin_lock_irqsave(&port->lock, flags);
    port->mctrl &= ~TIOCM_OUT2;
    tcc_serial_set_mctrl(port, port->mctrl);
    spin_unlock_irqrestore(&port->lock, flags);

    wr_regl(port, OFFSET_SCR, rd_regl(port, OFFSET_LCR) & ~UART_LCR_SBC);
    wr_regl(port, OFFSET_FCR, UART_FCR_FE | UART_FCR_RXFR | UART_FCR_TXFR);
    wr_regl(port, OFFSET_FCR, 0x0);
    
	port_used(port) = 0;	// for suspend/resume
}

/* while application opening the console device, this function will invoked
 * This function will initialize the interrupt handling 
 */
int bt_working_first = 0;
static int tcc_serial_startup(struct uart_port *port)
{
	int retval=0;
    unsigned long flags;
	unsigned int lcr;

    volatile PPIC tPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);	
	dbg("%s:%d\n", __func__, port->line);

    volatile PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
	volatile PUARTPORTMUX pUARTPortMux = (volatile PUARTPORTMUX)tcc_p2v(HwUARTPORTMUX_BASE);
// FIXME!  //MCC
#if defined(CONFIG_MACH_TCC9200S)
	pUARTPortMux->CHSEL = 0x00013245;	// soon__,
#else
	pUARTPortMux->CHSEL = 0x00513240;	// TCC9200
#endif

	if(bt_working_first)
	{
		//dbg("[BT] Rest CSR & Port configuration..\n");		
		// Reset BT,
		pGPIO->GPDCD0 &= ~(Hw22|Hw23);
		pGPIO->GPDCD0 |= Hw22;
		msleep(10);
		pGPIO->GPDFN1 &= 0xFFFF0FFF;
		pGPIO->GPDEN |= Hw11;
		pGPIO->GPDDAT |= Hw11;
		msleep(30);
		pGPIO->GPDDAT &= ~Hw11;
		msleep(500);
		pGPIO->GPDDAT |= Hw11;

		bt_working_first = 0;
	}

    //tca_serial_portinit(port->line);
    if(port->line == 0)
    {
        tca_ckc_setiobus(RB_UARTCONTROLLER0 , 1);
        tca_ckc_setperi(PERI_UART0,ENABLE,480000,PCDIRECTPLL1);  // Console:115.2K BT:3M
		//tca_ckc_setperi(PERI_UART0,ENABLE,147456,PCDIRECTPLL1);  // Console:115.2K BT:921.6K
        pGPIO->GPEEN    |= (Hw1|Hw0);
        pGPIO->GPEFN0   |= (Hw4|Hw0); // UTXD0, URXD0
		
    } 
	else if(port->line == 1) 
    {
        tca_ckc_setiobus(RB_UARTCONTROLLER1 , 1);		
		//tca_ckc_setperi(PERI_UART1,ENABLE,480000,PCDIRECTPLL1);  // Console:115.2K BT:3M
        tca_ckc_setperi(PERI_UART1,ENABLE,147456,PCDIRECTPLL1);  // Console:115.2K BT:921.6K

	    //pGPIO->GPEEN    |= (Hw4|Hw5|Hw6|Hw7);
	    //pGPIO->GPEFN0   |= (Hw28|Hw24|Hw20|Hw16); // UTXD1, URXD1,

		pGPIO->GPDFN1   |= (Hw28|Hw24|Hw20);
	    pGPIO->GPDFN2   |= (Hw0);		
		//msleep(100);		

		pGPIO->GPDCD0 &= ~(Hw26|Hw27|Hw28|Hw29|Hw30|Hw31);
		pGPIO->GPDCD0 |= (Hw27);
		pGPIO->GPDCD1 &= ~(Hw0|Hw1);
		pGPIO->GPDCD1 |= Hw1;		
		
		//wr_regl(port, OFFSET_MCR, UART_MCR_RTS);
		//tcc_serial_ports[port->line].mcr |= UART_MCR_RTS;
    }
#if 0
    else if(port->line == 2) { }
    else if(port->line == 3) { }
    else if(port->line == 4) { }
    else if(port->line == 5) { }
    else if(port->line == 6) { } 
#endif


	#if 0	// soon__,
    if(port->line == 1) 
        tcc_serial_ports[port->line].mcr |= UART_MCR_AFE;
    else
        tcc_serial_ports[port->line].mcr = 0;
	#endif
    
    retval = request_irq(port->irq, tcc_serial_interrupt, IRQF_SHARED, tcc_serial_ports[port->line].name , port);
    if (retval)
        return retval;

    dbg("request serial irq:%d,retval:%d\n", port->irq,retval);
	
	port_used(port) = 1;	// for suspend/resume

    /*
     * Clear the FIFO buffers and disable them.
     * (they will be reenabled in set_termios())
     */

	// soon__,
	#if 0
    lcr = rd_regl(port, OFFSET_LCR);
    wr_regl(port, OFFSET_LCR, (lcr | Hw7));	
    //wr_regl(port, OFFSET_FCR, UART_FCR_FE);
    wr_regl(port, OFFSET_FCR, UART_FCR_FE | UART_FCR_RXFR | UART_FCR_TXFR | UART_FCR_TXT16);
    //wr_regl(port, OFFSET_FCR, 0x0);
	wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));
	#endif

	spin_lock_irqsave(&port->lock, flags);	// soon__,
	
    tcc_serial_ports[port->line].fifosize = FIFOSIZE;
    tcc_serial_ports[port->line].reg.bFCR = 0x07;		// for resume restore

    /*
     * Clear the interrupt registers.
     */
    rd_regl(port, OFFSET_LSR);
    rd_regl(port, OFFSET_RBR);
    rd_regl(port, OFFSET_IIR);
    rd_regl(port, OFFSET_MSR);

    /*
     * Now, initialize the UART
     */
    wr_regl(port, OFFSET_LCR, UART_LCR_WLEN8);
    //lcr = rd_regl(port, OFFSET_LCR);
    //wr_regl(port, OFFSET_LCR, (lcr | Hw7));

    //spin_lock_irqsave(&port->lock, flags);
	
    port->mctrl |= TIOCM_OUT2;
    tcc_serial_set_mctrl(port, port->mctrl);
    spin_unlock_irqrestore(&port->lock, flags);

    //wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));
    /*
     * Finally, enable interrupts.  Note: Modem status interrupts
     * are set via set_termios(), which will be occurring imminently
     * anyway, so we don't enable them here.
     */
    tcc_serial_ports[port->line].ier = UART_IER_ELSI | UART_IER_ERXI ;
    wr_regl(port, OFFSET_IER, tcc_serial_ports[port->line].ier);

    /*
     * And clear the interrupt registers again for luck.
     */
    rd_regl(port, OFFSET_LSR);
    rd_regl(port, OFFSET_RBR);
    rd_regl(port, OFFSET_IIR);
    rd_regl(port, OFFSET_MSR);

    //tca_serial_intrdone();
    tPIC->INTMSK1   |= Hw15;

    return 0;

#if 0
    wr_regl(port, OFFSET_IER, 0x0);
    
    lcr = rd_regl(port, OFFSET_LCR);
    wr_regl(port, OFFSET_LCR, (lcr | Hw7));
    
    wr_regl(port, OFFSET_FCR, 0x07);
    tcc_serial_ports[port->line].fifosize = FIFOSIZE;
    tcc_serial_ports[port->line].reg.bFCR = 0x07;		// for resume restore

    wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));

    rd_regl(port, OFFSET_IIR);

    //wr_regl(port, OFFSET_IER, ERXI);
    wr_regl(port, OFFSET_IER, ERXI | ETXI);
    

    tca_serial_intrdone();

	return retval;
#endif
}

/* power power management control */

static void tcc_serial_pm(struct uart_port *port, unsigned int level, unsigned int old)
{
}

static void tcc_serial_set_termios(struct uart_port *port,
				       struct ktermios *termios,
				       struct ktermios *old)
{
	unsigned long flags;
	unsigned int baud, quot;
	unsigned int lcr = 0;
	unsigned int fcr = 0;
    unsigned int uart_clk = 0;

	switch (termios->c_cflag & CSIZE) {
	case CS5:
		lcr = UART_LCR_WLEN5;
		break;
	case CS6:
		lcr = UART_LCR_WLEN6;
		break;
	case CS7:
		lcr = UART_LCR_WLEN7;
		break;
	case CS8:
	default:
		lcr = UART_LCR_WLEN8;
		break;
	}

	/* preserve original lcon IR settings */
	if (termios->c_cflag & CSTOPB)
		lcr |= UART_LCR_STOP; 
	if (termios->c_cflag & PARENB) 
        lcr |= UART_LCR_PARITY;
    if (!(termios->c_cflag & PARODD)) 
        lcr |= UART_LCR_EPAR;

	// soon__,
	//if(port->line == 0 || port->line == 1)
	//	lcr |= UART_LCR_STOP;
	//if(port->line == 1)
	//	lcr |= (UART_LCR_PARITY | UART_LCR_EPAR);

	/*
	 * Ask the core to calculate the divisor for us.
	 */
	//baud = uart_get_baud_rate(port, termios, old, 0, port->uartclk/16);
	baud = uart_get_baud_rate(port, termios, old, 0, 3500000);

    uart_clk = (unsigned int)tca_ckc_getperi(port->line + PERI_UART0);

    quot = (uart_clk * 100)/(16*baud);

    dbg("baud:%d, uart_clk:%d, quot:%d\n", baud, uart_clk, quot);

	spin_lock_irqsave(&port->lock, flags);

    /* UUE */
	/*
	 * Update the per-port timeout.
	 */
	uart_update_timeout(port, termios->c_cflag, baud);

    port->read_status_mask = UART_LSR_OE | UART_LSR_THRE | UART_LSR_DR;    
    if (termios->c_iflag & INPCK)
        port->read_status_mask |= (UART_LSR_FE | UART_LSR_PE);         
    if (termios->c_iflag & (BRKINT | PARMRK))   
        port->read_status_mask |= UART_LSR_BI;  
	/*
	 * Which character status flags should we ignore?
	 */
	port->ignore_status_mask = 0;
	if (termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= UART_LSR_FE | UART_LSR_PE;
	if (termios->c_iflag & IGNBRK){
        port->ignore_status_mask |= UART_LSR_BI;   
        /*    
         * If we're ignoring parity and break indicators,  
         * ignore overruns too (for real raw support).      
         */              
        if (termios->c_iflag & IGNPAR)         
            port->ignore_status_mask |= UART_LSR_OE;
    }
	/*
	 * Ignore all characters if CREAD is not set.
	 */
	if ((termios->c_cflag & CREAD) == 0)
		port->ignore_status_mask |= UART_LSR_DR;


    /* Check CTSRTS */
    if(termios->c_cflag & CRTSCTS)
    {
        tcc_serial_ports[port->line].mcr |= UART_MCR_AFE; /* Hw5|Hw1 */
		wr_regl(port, OFFSET_AFT, 0x48);
    }
    else 
    {
        tcc_serial_ports[port->line].mcr &= ~UART_MCR_AFE;
		wr_regl(port, OFFSET_AFT, 0x00);
    }

	tcc_serial_ports[port->line].mcr |= UART_MCR_RTS; // RTS deassert
	
    wr_regl(port, OFFSET_MCR, tcc_serial_ports[port->line].mcr);
    
    /*
	do {
		lsr = rd_regl(port, OFFSET_LSR);
	} while (!(lsr & UART_LSR_TEMT));
    */

	dbg("setting lcr to %08x, brddiv to %d\n", lcr, quot);
    wr_regl(port, OFFSET_LCR, (lcr | UART_LCR_DLAB));
    if(quot > 0xFF) {
        wr_regl(port, OFFSET_DLL, 0xff);
        wr_regl(port, OFFSET_DLM, quot >> 8);
    }else if (quot > 0) {
        wr_regl(port, OFFSET_DLL, quot);
        wr_regl(port, OFFSET_DLM, 0x0);
    }
    wr_regl(port, OFFSET_LCR, lcr & (~UART_LCR_DLAB));
    tcc_serial_ports[port->line].lcr = lcr;

    //fcr = UART_FCR_FE | UART_FCR_TXT16;	// soon__, remove
    tcc_serial_set_mctrl(port, port->mctrl);
    //wr_regl(port, OFFSET_FCR, fcr);		// soon__, remove
    //wr_regl(port, OFFSET_FCR, (rd_regl(port, OFFSET_FCR) & ~Hw4 & ~Hw5 ));

	// soon__,
    lcr = rd_regl(port, OFFSET_LCR);
    wr_regl(port, OFFSET_LCR, (lcr | Hw7));
    wr_regl(port, OFFSET_FCR, (UART_FCR_FE | UART_FCR_RXFR | UART_FCR_TXFR | UART_FCR_TXT16));
	wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));	

	spin_unlock_irqrestore(&port->lock, flags);
}

static const char *tcc_serial_type(struct uart_port *port)
{
    return tcc_serial_ports[port->line].name;
}


#define MAP_SIZE (0x100)

static void tcc_serial_release_port(struct uart_port *port)
{
    /* TODO */
	//release_mem_region(port->mapbase, MAP_SIZE);
}

static int tcc_serial_request_port(struct uart_port *port)
{
    return 0;
    /*
	return request_mem_region(port->mapbase, MAP_SIZE, "tcc7901") ? 0 : -EBUSY;
    */
}

static void tcc_serial_config_port(struct uart_port *port, int flags)
{
	if (flags & UART_CONFIG_TYPE && tcc_serial_request_port(port) == 0)
        port->type = PORT_TCC;
}

/*
 * verify the new serial_struct (for TIOCSSERIAL).
 */
static int
tcc_serial_verify_port(struct uart_port *port, struct serial_struct *ser)
{
	if (ser->type != PORT_UNKNOWN && ser->type != PORT_TCC)
		return -EINVAL;
	return 0;
}


#ifdef CONFIG_SERIAL_TCC_CONSOLE

static struct console tcc_serial_console;

#define TCC_SERIAL_CONSOLE &tcc_serial_console
#else
#define TCC_SERIAL_CONSOLE NULL
#endif

static struct uart_ops tcc_serial_ops = {
	.pm				= tcc_serial_pm,
	.tx_empty		= tcc_serial_tx_empty,
	.get_mctrl		= tcc_serial_get_mctrl,
	.set_mctrl		= tcc_serial_set_mctrl,
	.stop_tx		= tcc_serial_stop_tx,
	.start_tx		= tcc_serial_start_tx,
	.stop_rx		= tcc_serial_stop_rx,
	.enable_ms		= tcc_serial_enable_ms,
	.break_ctl		= tcc_serial_break_ctl,
	.startup		= tcc_serial_startup,
	.shutdown		= tcc_serial_shutdown,
	.set_termios	= tcc_serial_set_termios,
	.type			= tcc_serial_type,
	.release_port	= tcc_serial_release_port,
	.request_port	= tcc_serial_request_port,
	.config_port	= tcc_serial_config_port,
	.verify_port	= tcc_serial_verify_port,
};


static struct uart_driver tcc_uart_drv = {
	.owner		= THIS_MODULE,
	.dev_name	= "tcc-uart",
	.nr		    = NR_PORTS,
	.cons		= TCC_SERIAL_CONSOLE,
	.driver_name    = TCC_SERIAL_NAME,
	.major		= TCC_SERIAL_MAJOR,
	.minor		= TCC_SERIAL_MINOR,
};


/*  initialise  serial port information */
/*
static struct tcc_uart_info tcc_uart_inf = {
	.name		    = "Telechips UART",
	.type	    	= PORT_TCC,
	.fifosize   	= FIFOSIZE,
};
*/

static int probe_index = 0;
#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_serial_early_suspend(struct early_suspend *h);
static void tcc92xx_serial_late_resume(struct early_suspend *h);
#endif

static int tcc_uart_probe(struct platform_device *dev)
{
	int ret;
    struct tcc_uart_port *ourport;
    //pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);
    volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	dbg("%s\n", __func__);
    
    //tca_serial_intrinit();
    pPIC->SEL1      |= Hw15;
    pPIC->INTMSK1   |= Hw15;
    pPIC->MODE1     |= Hw15;    // Level trigger

    ourport = &tcc_serial_ports[probe_index];

    probe_index++;

    ret = tcc_serial_init_ports(ourport, dev);

    //init_waitqueue_head(&(ourport->wait_q));

    if (ret < 0){
        dbg("tcc_serial_init_ports failure\n");
        goto probe_err;
    }

    ret = uart_add_one_port(&tcc_uart_drv, &ourport->port);

    if (ret){
        dbg("uart_add_one_port failure\n");
        return ret;
    }

    platform_set_drvdata(dev, &ourport->port);
#ifdef CONFIG_HAS_EARLYSUSPEND
	ourport->early_suspend.suspend = tcc92xx_serial_early_suspend;
	ourport->early_suspend.resume  = tcc92xx_serial_late_resume;
	ourport->early_suspend.level   = EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&ourport->early_suspend);
#endif    
	return ret;

probe_err:
    dbg("probe_err\n");
	return ret;
}


static int tcc_serial_remove(struct platform_device *dev)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);

	if (port)
		uart_remove_one_port(&tcc_uart_drv, port);

	return 0;
}

/* UART power management code */

#ifdef CONFIG_PM
/*-------------------------------------------------
 * TODO: handling DMA_PORT suspend/resume (TCC79X)
 *       DMA stop and start ...
 *-------------------------------------------------*/
static int tcc_serial_suspend(struct platform_device *dev, pm_message_t state)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;

	if (port && port_used(port)) {
		port->suspended = 1;

		// disable interrupt
		wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & (~UART_IER_ELSI)));

		reg->bLCR = rd_regl(port, OFFSET_LCR);

		// DLAB = 0
		wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
		reg->bIER = rd_regl(port, OFFSET_IER);

		// DLAB = 1
		wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
//		reg->bFCR = 0x07;						// set in tcc_serial_startup()			
		reg->bDLL = rd_regl(port, OFFSET_DLL);
		reg->bDLM = rd_regl(port, OFFSET_DLM);

		reg->bMCR = rd_regl(port, OFFSET_MCR);
		reg->bAFT = rd_regl(port, OFFSET_AFT);
		reg->bUCR = rd_regl(port, OFFSET_UCR);
	}
	
	return 0;
}

static int tcc_serial_resume(struct platform_device *dev)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;

	if (port && port_used(port)) {
		if (port->suspended) {
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, reg->bIER);

			// DLAB = 1
			wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
			wr_regl(port, OFFSET_FCR, reg->bFCR);
			wr_regl(port, OFFSET_DLL, reg->bDLL);
			wr_regl(port, OFFSET_DLM, reg->bDLM);

			wr_regl(port, OFFSET_MCR, reg->bMCR);
			wr_regl(port, OFFSET_AFT, reg->bAFT);
			wr_regl(port, OFFSET_UCR, reg->bUCR);

			port->suspended = 0;
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) | UART_IER_ELSI));

			wr_regl(port, OFFSET_LCR, reg->bLCR);
		}
	}
	
	return 0;
}

#else
#define tcc_serial_suspend NULL
#define tcc_serial_resume  NULL
#endif

static int tcc_serial_probe(struct platform_device *dev)
{
	return tcc_uart_probe(dev);
}

static struct platform_driver tcc_serial_drv = {
	.probe		= tcc_serial_probe,
	.remove		= tcc_serial_remove,
	.suspend	= tcc_serial_suspend,
	.resume		= tcc_serial_resume,
	.driver		= {
		.name	= DRV_NAME,
		.owner	= THIS_MODULE,
	},
};



#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_serial_early_suspend(struct early_suspend *h)
{
	printk(" %s\n", __func__);
	tcc_serial_drv.suspend;

}
static void tcc92xx_serial_late_resume(struct early_suspend *h)
{
	printk(" %s\n", __func__);
	tcc_serial_drv.resume;
}

#endif


static int tcc_serial_init(struct platform_driver *drv)
{
	dbg("%s\n", __func__);
	return platform_driver_register(drv);
}

static inline void tcc_serial_exit(void)
{
	platform_driver_unregister(&tcc_serial_drv);
}


/* module initialisation code */

static int __init tcc_serial_modinit(void)
{
	int ret;
	
	ret = uart_register_driver(&tcc_uart_drv);
	if (ret < 0) {
		dbg(KERN_ERR "failed to register UART driver\n");
		return -1;
	}
	//printk("Serial driver %s registered.\n", tcc_uart_drv.driver_name);
	
	tcc_serial_init(&tcc_serial_drv);

	return 0;
}

static void __exit tcc_serial_modexit(void)
{
	tcc_serial_exit();

	uart_unregister_driver(&tcc_uart_drv);
}


module_init(tcc_serial_modinit);
module_exit(tcc_serial_modexit);

/************************************************************
 *
 * The following is Console driver 
 *
 ************************************************************/

#ifdef CONFIG_SERIAL_TCC_CONSOLE

#define BOTH_EMPTY (UART_LSR_TEMT | UART_LSR_THRE)

/*
 *  Wait for transmitter & holding register to empty
 */
static inline void wait_for_xmitr(struct uart_port *port)
{
    unsigned int status, tmout = 10000;

    /* Wait up to 10ms for the character(s) to be sent. */
    do {
        status = rd_regl(port, OFFSET_LSR);

        if (status & UART_LSR_BI)
            tcc_serial_ports[port->line].lsr_break_flag  = UART_LSR_BI;

        if (--tmout == 0)
            break;
        udelay(1);
    } while ((status & BOTH_EMPTY) != BOTH_EMPTY);

    /* Wait up to 1s for flow control if necessary */
    if (port->flags & UPF_CONS_FLOW) {
        tmout = 1000000;
        while (--tmout &&
               ((rd_regl(port, OFFSET_MSR) & UART_MSR_CTS) == 0))
            udelay(1);
    }
}

static struct uart_port *cons_uart;

static void tcc_serial_console_putchar(struct uart_port *port, int ch)
{
/*
	while (!(rd_regl(port, OFFSET_LSR) & UART_LSR_THRE))
		barrier();
*/

    wait_for_xmitr(port);
    wr_regb(port, OFFSET_THR, ch);

}

void tcc_serial_uart_putchar(struct uart_port *port, int ch)
{
	while (!(rd_regl(port, OFFSET_LSR) & UART_LSR_THRE))
		barrier();

    wr_regb(port, OFFSET_THR, ch);
}


static void tcc_console_write(struct console *co, const char *s,
			     unsigned int count)
{
	struct uart_port *port;
	//unsigned int t_ier, b_ier; 
	unsigned int ier; 

	port = &tcc_serial_ports[co->index].port;

    ier = rd_regl(port, OFFSET_IER);

    wr_regl(port, OFFSET_IER, ier & ~UART_IER_ETXI);
    uart_console_write(cons_uart, s, count, tcc_serial_console_putchar);

    wait_for_xmitr(port);
    wr_regl(port, OFFSET_IER, ier);

/*
    t_ier = rd_regl(port, OFFSET_IER);
    b_ier = t_ier;

    wr_regl(port, OFFSET_IER, t_ier & ~ETXI);
    uart_console_write(cons_uart, s, count, tcc_serial_console_putchar);
    wr_regl(port, OFFSET_IER, b_ier);
*/
}

static void __init
tcc_serial_get_options(struct uart_port *port, int *baud,
			   int *parity, int *bits)
{
}


/* tcc_serial_init_ports
 * initialise the serial ports from the machine provided initialisation data.
 */
int tcc_serial_init_ports(struct tcc_uart_port *ourport,
					struct platform_device *pdev)
{
	struct uart_port *port = &ourport->port;
    struct resource *res;
    int ret;

	dbg("tcc_serial_init_ports: initialising ports...\n");
	
	port->iotype	= UPIO_MEM;
	port->flags     = UPF_IOREMAP | UPF_BOOT_AUTOCONF;
	port->ops	    = &tcc_serial_ops;
	port->fifosize  = FIFOSIZE;
	port->line	    = pdev->id;
	port->dev	    = &pdev->dev;
	port->type      = PORT_TCC;


    res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (res == NULL) {
		printk(KERN_ERR "failed to find memory resource for uart\n");
		return -EINVAL;
	}

    /*
    tcc_uart_base = ioremap(res->start, res->end - res->start + 1);
    if(tcc_uart_base == NULL) {
        dev_err(&pdev->dev, "failed ioremap()\n");
        ret = -EINVAL;
        iounmap(tcc_uart_base);
    }
    */

	//port->mapbase	= pdev->resource[0].start;
	port->mapbase	= res->start;
	//port->membase	= ioremap_nocache(port->mapbase, MAP_SIZE);
	port->membase	= (unsigned char __iomem *)port->mapbase;
    port->irq       = platform_get_irq(pdev, 0);

    ret = (int)port->line;

	return ret;
}

static int __init
tcc_console_setup(struct console *co, char *options)
{
	struct uart_port *port;
	int baud = BAUDRATE;
	int bits = 8;
	int parity = 'n';
	int flow = 'n';

	dbg("tcc_serial_console_setup: co=%p (%d), %s\n", co, co->index, options);

	/* is this a valid port */

	if (co->index == -1 || co->index >= NR_PORTS)
		co->index = CONSOLE_PORT;

	port = &tcc_serial_ports[co->index].port;

	/* is the port configured? */

	if (port->mapbase == 0x0) {
        port->mapbase = (resource_size_t)&(CONSOLE_BASE);
        port->membase = (unsigned char __iomem *)port->mapbase;
		co->index = CONSOLE_PORT;
		port = &tcc_serial_ports[co->index].port;
	}
    
	cons_uart = port;

	if (options) {
        dbg("uart_parse_options\n");
		uart_parse_options(options, &baud, &parity, &bits, &flow);
    } else {
        dbg("tcc_serial_get_options\n");
		tcc_serial_get_options(port, &baud, &parity, &bits);
    }
	
	dbg("tcc_serial_console_setup: port=%p (%d)\n", port, co->index);
	return uart_set_options(port, co, baud, parity, bits, flow);
}

/* tcc_console_init
 *
 * initialise the console from one of the uart drivers
*/

static struct console tcc_serial_console =
{
	.name		= TCC_SERIAL_NAME,
	.device		= uart_console_device,
	.flags		= CON_PRINTBUFFER | CON_ENABLED,
	.index		= CONSOLE_PORT,
	.write		= tcc_console_write,
	.setup		= tcc_console_setup,
};

static int tcc_console_init(void)
{
	dbg("%s\n", __func__);
	
	tcc_serial_console.data = &tcc_uart_drv;
	register_console(&tcc_serial_console);
	
	return 0;
}

console_initcall(tcc_console_init);

#endif /* CONFIG_SERIAL_TCC_CONSOLE */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC Serial port driver");
