/**************************************************************************************
*
* FILE NAME   : otgcore.c
* DESCRIPTION : This is a OTG CORE driver source code
*
* ====================================================================================
*
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*
* ====================================================================================
*
* FILE HISTORY:
* 	Date: 2009.03.16	Start source coding
*
**************************************************************************************/
#include <bsp.h>
#include "usb_defs.h"
#include "tcc_otg.h"
#include "otgregs.h"


#define SLAVE_MODE_INCLUDE
//#define DMA_MODE_INCLUDE

extern PUSB20OTG pOTG;
extern PUSBOTGCFG pOTGCFG;


//unsigned char* OTGCORE_TellLibraryVersion(void)
//{
//	return (unsigned char*)OTGCORE_C_Version;
//}

void OTGCORE_Init(void)
{
#if 1
	BITCLR(HwIOBUSCFG->HRSTEN0, HwIOBUSCFG_USB);
	{
		volatile unsigned int t=1000;
		while(t-->0);
	}
	BITSET(HwIOBUSCFG->HRSTEN0, HwIOBUSCFG_USB);
#else
	// Core Soft Reset
	pOTG->GRSTCTL |= GRSTCTL_CSftRst;
	// wait self clear
	while( (pOTG->GRSTCTL & GRSTCTL_CSftRst) != 0 );
	// wait 3 PHY clocks (synchronization delay)
#endif
	// wait AHB idle
	while( !(pOTG->GRSTCTL & GRSTCTL_AHBIdle) );

#if defined(SLAVE_MODE_INCLUDE)
	pOTG->GUSBCFG |= GUSBCFG_PHYIf_16BITS|GUSBCFG_TOutCal(7);	// set interface, timeout
	pOTG->GOTGCTL |= GOTGCTL_SesReq;	// session request
	pOTG->GINTSTS = 0xFFFFFFFF;		// clear interrupts
	pOTG->GAHBCFG = GAHBCFG_NPTXFEmpLvl_COMPLETELY|GAHBCFG_GlblIntrMsk_UNMASK;	// enable global interrupt
#elif defined(DMA_MODE_INCLUDE)
	// device mode
	pOTG->OTGID |= OTGID_ID_DEVICE;

	// Core Initialization
	pOTG->GINTSTS = 0xFFFFFFFF;		// clear interrupts
	pOTG->GAHBCFG = GAHBCFG_DMAEn|GAHBCFG_HBstLen_SINGLE|GAHBCFG_GlblIntrMsk_MASK;
	pOTG->GUSBCFG = GUSBCFG_USBTrdTim_16BIT_UTMIP|GUSBCFG_PHYIf_16BITS|GUSBCFG_TOutCal(7);
	pOTG->GOTGCTL |= GOTGCTL_SesReq;	// session request
	pOTG->GINTMSK = 0/*GINTMSK_OTGIntMsk_UNMASK|GINTMSK_ModeMisMsk_UNMASK*/;
#endif
}

void OTGCORE_SetID(unsigned int ID)
{
	if( ID )
		pOTGCFG->OTGID |= OTGID_ID_DEVICE;
	else
		pOTGCFG->OTGID &= OTGID_ID_HOST;
}

void OTGCORE_Reset(void)
{
	// Core Soft Reset
	pOTG->GRSTCTL |= GRSTCTL_CSftRst;
	// wait self clear
	while( (pOTG->GRSTCTL & GRSTCTL_CSftRst) != 0 );

	// wait 3 PHY clocks (synchronization delay)
	{
		volatile unsigned int i,cnt=0;
		for(i=0;i<1000000;i++)
			cnt++;
	}

	// wait AHB idle
	while( !(pOTG->GRSTCTL & GRSTCTL_AHBIdle) );
}

void OTGDEV_IO_Init(void)
{
	pOTG->DCTL = 0;
#if defined(SLAVE_MODE_INCLUDE)
	pOTG->DIEPMSK = DIEPMSK_XferComplMsk_UNMASK;
	// Unmask core interrupt
	pOTG->GINTMSK |= GINTMSK_EnumDoneMsk_UNMASK|GINTMSK_USBRstMsk_UNMASK|GINTMSK_RxFLvlMsk_UNMASK;
#elif defined(DMA_MODE_INCLUDE)
	pOTG->DCFG = DCFG_PerFrInt(0)|DCFG_NZStsOUTHShk_STALL|DCFG_DevSpd_HS;
	pOTG->GINTMSK |= GINTMSK_USBRstMsk_UNMASK|GINTMSK_EnumDoneMsk_UNMASK|GINTMSK_ErlySuspMsk_UNMASK|GINTMSK_USBSuspMsk_UNMASK|GINTMSK_SofMsk_UNMASK;
#endif
}

#define MAX_OUT_EPn_PACKET_SIZE			512
#define MAX_IN_EP0_PACKET_SIZE			64
#define MAX_IN_EPn_PACKET_SIZE			512
#define SUPPORTED_CONTROL_EP_NUMBER		1
#define OUT_EP_NUMBER					16
#define GRXFSIZ_COMMON					( ( 4*( (MAX_OUT_EPn_PACKET_SIZE/4) + 1 ) ) + (4*SUPPORTED_CONTROL_EP_NUMBER) + 6 + (2*OUT_EP_NUMBER) + 1 )
#define GNPTXFSIZ_COMMON				(1*(MAX_IN_EP0_PACKET_SIZE/4))
#define DIEPTXFn_COMMON					(4*(MAX_IN_EPn_PACKET_SIZE/4))
#define IN_EP_NUMBER_COMMON				4 /* jmlee-090416 : adb bulk ep : 2 -> 4 */
void OTGDEV_IO_Reset(void)
{
	unsigned int i;
	unsigned int fifoAddress;

	pOTG->DCFG = 0;

	// Set the NAK bit for all OUT endpoints
	pOTG->DOEPCTL0 = DOEPCTL0_SNAK;
	for(i=0;i<15;i++)
		pOTG->DEVOUTENDPT[i][DOEPCTL_INDEX] = DOEPCTL_SNAK;

	pOTG->GRXFSIZ = GRXFSIZ_COMMON;
	fifoAddress = GRXFSIZ_COMMON;
	pOTG->GNPTXFSIZ = GNPTXFSIZ_INEPTxF0Dep(GNPTXFSIZ_COMMON)|GNPTXFSIZ_INEPTxF0StAddr(fifoAddress);
	fifoAddress += GNPTXFSIZ_COMMON;
	for( i = 0 ; i < IN_EP_NUMBER_COMMON ; i++ )
	{
		pOTG->DIEPTXFn[i] = DIEPTXF_INEPnTxFDep(DIEPTXFn_COMMON)|DIEPTXF_INEPnTxFStAddr(fifoAddress);
		fifoAddress += DIEPTXFn_COMMON;
	}
	if( fifoAddress > 4160 )
		while(1);	// FIFO allocation overflow is occurred!!!

	// Reset - set and flush All TxFIFO, flush RxFIFO
	GRSTCTL_TxFNum_Write(pOTG->GRSTCTL, 0x10);
	BITSET(pOTG->GRSTCTL,GRSTCTL_TxFFlsh|GRSTCTL_RxFFlsh);
	do
	{
		if( (pOTG->GRSTCTL&(GRSTCTL_TxFFlsh|GRSTCTL_RxFFlsh)) == 0 )
			break;
	} while(1);

#if defined(SLAVE_MODE_INCLUDE)
	pOTG->DIEPINT0 = pOTG->DIEPINT0;
	for(i=0;i<15;i++)
		pOTG->DEVOUTENDPT[i][DIEPINT_INDEX] = pOTG->DEVOUTENDPT[i][DIEPINT_INDEX];

	pOTG->DAINTMSK = DAINTMSK_InEPInt(0);
	pOTG->DIEPEMPMSK = 0;
	// Unmask core interrupt
	pOTG->GINTMSK |= GINTMSK_IEPIntMsk_UNMASK;
#elif defined(DMA_MODE_INCLUDE)
#endif
}


void OTGDEV_IO_EnumDone(void)
{
/*	unsigned int enumSpeed;	// 0:high speed(30/60MHz), 1:full speed(30/60MHz), 2:low speed(6MHz), 3: full speed(48MHz)
	enumSpeed = DSTS_EnumSpd_Read(pOTG->DSTS);
	if( enumSpeed == 0 )
	{
		g_OTGDEV_IO_Driver.currentSpeed = USBDEV_HIGH_SPEED;
	}
	else
	{
		g_OTGDEV_IO_Driver.currentSpeed = USBDEV_FULL_SPEED;
	}
	*/
#if defined(SLAVE_MODE_INCLUDE)
	// OUTEP0 Control
	pOTG->DOEPCTL0 = DOEPCTL0_EPEna|DOEPCTL0_CNAK;

	pOTG->DIEPCTL0 = DIEPCTL0_TxFNum(0)|DIEPCTL0_EPType|DIEPCTL0_USBActEP_MASK|DIEPCTL0_MPS_64BYTES;
	// OUTEP1 Control
	//pOTG->DEVOUTENDPT[0][DOEPCTL_INDEX] = DOEPCTL_EPEna|DOEPCTL_SetD0PID|DOEPCTL_CNAK|DOEPCTL_EPType_BULK|DOEPCTL_USBActEP|DOEPCTL_MPS(512);

	// OUTEP2 Control
	//pOTG->DEVOUTENDPT[1][DOEPCTL_INDEX] = DOEPCTL_EPEna|DOEPCTL_SetD0PID|DOEPCTL_CNAK|DOEPCTL_EPType_BULK|DOEPCTL_USBActEP|DOEPCTL_MPS(512);

	// INEP1 Control
	//pOTG->DEVINENDPT[0][DIEPCTL_INDEX] = DIEPCTL_SetD0PID|DIEPCTL_TxFNum(1)|DIEPCTL_EPType_BULK|DIEPCTL_USBActEP|DIEPCTL_MPS(512);

	// INEP2 Control
	//pOTG->DEVINENDPT[1][DIEPCTL_INDEX] = DIEPCTL_SetD0PID|DIEPCTL_TxFNum(2)|DIEPCTL_EPType_BULK|DIEPCTL_USBActEP|DIEPCTL_MPS(512);
#elif defined(DMA_MODE_INCLUDE)
	BITCSET(pOTG->DIEPCTL0,DIEPCTL0_MPS_MASK,DIEPCTL0_MPS_64BYTES);
	pOTG->DOEPCTL0 |= DOEPCTL0_EPEna|DOEPCTL0_CNAK;
#endif
}

unsigned int _loc_WaitForSet(volatile unsigned int *pReg, unsigned int mask)
{
	while( !( (*pReg) & mask ) )
	{
		//if( USBDEV_IsConnected() == FALSE )
		//	return FALSE;
	}
	return TRUE;
}

unsigned int _loc_WaitForClear(volatile unsigned int *pReg, unsigned int mask)
{
	while( (*pReg) & mask )
	{
		//if( USBDEV_IsConnected() == FALSE )
		//	return FALSE;
	}
	return TRUE;
}

void _loc_Disable_IN_EP(unsigned int nEndPoint)
{
	if( nEndPoint < 16 )
	{
		volatile unsigned int *pDIEPINT = (volatile unsigned int*)((unsigned int)&pOTG->DIEPINT0 + 0x20*nEndPoint);
		volatile unsigned int *pDIEPCTL = (volatile unsigned int*)((unsigned int)&pOTG->DIEPCTL0 + 0x20*nEndPoint);

		if( !(*pDIEPCTL&DIEPCTL_NakSts_MASK) )
		{
			*pDIEPINT = DIEPINT_INEPNakEff;	// clear
			BITSET(*pDIEPCTL, DIEPCTL_SNAK);
			if( _loc_WaitForSet(pDIEPINT, DIEPINT_INEPNakEff) == FALSE )
				return;
		}

		if( *pDIEPCTL & DIEPCTL_EPEna )
		{
			*pDIEPINT = DIEPINT_EPDisabld;	// clear
			BITSET(*pDIEPCTL, DIEPCTL_EPDis|DIEPCTL_SNAK);
			if( _loc_WaitForSet(pDIEPINT, DIEPINT_EPDisabld) == FALSE )
				return;
		}

		GRSTCTL_TxFNum_Write(pOTG->GRSTCTL,nEndPoint);
		BITSET(pOTG->GRSTCTL, GRSTCTL_TxFFlsh);
		if( _loc_WaitForClear(&pOTG->GRSTCTL, GRSTCTL_TxFFlsh) == FALSE )
			return;
	}
}

void OTGDEV_IO_EP_Stall(unsigned char epAddress)
{
	unsigned int nEndPoint = (unsigned int)(epAddress & 0x0F);
	volatile unsigned int *pDEPCTL;
	if( epAddress & 0x80 )
	{
		// IN
		pDEPCTL = &pOTG->DIEPCTL0;
		_loc_Disable_IN_EP(nEndPoint);
	}
	else
	{
		// OUT
		pDEPCTL = &pOTG->DOEPCTL0;
		//_loc_Disable_OUT_EP(nEndPoint);
	}
	pDEPCTL = (volatile unsigned int*)( (unsigned int)pDEPCTL + 0x20*nEndPoint );

	BITSET(*pDEPCTL,Hw21);	// stall
}

void OTGDEV_IO_EP_Unstall(unsigned char epAddress)
{
	unsigned int nEndPoint = (unsigned int)(epAddress & 0x0F);
	volatile unsigned int *pDEPCTL;
	unsigned int flagUnstall = FALSE;
	if( epAddress & 0x80 )
	{
		// IN
		//if( gOTGDEV_IO_IN_EP_UnstallEnable & (1<<nEndPoint) )
		{
			flagUnstall = TRUE;
			pDEPCTL = &pOTG->DIEPCTL0;
			//gOTGDEV_InEp[nEndPoint].fFirstTransfer=TRUE;
		}
	}
	else
	{
		// OUT
		//if( gOTGDEV_IO_OUT_EP_UnstallEnable & (1<<nEndPoint) )
		{
			flagUnstall = TRUE;
			pDEPCTL = &pOTG->DOEPCTL0;
		}
	}
	if( flagUnstall )
	{
		pDEPCTL = (volatile unsigned int*)( (unsigned int)pDEPCTL + 0x20*nEndPoint );
		BITSET(*pDEPCTL, Hw28);	// Set "PID 0" for interrupt/bulk, Set "even frame" for isochronous
		BITCLR(*pDEPCTL, Hw21);	// unstall
	}
}

void OTGDEV_IO_TestMode(unsigned char value)
{
	if( 0 < value && value < 6 )
	{
		BITCSET(pOTG->DCTL, DCTL_TstCtl_MASK, DCTL_TstCtl(value));
	}
	else
	{
		// Reserved
		BITCLR(pOTG->DCTL, DCTL_TstCtl_MASK);
	}
}

unsigned int OTGDEV_IO_IsStalled(unsigned char epAddress)
{
	unsigned int nEndPoint = (unsigned int)(epAddress & 0x0F);
	volatile unsigned int *pDEPCTL;
	if( epAddress & 0x80 )
	{
		// IN
		pDEPCTL = &pOTG->DIEPCTL0;
	}
	else
	{
		// OUT
		pDEPCTL = &pOTG->DOEPCTL0;
	}
	pDEPCTL = (volatile unsigned int*)( (unsigned int)pDEPCTL + 0x20*nEndPoint );
	if( (*pDEPCTL&Hw21) != 0 )	// stall
		return TRUE;
	else
		return FALSE;
}
