/*
 * linux/drivers/video/tcc93xxfb_interface.c
 *
 * Based on:    Based on s3c2410fb.c, sa1100fb.c and others
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: TCC LCD Controller Frame Buffer Driver
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/cpufreq.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
//#include <mach/tcc_used_mem.h>
#include <asm/tlbflush.h>
#include <asm/cacheflush.h>

#include <mach/globals.h>
#include <mach/reg_physical.h>
#include <mach/tca_ckc.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include "tcc93xxfb.h"
#include <mach/tcc_scaler_ctrl.h>

#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_composite_ioctl.h>
#include <mach/tcc_component_ioctl.h>

#include <mach/tca_lcdc.h>
#include <mach/tca_backlight.h>
#include <mach/tca_fb_hdmi.h>
#include <mach/tca_fb_output.h>
#include <linux/tcc_pwm.h>

/* define vsync methods */
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
#undef USE_VSYNC_INTERRUPT
#else
#define USE_VSYNC_INTERRUPT
#endif//

#ifdef USE_VSYNC_INTERRUPT 
#define FB_M2M_COPY
#endif//
#undef USE_VSYNC_POLLING


/* Debugging stuff */
static int debug = 0;
#define dprintk(msg...)	if (debug) { printk( "fb93_I: " msg); }




#ifdef FB_M2M_COPY
extern TCC_OUTPUT_TYPE	Output_SelectMode;

extern unsigned char tccxxx_overlay_use(void);
extern char M2M_Scaler0_Ctrl_External(SCALER_TYPE *scale_img);

dma_addr_t		Gmap_dma;	/* physical */
u_char *		Gmap_cpu;	/* virtual */

typedef struct{
	unsigned int update;
	unsigned int left;
	unsigned int top; 
	unsigned int eright;
	unsigned int ebottom;
	struct tccfb_info fbi;
}tcc_patial_info;
tcc_patial_info tccfb_parial;

#endif//

struct lcd_struct {
	spinlock_t lock;
	wait_queue_head_t waitq;
	char state;
};
static struct lcd_struct lcdc_struct;


char Fb_Lcdc_num = 1;
static PLCDC pLCDC;
static PPIC pHwPIC;


static struct clk *lcdc_clk;
static struct clk *ddi_cache;
#ifdef CONFIG_LCD_CPU_INTERFACE
static struct clk *lcd_si;
#endif//CONFIG_LCD_CPU_INTERFACE
#ifdef FB_M2M_COPY
static struct clk *scaler0_clk;		// fb image scale
#endif// FB_M2M_COPY

void tca93xxfb_clock_init(void)
{
	if(Fb_Lcdc_num)
		lcdc_clk = clk_get(0, "lcdc1");
	else
		lcdc_clk = clk_get(0, "lcdc0");

	ddi_cache = clk_get(0, "ddi_cache");

	#ifdef CONFIG_LCD_CPU_INTERFACE
	lcd_si = clk_get(0, "lcdsi");
	#endif//

	BUG_ON(lcdc_clk == NULL);
	
	BUG_ON(ddi_cache == NULL);
	#ifdef CONFIG_LCD_CPU_INTERFACE
	BUG_ON(lcd_si == NULL);
	#endif//

	#ifdef FB_M2M_COPY
	scaler0_clk = clk_get(0, "m2m0");
	BUG_ON(scaler0_clk == NULL);
	#endif//

}

void tca93xxfb_clock_delete(void)
{
	clk_put(lcdc_clk);
	clk_put(ddi_cache);

	#ifdef CONFIG_LCD_CPU_INTERFACE
	clk_put(lcd_si);
	#endif//

	#ifdef FB_M2M_COPY
	clk_put(scaler0_clk);
	#endif//
}

extern struct tcc_freq_table_t stFBClockLimitTable;
static int  tca93xxfb_clock_set(int cmd)
{
	int ret = 0;
	
    switch (cmd) {
	    case PWR_CMD_OFF:
#if defined(CONFIG_MACH_TCC9300) // temp : DDR2, LPDDR only
			tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 0);
#endif			
			clk_disable(lcdc_clk);
			clk_disable(ddi_cache);

			#ifdef CONFIG_LCD_CPU_INTERFACE
			clk_disable(lcd_si);
			#endif//
			
			#ifdef FB_M2M_COPY
			clk_disable(scaler0_clk);
			#endif//
			
			break;
			
	    case PWR_CMD_ON:
			clk_enable(lcdc_clk);
			clk_enable(ddi_cache);

			#ifdef CONFIG_LCD_CPU_INTERFACE
			clk_enable(lcd_si);
			#endif//

			#ifdef FB_M2M_COPY
			clk_enable(scaler0_clk);
			#endif//

#if defined(CONFIG_MACH_TCC9300) // temp : DDR2, LPDDR only
			tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 1);
#endif			
			break;

	    default:
			ret = -EINVAL;
	        break;
    }
    return ret;
	
}

#ifdef FB_M2M_COPY
void tcc_scaler_copy(unsigned int left, unsigned int top, unsigned int eright, unsigned int ebottom, struct tccfb_info *fbi)
{
	SCALER_TYPE fbscaler;
	unsigned int src_addr, dest_addr;
	unsigned int offset, size, i;

	offset = top * (fbi->fb->var.xres) * (fbi->fb->var.bits_per_pixel / 8);
	
	size = (ebottom - top) * (fbi->fb->var.xres) * (fbi->fb->var.bits_per_pixel / 8);
	src_addr = fbi->map_dma + fbi->fb->var.xres * fbi->fb->var.yoffset *(fbi->fb->var.bits_per_pixel / 8);

	
	if(fbi->fb->var.yoffset > fbi->fb->var.yres)	{
		src_addr = PAGE_ALIGN(src_addr);
	}
	
	dprintk("%s left:%d  top:%d right:%d ebottom:%d addr:0x%x addr:0x%x \n", __func__, left, top, eright, ebottom, Gmap_dma, src_addr);

	src_addr = src_addr + offset;
	dest_addr = Gmap_dma + offset;
	
	fbscaler.responsetype = SCALER_NOWAIT;

	fbscaler.src_Yaddr = src_addr;
	fbscaler.src_Uaddr = 0;
	fbscaler.src_Vaddr = 0;
	fbscaler.src_fmt = SCALER_YUV422_sq0;
	fbscaler.src_ImgWidth = (eright - left)* 2;
	fbscaler.src_ImgHeight = (ebottom - top);

	fbscaler.src_winLeft = 0;
	fbscaler.src_winTop = 0;
	fbscaler.src_winRight = (eright - left)* 2;
	fbscaler.src_winBottom = (ebottom - top);

	fbscaler.dest_Yaddr = dest_addr;	// destination image address
	fbscaler.dest_Uaddr = 0;	// destination image address
	fbscaler.dest_Vaddr = 0;	// destination image address
	fbscaler.dest_fmt = SCALER_YUV422_sq0;		// destination image format
	fbscaler.dest_ImgWidth = (eright - left)* 2;	// destination image width
	fbscaler.dest_ImgHeight = (ebottom - top); // destination image height

	fbscaler.dest_winLeft = 0;
	fbscaler.dest_winTop = 0;
	fbscaler.dest_winRight = (eright - left)* 2;
	fbscaler.dest_winBottom = (ebottom - top);

	M2M_Scaler0_Ctrl_External(&fbscaler);

}
#endif//


/* tccfb_pan_display
 *
 * pandisplay (set) the controller from the given framebuffer
 * information
*/
int tca_fb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	struct tccfb_info *fbi =(struct tccfb_info *) info->par;
	
	unsigned int regl;
	unsigned int base_addr = 0;
	unsigned int lcd_width, lcd_height, img_width, img_height;
	unsigned int bpp = fbi->fb->var.bits_per_pixel / 8;	/* bytes per pixel */
#ifdef USE_VSYNC_POLLING
	unsigned long buffer_size = fbi->fb->var.xres * bpp * fbi->fb->var.yres
#endif

	regl = pLCDC->LDS; // get LCD size 
 	base_addr = fbi->map_dma +fbi->fb->var.xres * bpp * var->yoffset;
	if(var->yoffset > var->yres)	{
		base_addr = PAGE_ALIGN(base_addr);
	}
	
	#ifdef FB_M2M_COPY
	if(!tccxxx_overlay_use() && !Output_SelectMode)
		base_addr = Gmap_dma;
	#endif//

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);
	img_width = fbi->fb->var.xres;
	img_height = fbi->fb->var.yres;

	if(img_width > lcd_width)	
		img_width = lcd_width;
	
	if(img_height > lcd_height)	
		img_height = lcd_height;

	
	pLCDC->LI2BA0 	= base_addr;
	pLCDC->LI2S	= (img_height<< 16) | (img_width); //Size
	pLCDC->LI2O = fbi->fb->var.xres * bpp;
	pLCDC->LI2C |=  (HwLIC_IEN | HwLIC_UPD);    //  image enable

	#ifdef USE_VSYNC_INTERRUPT
		#ifdef FB_M2M_COPY
			if(tccfb_parial.update == 1)	{
				lcdc_struct.state = 0;
				wait_event_interruptible(lcdc_struct.waitq, lcdc_struct.state == 1);
			}

			if((fbi->fb->var.reserved[0] == 0x54445055) && (fbi->fb->var.reserved[2] != 0))	{
				tccfb_parial.fbi = *fbi;
				tccfb_parial.update = 1;
				tccfb_parial.left = fbi->fb->var.reserved[1] & 0xffff;
				tccfb_parial.top = fbi->fb->var.reserved[1] >> 16;
				tccfb_parial.eright= fbi->fb->var.reserved[2] & 0xffff;
				tccfb_parial.ebottom= fbi->fb->var.reserved[2] >> 16;
			}
			else {
				tccfb_parial.fbi = *fbi;
				tccfb_parial.update = 1;
				tccfb_parial.left = 0;
				tccfb_parial.top = 0;
				tccfb_parial.eright= fbi->fb->var.xres;
				tccfb_parial.ebottom= fbi->fb->var.yres;
			}
			
		pLCDC->LSTATUS = 0xFFFFFFFF; 
		tca_lcdc_interrupt_onoff(TRUE, Fb_Lcdc_num);
		#else
		
		pLCDC->LSTATUS = 0xFFFFFFFF; 
		tca_lcdc_interrupt_onoff(TRUE, Fb_Lcdc_num);
		lcdc_struct.state = 0;
		wait_event_interruptible(lcdc_struct.waitq, lcdc_struct.state == 1);
		#endif//FB_M2M_COPY
	#endif //USE_VSYNC_INTERRUPT

	return 0;
}
EXPORT_SYMBOL(tca_fb_pan_display);

void tca_fb_pan_display_sync(void)
{

}
EXPORT_SYMBOL(tca_fb_pan_display_sync);

/* tccfb_activate_var
 * activate (set) the controller from the given framebuffer
 * information
*/
void tca_fb_activate_var(struct tccfb_info *fbi,  struct fb_var_screeninfo *var)
{
	unsigned int imgch = 0;
	unsigned int bpp_value;
	unsigned int tmp_value;

	if(fbi->fb->var.bits_per_pixel == 32)
		bpp_value = 0x000C; //RGB888
	else if(fbi->fb->var.bits_per_pixel == 16)
		bpp_value = 0x000A; //RGB565
	else	{
		dprintk("%s:fb%d Not Supported BPP!\n", __FUNCTION__, fbi->fb->node);
		return;
	}

	/* write new registers */
	switch(imgch)
	{
		case 0:
			tmp_value = pLCDC->LI2C;
			pLCDC->LI2C = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC->LI2O = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;

		case 1:
			tmp_value = pLCDC->LI1C;
			pLCDC->LI1C = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC->LI1O = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;

		case 2:
			tmp_value = pLCDC->LI0C;
			pLCDC->LI0C = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC->LI0O = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;
	}
	
	return;
	
}
EXPORT_SYMBOL(tca_fb_activate_var);



#ifdef CONFIG_HAS_EARLYSUSPEND
void tca_fb_early_suspend(struct early_suspend *h)
{
	printk("%s: START Fb_Lcdc_num:%d ", __FUNCTION__, Fb_Lcdc_num);

	tca93xxfb_clock_set(PWR_CMD_OFF);
	
	printk(" clock off");

//to do	tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 0);

	printk("  end ~\n ");
}
EXPORT_SYMBOL(tca_fb_early_suspend);


void tca_fb_earlier_suspend(struct early_suspend *h)
{
	printk("%s: START Fb_Lcdc_num:%d \n", __FUNCTION__, Fb_Lcdc_num);
  	tca_bkl_powerdown();
}
EXPORT_SYMBOL(tca_fb_earlier_suspend);


void tca_fb_late_resume(struct early_suspend *h)
{
	printk("%s: ", __FUNCTION__);

//to do	tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 1);

	tca93xxfb_clock_set(PWR_CMD_ON);
	tcc_ddi_cache_setting();

	printk(" end \n ");
}
EXPORT_SYMBOL(tca_fb_late_resume);


void tca_fb_later_resume(struct early_suspend *h)
{
	printk("%s: START Fb_Lcdc_num:%d \n", __FUNCTION__, Fb_Lcdc_num);

  	tca_bkl_powerup();
}
EXPORT_SYMBOL(tca_fb_later_resume);
#endif



/* suspend and resume support for the lcd controller */
int tca_fb_suspend(struct platform_device *dev, pm_message_t state)
{
	printk("%s:  \n", __FUNCTION__);
	return 0;
}
EXPORT_SYMBOL(tca_fb_suspend);


int tca_fb_resume(struct platform_device *dev)
{
	printk("%s:  \n", __FUNCTION__);
	return 0;
}
EXPORT_SYMBOL(tca_fb_resume);


irqreturn_t tca_fb_lcdc_handler(void)
{
	unsigned int LCDCstatus = pLCDC->LSTATUS;

	LCDCstatus = pLCDC->LSTATUS;
// 	dprintk("%s lcdc_struct.state:%d STATUS:0x%x\ 0x%x \n",__func__, lcdc_struct.state, LCDCstatus,  pLCDC->LSTATUS);

//	if(LCDCstatus & HwLSTATUS_RU)
	{
		#ifdef FB_M2M_COPY
		if(tccfb_parial.update)		{
			// check scaler using 
			if(!tccxxx_overlay_use() && !Output_SelectMode)
			{
				tcc_scaler_copy(tccfb_parial.left, tccfb_parial.top, tccfb_parial.eright, tccfb_parial.ebottom, &tccfb_parial.fbi);
			}
			else	{
				unsigned int src_addr = 0;
				struct tccfb_info *fbi = &tccfb_parial.fbi;
				
				src_addr = fbi->map_dma + fbi->fb->var.xres * fbi->fb->var.yoffset *(fbi->fb->var.bits_per_pixel / 8);
				
				if(fbi->fb->var.yoffset > fbi->fb->var.yres)	{
					src_addr = PAGE_ALIGN(src_addr);
				}
				pLCDC->LI2BA0 	= src_addr;			
			}
			pLCDC->LI2C |=  (HwLIC_UPD | HwLIC_IEN);    //  image enable
			tccfb_parial.update = 0;
		}
		#endif//
		
		pLCDC->LSTATUS = 0xFFFFFFFF;

		tca_lcdc_interrupt_onoff(FALSE, Fb_Lcdc_num);

		if(lcdc_struct.state == 0)		{
			lcdc_struct.state = 1;
			wake_up_interruptible(&lcdc_struct.waitq);
		}
	}

	
// 	dprintk("%s lcdc_struct.state:%d STATUS:0x%x\ 0x%x \n",__func__, lcdc_struct.state, LCDCstatus,  pLCDC->LSTATUS);
	return IRQ_HANDLED;
}
EXPORT_SYMBOL(tca_fb_lcdc_handler);


int tca_fb_init(void)
{
    printk(KERN_INFO " tcc93xx (built %s %s)\n", __func__, __DATE__, __TIME__);

#ifdef TCC_LCDC1_USE
	pLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	Fb_Lcdc_num = 1;
#else
	pLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	Fb_Lcdc_num = 0;
#endif//	TCC_LCDC1_USE
	pHwPIC  = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	tca93xxfb_clock_init();
	tca93xxfb_clock_set(PWR_CMD_ON);

	tca_bkl_init(Fb_Lcdc_num);

	tcc_ddi_cache_setting();

	TCC_OUTPUT_LCDC_Init();

    init_waitqueue_head(&lcdc_struct.waitq);
	lcdc_struct.state = 1;

	#ifdef FB_M2M_COPY
	Gmap_cpu  = dma_alloc_writecombine(NULL, SCREEN_WIDTH* SCREEN_HEIGHT * 4 , &Gmap_dma, GFP_KERNEL);
	#endif//

	printk(" %s LCDC:%d  end \n", __func__, Fb_Lcdc_num);

	return 0;
}
EXPORT_SYMBOL(tca_fb_init);


void tca_fb_cleanup(void)
{
	dprintk(" %s LCDC:%d \n", __func__, Fb_Lcdc_num);
	tca93xxfb_clock_delete();
}
EXPORT_SYMBOL(tca_fb_cleanup);


