/*
 * linux/drivers/video/tccfb.c
 *	Copyright (c) Telechips, 2007 -
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file COPYING in the main directory of this archive for
 * more details.
 *
 *	    TCC LCD Controller Frame Buffer Driver
 *	    based on s3c2410fb.c, sa1100fb.c and others
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#include "tca_backlight.h"
#endif

#if defined(_TCC79x_)
#include <mach/tcc79x.h>
#elif defined(_TCC83x_)
#include <mach/tcc83x_virt.h>
#endif

#include <asm/plat-tcc/tcc_ckc.h>

#include "tccfb.h"

#if defined(CONFIG_FB_TCC7901_DEMOBOARD)
#include "tcc_lcdc.h"
#elif defined(CONFIG_FB_TCC7901_LMS350DF01)
#include "lms350df01_lcdc.h"
#endif

#include <mach/tccfb_ctrl.h>

#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
/* Debugging stuff */
//#ifdef CONFIG_FB_DEBUG
#if 0
static int debug	   = 1;
#else
static int debug	   = 0;
#endif
#define dprintk(msg...)	if (debug) { printk( "tccfb: " msg); }

//#define POST_SET

#define TCC_LCD_FB_IOCTL		0x46FF


#define CONFIG_FB_TCC_DEVS_MAX	3	// do not change!
#define CONFIG_FB_TCC_DEVS		1

#if (CONFIG_FB_TCC_DEVS > CONFIG_FB_TCC_DEVS_MAX)
	#undef CONFIG_FB_TCC_DEVS
	#define CONFIG_FB_TCC_DEVS	CONFIG_FB_TCC_DEVS_MAX
#endif


#define SCREEN_DEPTH_MAX	32	// 32 or 16 
//								 : 32 - 32bpp(alpah+rgb888)
//								 : 16 - 16bpp(rgb565)

const unsigned int default_scn_depth[CONFIG_FB_TCC_DEVS_MAX] = 
{
/* fb0, Layer0: bottom */  (16), // 32 or 16
/* fb1, Layer1: middle */  (16), //  "
/* fb2, Layer2: top    */  (16)  //  "

};

void LCD_Backlight_onoff(char onoff)
{
	BITCSET(HwPORTCFG4, HwPORTCFG4_GPIOA5(0xF), HwPORTCFG4_GPIOA5(0x1));

	BITSET( HwGPAEN , Hw5);

	if(onoff)
		BITSET( HwGPADAT , Hw5);
	else
		BITCLR( HwGPADAT , Hw5);
}

void DImage_Set_Info(unsigned int DC, DImageInfoType *ImageInfo)
{
	dprintk("DImage_Set_Info [dc:%d]  [w:%d] [h:%d] [sx:%d] [sy:%d] [fmt:%d]  \n", 
		DC, ImageInfo->width ,ImageInfo->height, ImageInfo->pos_x, ImageInfo->pos_y, ImageInfo->ImageFmt);

	LCDC_SetImageInfo(SET_IMAGE_ALL, DC, ImageInfo->width ,ImageInfo->height, 
						ImageInfo->pos_x, ImageInfo->pos_y,
						0, 0, 0, 0, 
						ImageInfo->ImageFmt);
}

/*------------------------------------------------------------------
Display image address setting function
DC : display channel (IMG_CH0	, IMG_CH1 , IMG_CH2)
ImgAddress : Image address
-------------------------------------------------------------------*/
void DImage_Set_address(unsigned int DC, char* AddrY, char* AddrU, char* AddrV)
{
	dprintk("DImage_Set_address DC:%d, Y:0x%x U:0x%x V:0x%x\n", DC, (unsigned int)AddrY, (unsigned int)AddrU, (unsigned int)AddrV);
	LCDC_SetImageBase(DC, (unsigned int)AddrY, (unsigned int)AddrU, (unsigned int)AddrV);
}


/*------------------------------------------------------------------
Display image alphaBlending enable
DC : display channel (IMG_CH1 , IMG_CH2)
Img_info : image information structure
-------------------------------------------------------------------*/
void DImage_Alphablend_en(unsigned int channel, DC_EnableType DC_enable)
{

	unsigned int DCenable = 0;
	dprintk("DImage_Alphablend_en channel:%d, enable:%d \n", channel, DC_enable);
	if(channel == IMG_CH1)
	{
		if(DC_enable == DC_ENABLE)
			DCenable |=  SET_CH1ALPHA_ENABLE;
		else
			DCenable |=  SET_CH1ALPHA_DISABLE;
	}
	else if(channel == IMG_CH2)
	{
		if(DC_enable == DC_ENABLE)
			DCenable |=  SET_CH2ALPHA_ENABLE;
		else
			DCenable |=  SET_CH2ALPHA_DISABLE;
	}

	LCDC_SetControlInfo(PAGE0, DCenable);
	
}

void DImage_Chroma_en(unsigned int channel, DC_EnableType DC_enable)
{
	dprintk("DImage_Chroma_en channel:%d, enable:%d \n", channel, DC_enable);

	if(channel == IMG_CH1)
	{
		if(DC_enable == DC_ENABLE)
			LCDC_SetControlInfo(PAGE0, SET_CH1CHROMA_ENABLE);
		else
			LCDC_SetControlInfo(PAGE0, SET_CH1CHROMA_DISABLE);
	}
	else if(channel == IMG_CH2)
	{
		if(DC_enable == DC_ENABLE)
			LCDC_SetControlInfo(PAGE0, SET_CH2CHROMA_ENABLE);
		else
			LCDC_SetControlInfo(PAGE0, SET_CH2CHROMA_DISABLE);
	}


}


/*------------------------------------------------------------------
Display image overlay setting function
DC : display channel 
IMG_CH1 , IMG_CH2
Overlay_info : Overlay information structure
-------------------------------------------------------------------*/
#define LCDC_ALPHASET		(SET_LCD_ALPHA_VALUE | SET_LCD_ALPHAMASK_VALUE)
void DImage_Set_alphablending(unsigned int DC, DImageAlphaSetType *AlphablendInfo)
{
	dprintk("DImage_Set_alphablending DC:%d, type:%d [value0:%d] [value1:%d]\n", DC, AlphablendInfo->alpha_type, 
												AlphablendInfo->alpha_value0, AlphablendInfo->alpha_value1);
	LCDC_SetAlphatype(IMG_CH1, AlphablendInfo->alpha_type);

	LCDC_SetImageOverlayKey(LCDC_ALPHASET, DC, 
							AlphablendInfo->alpha_value0,
							0, 0, 0, 
							AlphablendInfo->alpha_value1, 
							0, 0,0 );
}


#define LCDC_CHROMAKEYSET	(SET_LCD_KEY_BLUE | SET_LCD_KEY_GREEN | SET_LCD_KEY_RED \
							| SET_LCD_KEYMASK_BLUE | SET_LCD_KEYMASK_GREEN |SET_LCD_KEYMASK_RED)
void DImage_Set_Chromakey(unsigned int DC, DImageChromaSetType *ChromaInfo)
{
	dprintk("DImage_Set_Chromakey DC:%d, [R:%x][G:%x][B:%x][MR:%x][MG:%x][MB:%x] \n", DC, 
							ChromaInfo->chromakeyR, ChromaInfo->chromakeyG, ChromaInfo->chromakeyB, 
							ChromaInfo->chromakeyMR, ChromaInfo->chromakeyMG, ChromaInfo->chromakeyMB);

	LCDC_SetImageOverlayKey(LCDC_CHROMAKEYSET, DC, 0,
							ChromaInfo->chromakeyR, 
							ChromaInfo->chromakeyG, 
							ChromaInfo->chromakeyB, 
							0, 
							ChromaInfo->chromakeyMR, 
							ChromaInfo->chromakeyMG, 
							ChromaInfo->chromakeyMB);
}




/*------------------------------------------------------------------
Display image channel control function
ch0 : image channel 0
ch1 : image channel 1
ch2 : image channel 2
-------------------------------------------------------------------*/
void DImage_channel_enable(unsigned int channel, DC_EnableType DC_enable)
{
	dprintk("DImage_channel_enable channel:%d, enable:%d \n", channel, DC_enable);

	if(channel == IMG_CH0)
	{
		if(DC_enable == DC_ENABLE)
			LCDC_SetControlInfo(PAGE0, SET_CH0_ENABLE);
		else
			LCDC_SetControlInfo(PAGE0, SET_CH0_DISABLE);
	}
	else if(channel == IMG_CH1)
	{
		if(DC_enable == DC_ENABLE)
			LCDC_SetControlInfo(PAGE0, SET_CH1_ENABLE);
		else
			LCDC_SetControlInfo(PAGE0, SET_CH1_DISABLE);
	}
	else if(channel == IMG_CH2)
	{
		if(DC_enable == DC_ENABLE)
			LCDC_SetControlInfo(PAGE0, SET_CH2_ENABLE);
		else
			LCDC_SetControlInfo(PAGE0, SET_CH2_DISABLE);
	}

}

/*------------------------------------------------------------------
LCD Display on
-------------------------------------------------------------------*/
void DImage_Draw(void)
{
	dprintk("DImage_Draw \n");
	
	LCDC_SetControlInfo(PAGE0, SET_LCD_ENABLE);
	LCDC_SetControlInfo(PAGE0, SET_LCD_DISABLE);
}

void DDevice_initialize(void)
{
}
void DDevice_on(void)
{
}

void DDevice_off(void)
{
}
void DDevice_tearing(char onoff)
{
}


static int tccfb_ioctl(struct fb_info *info, unsigned int cmd, unsigned long arg)
{
	unsigned int channel;
	DImageInfoType		InfomationSet;
	DImageAddressType	AddressSet;
	DImageSetAllType	ImageSetAll;
	DImageChromaSetType ChromaSet;
	DImageAlphaSetType	AlphaSet;

	struct tccfb_info *fbi = info->par;

	dprintk("IOCTL [cmd:%d] [node:%d]\n", cmd, fbi->fb->node);

	if(fbi->fb->node == 0) channel= IMG_CH0;
	else if(fbi->fb->node == 1) channel= IMG_CH1;
	else if(fbi->fb->node == 2) channel= IMG_CH2;
	else 
		return 0;
	
	switch(cmd)
	{
		case TCC83X_LCDFB_IOCTL_DRAW:
			DImage_Draw();
			break;
			
		case TCC83X_LCDFB_IOCTL_CHANNEL_EN:
			DImage_channel_enable(channel, arg);
			break;

		case TCC83X_LCDFB_IOCTL_INFO_SET:
			copy_from_user(&InfomationSet, (const void __user *)arg, sizeof(DImageInfoType));
			DImage_Set_Info(channel, &InfomationSet);
			break;

		case TCC83X_LCDFB_IOCTL_ADDRESS_SET:
			copy_from_user(&AddressSet, (const void __user *)arg, sizeof(DImageAddressType));
			if(AddressSet.UseDefaultAddress)
			{
				DImage_Set_address(channel, (char *)fbi->map_dma, (char *)fbi->map_dma + SCREEN_WIDTH*SCREEN_HEIGHT, 
									(char *)fbi->map_dma + SCREEN_WIDTH*SCREEN_HEIGHT + (SCREEN_WIDTH*SCREEN_HEIGHT)/2);
			}
			else
			{
				DImage_Set_address(channel, AddressSet.addressY, AddressSet.addressU, AddressSet.addressV);
			}

			break;
			
		case TCC83X_LCDFB_IOCTL_IMAGESET_ALL:
			copy_from_user(&ImageSetAll, (const void __user *)arg, sizeof(DImageSetAllType));

			DImage_channel_enable(channel, DC_ENABLE);

			DImage_Set_Info(channel, &(ImageSetAll.ImgInfo));

			if(ImageSetAll.ImgAddress.UseDefaultAddress)	{
				DImage_Set_address(channel, (char *)fbi->map_dma, (char *)fbi->map_dma + SCREEN_WIDTH*SCREEN_HEIGHT, 
									(char *)fbi->map_dma + SCREEN_WIDTH*SCREEN_HEIGHT + (SCREEN_WIDTH*SCREEN_HEIGHT)/2);
			}
			else {
				DImage_Set_address(channel, ImageSetAll.ImgAddress.addressY, ImageSetAll.ImgAddress.addressU, ImageSetAll.ImgAddress.addressV);
			}

			if(ImageSetAll.Draw_enable)
				DImage_Draw();

			break;

		case TCC83X_LCDFB_IOCTL_ALPHA_EN:
			DImage_Alphablend_en(channel, arg);
			break;

		case TCC83X_LCDFB_IOCTL_ALPHA_SET:
			copy_from_user(&AlphaSet, (const void __user *)arg, sizeof(DImageAlphaSetType));
			DImage_Set_alphablending(channel, &AlphaSet);
			break;

		case TCC83X_LCDFB_IOCTL_CHROMA_EN:
			DImage_Chroma_en(channel, arg);
			break;

		case TCC83X_LCDFB_IOCTL_CHROMA_SET:
			copy_from_user(&ChromaSet, (const void __user *)arg, sizeof(DImageChromaSetType));
			DImage_Set_Chromakey(channel, &ChromaSet);
			break;

		case TCC83X_LCDD_IOCTL_INITIAL:
			DDevice_initialize();
			break;
		case TCC83X_LCDD_IOCTL_ONOFF:
			if(arg)
				DDevice_on();
			else
				DDevice_off();
			
			break;

		case TCC83X_LCDD_IOCTL_TEARING:
			DDevice_tearing(arg);
			break;			

		default:
			printk("TCC8300fb_ioctl");
			break;			
	}

	return 0;
}


static void tcc_stop_lcd_int_dd(void)
{
	// interrupt disable

	// LCD display off

}

static void tcc_set_reg_for_int_dd(void)
{
	LCDC_SetControlInfo(PAGE0, SET_LCD_ENABLE);
	LCDC_SetControlInfo(PAGE0, SET_LCD_DISABLE);
}

#if 0
static int tccfb_mmap(struct file *file, struct vm_area_struct *vma)
{
	if(remap_pfn_range(vma,vma->vm_start, vma->vm_pgoff , vma->vm_end - vma->vm_start, vma->vm_page_prot))
	{
		return -EAGAIN;
	}

	vma->vm_ops		= NULL;
	vma->vm_flags 	|= VM_IO;
	vma->vm_flags 	|= VM_RESERVED;
	
	return 0;
}
#endif 

//static struct tccfb_mach_info *mach_info;

/*
 *	tccfb_check_var():
 *	Get the video params out of 'var'. If a value doesn't fit, round it up,
 *	if it's too big, return -EINVAL.
 *
 */
static int tccfb_check_var(struct fb_var_screeninfo *var,
			       struct fb_info *info)
{
//	dprintk("check_var(var=%p, info=%p)\n", var, info);
	/* validate bpp */
	if (var->bits_per_pixel > 32)
		var->bits_per_pixel = 32;
	else if (var->bits_per_pixel < 16)
		var->bits_per_pixel = 16;
	
	/* set r/g/b positions */
	if (var->bits_per_pixel == 16) {
		var->red.offset 	= 11;
		var->green.offset	= 5;
		var->blue.offset	= 0;
		var->red.length 	= 5;
		var->green.length	= 6;
		var->blue.length	= 5;
		var->transp.length	= 0;
	} else if (var->bits_per_pixel == 32) {
		var->red.offset 	= 16;
		var->green.offset	= 8;
		var->blue.offset	= 0;
		var->transp.offset	= 24;
		var->red.length 	= 8;
		var->green.length	= 8;
		var->blue.length	= 8;
		var->transp.length	= 4;
	} else {
		var->red.length 	= var->bits_per_pixel;
		var->red.offset 	= 0;
		var->green.length	= var->bits_per_pixel;
		var->green.offset	= 0;
		var->blue.length	= var->bits_per_pixel;
		var->blue.offset	= 0;
		var->transp.length	= 0;
	}

	return 0;
}

/* tccfb_activate_var
 *
 * activate (set) the controller from the given framebuffer
 * information
*/

static void tccfb_activate_var(struct tccfb_info *fbi,
				   struct fb_var_screeninfo *var)
{
	unsigned int imgch = 0;
	unsigned int bpp_value;

//	dprintk("%s:fb%d var->bpp	= %d\n", __FUNCTION__, fbi->fb->node, var->bits_per_pixel);

	switch(fbi->fb->node)
	{
		case 0:
			imgch = IMG_CH0;
			break;

		case 1:
			imgch = IMG_CH1;
			break;
			
		case 2:
			imgch = IMG_CH2;
			break;

		default:
			dprintk("%s:fb%d Not Supported BPP!\n", __FUNCTION__, fbi->fb->node);
			return;
	}

	if(fbi->fb->var.bits_per_pixel == 16)
		bpp_value = (IMG_LIT_ENDIAN | IMG_RGB | IMG_BPP_565);
	else if(fbi->fb->var.bits_per_pixel == 32)
		bpp_value = (IMG_LIT_ENDIAN | IMG_RGB | IMG_BPP_888);
	else
	{
		dprintk("%s:fb%d Not Supported BPP!\n", __FUNCTION__, fbi->fb->node);
		return;
	}

	LCDC_SetImageInfo(SET_IMAGE_ALL, IMG_CH2, SCREEN_WIDTH ,SCREEN_HEIGHT, 
						0, 0, 0, 0, 0, 0, 
						bpp_value);

	return;
}


/*
 *      tccfb_set_par - Optional function. Alters the hardware state.
 *      @info: frame buffer structure that represents a single frame buffer
 *
 */
static int tccfb_set_par(struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	struct fb_var_screeninfo *var = &info->var;

//	dprintk("set_par\n");
	
	if (var->bits_per_pixel == 16)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else if (var->bits_per_pixel == 32)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else
		fbi->fb->fix.visual = FB_VISUAL_PSEUDOCOLOR;

	fbi->fb->fix.line_length = (var->xres*var->bits_per_pixel)/8;

	/* activate this new configuration */

	tccfb_activate_var(fbi, var);
	return 0;
}



static void schedule_palette_update(struct tccfb_info *fbi,
				    unsigned int regno, unsigned int val)
{
	unsigned long flags;

	local_irq_save(flags);

	local_irq_restore(flags);
}

/* from pxafb.c */
static inline unsigned int chan_to_field(unsigned int chan, struct fb_bitfield *bf)
{
	chan &= 0xffff;
	chan >>= 16 - bf->length;
	return chan << bf->offset;
}

static int tccfb_setcolreg(unsigned regno,
			       unsigned red, unsigned green, unsigned blue,
			       unsigned transp, struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	unsigned int val;

	/* dprintk("setcol: regno=%d, rgb=%d,%d,%d\n", regno, red, green, blue); */

	switch (fbi->fb->fix.visual) {
	case FB_VISUAL_TRUECOLOR:
		/* true-colour, use pseuo-palette */

		if (regno < 16) {
			u32 *pal = fbi->fb->pseudo_palette;

			val  = chan_to_field(red,   &fbi->fb->var.red);
			val |= chan_to_field(green, &fbi->fb->var.green);
			val |= chan_to_field(blue,  &fbi->fb->var.blue);

			pal[regno] = val;
		}
		break;

	case FB_VISUAL_PSEUDOCOLOR:
		if (regno < 256) {
			/* currently assume RGB 5-6-5 mode */

			val  = ((red   >>  0) & 0xf800);
			val |= ((green >>  5) & 0x07e0);
			val |= ((blue  >> 11) & 0x001f);

			//writel(val, S3C2410_TFTPAL(regno));
			schedule_palette_update(fbi, regno, val);
		}

		break;

	default:
		return 1;   /* unknown type */
	}

	return 0;
}


/**
 *      tccfb_blank
 *	@blank_mode: the blank mode we want.
 *	@info: frame buffer structure that represents a single frame buffer
 *
 *	Blank the screen if blank_mode != 0, else unblank. Return 0 if
 *	blanking succeeded, != 0 if un-/blanking failed due to e.g. a
 *	video mode which doesn't support it. Implements VESA suspend
 *	and powerdown modes on hardware that supports disabling hsync/vsync:
 *	blank_mode == 2: suspend vsync
 *	blank_mode == 3: suspend hsync
 *	blank_mode == 4: powerdown
 *
 *	Returns negative errno on error, or zero on success.
 *
 */
static int tccfb_blank(int blank_mode, struct fb_info *info)
{
	dprintk("blank(mode=%d, info=%p)\n", blank_mode, info);

	return 0;
}

static int tccfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	struct tccfb_info *fbi =(struct tccfb_info *) info->par;
	//	struct fb_var_screeninfo *var2 = var;


//	unsigned long irq_flags;
//	int base_update_count;

//	spin_lock_irqsave(&fb->lock, irq_flags);
//	printk(" %d, %d ",fbi->fb->var.xres,var->yoffset   );
	LCDC_SetImageBase(IMG_CH0, fbi->map_dma +fbi->fb->var.xres * 2 *var->yoffset  , 0, 0);

//	spin_unlock_irqrestore(&fb->lock, irq_flags);
	return 0;
}

#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc_fb_early_suspend(struct early_suspend *h)
{
	//struct tccfb_info *fb = container_of(h, struct tccfb_info, early_suspend);
	printk("\n***************** tcc_fb_early_suspend ****************\n");
//	writel(1, fb->reg_base + FB_SET_BLANK);
  LCD_Backlight_onoff(0);
}

static void tcc_fb_late_resume(struct early_suspend *h)
{
	//struct tccfb_info *fb = container_of(h, struct tccfb_info, early_suspend);
	printk("\n***************** tcc_fb_late_resume ****************\n");	
//        writel(0, fb->reg_base + FB_SET_BLANK);
  LCD_Backlight_onoff(1);
}
#endif
static struct fb_ops tccfb_ops = {
	.owner			= THIS_MODULE,
	.fb_check_var	= tccfb_check_var,
	.fb_set_par		= tccfb_set_par,
	.fb_blank		= tccfb_blank,
	.fb_setcolreg	= tccfb_setcolreg,
	.fb_fillrect	= cfb_fillrect,
	.fb_copyarea	= cfb_copyarea,
	.fb_imageblit	= cfb_imageblit,
       .fb_pan_display = tccfb_pan_display,
//	.fb_mmap		= tccfb_mmap,
	.fb_ioctl		= tccfb_ioctl,
};


/*
 * tccfb_map_video_memory():
 *	Allocates the DRAM memory for the frame buffer.  This buffer is
 *	remapped into a non-cached, non-buffered, memory region to
 *	allow palette and pixel writes to occur without flushing the
 *	cache.  Once this area is remapped, all virtual memory
 *	access to the video memory should occur at the new region.
 */
static int __init tccfb_map_video_memory(struct tccfb_info *fbi, int plane)
{
	dprintk("map_video_memory(fbi=%p), map_size:%08x\n", fbi, fbi->map_size);
//	if(plane==0)
#if defined(CONFIG_FB_TCC7901_DEMOBOARD)
		fbi->map_size = PAGE_ALIGN(fbi->fb->fix.smem_len + PAGE_SIZE);
#elif defined(CONFIG_FB_TCC7901_LMS350DF01)
                //XXX: why???
		fbi->map_size = PAGE_ALIGN(fbi->fb->fix.smem_len*2 + PAGE_SIZE);
#endif
//	else
///		fbi->map_size = PAGE_ALIGN(fbi->fb->fix.smem_len*4 + PAGE_SIZE);
	// allocate the memory space for frame buffer,
	// fbi->map_cpu store the virtual address
	// fbi->map_dma store the physical address
	fbi->map_cpu  = dma_alloc_writecombine(fbi->dev, fbi->map_size,
					       					&fbi->map_dma, GFP_KERNEL);

	fbi->map_size = fbi->fb->fix.smem_len;

	if (fbi->map_cpu) {
		/* prevent initial garbage on screen */
		dprintk("map_video_memory: clear %p:%08x\n", fbi->map_cpu, fbi->map_size);

		memset(fbi->map_cpu, 0x00, fbi->map_size);

		fbi->screen_dma		= fbi->map_dma;
		fbi->fb->screen_base	= fbi->map_cpu;
		fbi->fb->fix.smem_start  = fbi->screen_dma;

		// Set the LCD frame buffer start address
		switch (plane)
		{
			case 2:	// IMG2
				LCDC_SetImageBase(IMG_CH2, fbi->map_dma, 0, 0);
				break;

			case 1:	// IMG1
				LCDC_SetImageBase(IMG_CH1, fbi->map_dma, 0, 0);				
				break;
				
			case 0:	// IMG0
				LCDC_SetImageBase(IMG_CH0, fbi->map_dma, 0, 0);
				break;
		}
		dprintk("map_video_memory: dma=%08x cpu=%p size=%08x\n",
					fbi->map_dma, fbi->map_cpu, fbi->fb->fix.smem_len);
		
	}

	return fbi->map_cpu ? 0 : -ENOMEM;
}

static inline void tccfb_unmap_video_memory(struct tccfb_info *fbi)
{
	dma_free_writecombine(fbi->dev,fbi->map_size,fbi->map_cpu, fbi->map_dma);
}

static irqreturn_t tcc_lcd_handler(int irq, void *dev_id)
{
    unsigned int reg_temp = 0;
    unsigned long flags = 0;

    local_save_flags(flags);
    local_irq_disable();

    reg_temp = HwLSTATUS;
    BITSET(HwLSTATUS, Hw8);

    local_irq_restore(flags);

    if (reg_temp & Hw4) {
        tcc_set_reg_for_int_dd();
    }

    return IRQ_HANDLED;
}

static char driver_name[]="tccfb";
static int __init tccfb_probe(struct platform_device *pdev)
{
	struct tccfb_info *info;
	struct fb_info *fbinfo;
	int ret;
	int plane = 0;
	unsigned int dc_format = 0;
	
	dprintk("+probe\n");

	if(default_scn_depth[2] == 16)
		dc_format = (IMG_LIT_ENDIAN | IMG_RGB | IMG_BPP_565);
	else
		dc_format = (IMG_LIT_ENDIAN | IMG_RGB | IMG_BPP_888);

	switch(CONFIG_FB_TCC_DEVS)
	{
		case 3:
			LCDC_SetImageInfo(SET_IMAGE_ALL, IMG_CH2, SCREEN_WIDTH ,SCREEN_HEIGHT, 
								0, 0, 0, 0, 0, 0, 
								dc_format);
		case 2:
			LCDC_SetImageInfo(SET_IMAGE_ALL, IMG_CH1, SCREEN_WIDTH ,SCREEN_HEIGHT, 
								0, 0, 0, 0, 0, 0, 
								dc_format);
		case 1:
			LCDC_SetImageInfo(SET_IMAGE_ALL, IMG_CH0, SCREEN_WIDTH ,SCREEN_HEIGHT, 
								0, 0, 0, 0, 0, 0, 
								dc_format);
			break;
	}

	/*mach_info = pdev->dev.platform_data;
	if (mach_info == NULL) {
		dev_err(&pdev->dev,"no platform data for lcd, cannot attach\n");
		//return -EINVAL;
	}
    */
	for (plane = 0; plane < CONFIG_FB_TCC_DEVS; plane++)
	{
		fbinfo = framebuffer_alloc(sizeof(struct tccfb_info), &pdev->dev);

		info = fbinfo->par;
		info->fb = fbinfo;
		//platform_set_drvdata(pdev, fbinfo);

		strcpy(fbinfo->fix.id, driver_name);

		//info->mach_info		    = pdev->dev.platform_data;

		fbinfo->fix.type			= FB_TYPE_PACKED_PIXELS;
		fbinfo->fix.type_aux		= 0;
		fbinfo->fix.xpanstep		= 0;
		fbinfo->fix.ypanstep		= 1;
		fbinfo->fix.ywrapstep		= 0;
		fbinfo->fix.line_length 	= SCREEN_WIDTH * 2;
		fbinfo->fix.accel			= FB_ACCEL_NONE;

		fbinfo->var.nonstd			= 0;
		fbinfo->var.activate		= FB_ACTIVATE_NOW;

		fbinfo->var.accel_flags		= 0;


		fbinfo->fbops				= &tccfb_ops;
		fbinfo->flags				= FBINFO_FLAG_DEFAULT;

		fbinfo->var.xres			= SCREEN_WIDTH;
		fbinfo->var.xres_virtual	= SCREEN_WIDTH;
		fbinfo->var.yres			= SCREEN_HEIGHT;
		fbinfo->var.yres_virtual	= SCREEN_HEIGHT * 2;
		fbinfo->var.bits_per_pixel	= default_scn_depth[plane];
		fbinfo->var.height			= 54;
		fbinfo->var.width			= 95;


		fbinfo->var.red.offset = 11;
		fbinfo->var.red.length = 5;
		fbinfo->var.green.offset = 5;
		fbinfo->var.green.length = 6;
		fbinfo->var.blue.offset = 0;
		fbinfo->var.blue.length = 5;

		tccfb_check_var(&fbinfo->var, fbinfo);

		// the memory size that LCD should occupy
		fbinfo->fix.smem_len = fbinfo->var.xres *
								fbinfo->var.yres *	2 * 2;

#ifdef POST_SET
		if (plane == 0)
		{
		#if defined(_TCC79x_)
			tca_ckc_setperi(PERI_LCD, MHz, 156, DIRECTPLL0);
		#elif defined(_TCC83x_)
			tca_ckc_setperi(PERI_LCD, MHz, 78, DIRECTPLL0);
		#endif
			msleep(1);
		}
#endif

		/* Initialize video memory */
		ret = tccfb_map_video_memory(info, plane);
		if (ret) {
			printk( KERN_ERR "Failed to allocate video RAM: %d\n", ret);
			ret = -ENOMEM;
		}

		ret = register_framebuffer(fbinfo);
		
		if (ret < 0) {
			printk(KERN_ERR "Failed to register framebuffer device: %d\n", ret);
			goto free_video_memory;
		}

		tccfb_set_par(fbinfo);
		msleep(1);
		
		if (plane == 0)	// top layer
			if (fb_prepare_logo(fbinfo,	FB_ROTATE_UR)) {
			/* Start display and show logo on boot */
			fb_set_cmap(&fbinfo->cmap, fbinfo);

			printk("fb_show_logo\n");
			fb_show_logo(fbinfo, FB_ROTATE_UR);
        }

		dprintk(KERN_INFO "fb%d: %s frame buffer device\n",	fbinfo->node, fbinfo->fix.id);
	}

	LCDC_SetControlInfo(PAGE0, SET_CH0_ENABLE | SET_CH1_DISABLE | SET_CH2_DISABLE);
	LCDC_SET_INTTERUPT(1);
	
    request_irq(IRQ_LCD,   tcc_lcd_handler,
                IRQF_SHARED,  "TCC_LCD",
                tcc_lcd_handler);


#ifdef CONFIG_HAS_EARLYSUSPEND
	info->early_suspend.suspend = tcc_fb_early_suspend;
	info->early_suspend.resume = tcc_fb_late_resume;
	info->early_suspend.level = EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&info->early_suspend);
#endif
	dprintk("-probe\n");
	return 0;

free_video_memory:
	tccfb_unmap_video_memory(info);
	dprintk("-probe:Error\n");
	return ret;
}

/* tccfb_stop_lcd
 *
 * shutdown the lcd controller
*/

static void tccfb_stop_lcd(struct tccfb_info *fbi)
{
	dprintk("stop_lcd\n");
	return;
}

/*
 *  Cleanup
 */
static int tccfb_remove(struct platform_device *pdev)
{
	struct fb_info	   *fbinfo = platform_get_drvdata(pdev);
	struct tccfb_info *info = fbinfo->par;

	dprintk("remove\n");

	tcc_stop_lcd_int_dd();
	free_irq(IRQ_LCD, tcc_lcd_handler);

	tccfb_stop_lcd(info);
	tca_ckc_setperi(PERI_LCD, MHz, 0, DIRECTPLL0);

	msleep(1);

	tccfb_unmap_video_memory(info);

	//release_mem_region((unsigned long)S3C24XX_VA_LCD, S3C24XX_SZ_LCD);
	unregister_framebuffer(fbinfo);
#ifdef CONFIG_HAS_EARLYSUSPEND
        unregister_early_suspend(&info->early_suspend);
#endif
	return 0;
}

#ifdef CONFIG_PM

/* suspend and resume support for the lcd controller */

static int tccfb_suspend(struct platform_device *dev, pm_message_t state)
{
	dprintk("suspend\n");
	tca_bkl_powerdown();

	return 0;
}

static int tccfb_resume(struct platform_device *dev)
{
	dprintk("resume\n");
	tca_bkl_powerup();

	return 0;
}

static void tccfb_shutdown(struct platform_device *dev)
{
	pm_message_t state = {0};
	tccfb_suspend(dev, state);
}

#else
#define tccfb_suspend	NULL
#define tccfb_resume	NULL
#define tccfb_shutdown	NULL
#endif

static struct platform_driver tccfb_driver = {
	.probe		= tccfb_probe,
	.remove		= tccfb_remove,
	.suspend	= tccfb_suspend,
	.shutdown	= tccfb_shutdown,
	.resume		= tccfb_resume,
	.driver		= {
		.name	= "tccxxx-lcd",
		.owner	= THIS_MODULE,
	},
};

int __devinit tccfb_init(void)
{
	return platform_driver_register(&tccfb_driver);
}

static void __exit tccfb_cleanup(void)
{
	platform_driver_unregister(&tccfb_driver);
}


module_init(tccfb_init);
module_exit(tccfb_cleanup);

MODULE_AUTHOR("Mac Chen <604sworder@163.com>");
MODULE_DESCRIPTION("Framebuffer driver for TCCxxx");
MODULE_LICENSE("GPL");
