/****************************************************************************
 *   FileName    : tcc_tsif_parallel_hwset.h
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifndef __TCC_TSIF_PARALLEL_HWSET_H__
#define __TCC_TSIF_PARALLEL_HWSET_H__

#include <mach/memory.h>

//#define VPU_DMA_USE
#define TSIF_USE_DEBUG			0

#define TSIF_CH 				0		//tsif use ch0
#define TSIF_DMA_CONTROLLER		1		//tsif use dma controller 1
#define TSIF_DMA_CH				2		//tsif use dma ch2-n (n:0,1,2)

#define HwTSIF0_BASE			*(volatile unsigned long *)0xF053B000
#define HwTSIF1_BASE			*(volatile unsigned long *)0xF053B100

#define TSIF_BYTE_SIZE			4		// 1, 2, 4byte
#define TSIF_RXFIFO_THRESHOLD	1		// 0-7 (0=>8 depth)

#define MPEG_PACKET_SIZE 		188
#ifdef VPU_DMA_USE
	#define TSIF_DMA_SIZE 			0x0
	#define TSIF_MASK_SIZE			0x000000
#else
	//20101025 koo test
	//#define TSIF_DMA_SIZE 			(CONSISTENT_DMA_SIZE >> 1)
	#define TSIF_DMA_SIZE 			(CONSISTENT_DMA_SIZE / 8)
	#define TSIF_MASK_SIZE			((0x10000000 - TSIF_DMA_SIZE) >> 4)
#endif

//#define TSIF_DMA_HOPE_CNT		((0x10000 - (0x10000 % MPEG_PACKET_SIZE)) / 4)
#define TSIF_DMA_HOPE_CNT(cnt)  (((cnt) * MPEG_PACKET_SIZE) / (TSIF_BYTE_SIZE * TSIF_RXFIFO_THRESHOLD))
#define TSIF_DMA_INCREASE_SIZE	0x4

#define WAIT_TIME_FOR_DMA_DONE	(1000 * 8)

#pragma pack(push, 4)
struct tcc_tsif_regs {
    volatile unsigned long TSDI, TSCR, TSPID, TSCTRL, TSSTS;
};
#pragma pack(pop)


struct tea_dma_buf {
    void *v_addr;
    unsigned int dma_addr;
    int buf_size; // total size of DMA
};

typedef struct tcc_tsif_parallel_handle tcc_tsif_parallel_handle_t;
typedef int (*dma_alloc_f)(struct tea_dma_buf *tdma, unsigned int size);
typedef void (*dma_free_f)(struct tea_dma_buf *tdma);

struct tcc_tsif_parallel_handle {
    volatile struct tcc_tsif_regs *regs;
    struct tea_dma_buf rx_dma;
	//int flag;
    int irq;
    void *private_data;
    int id;
	int dma_controller;
	int dma_ch;
	//int is_slave;
	int gpio_set;

    int (*dma_stop)(struct tcc_tsif_parallel_handle *h);
    int (*dma_start)(struct tcc_tsif_parallel_handle *h);
    void (*clear_fifo_packet)(struct tcc_tsif_parallel_handle *h);
	void (*tsif_set)(struct tcc_tsif_parallel_handle *h);
	void (*tsif_isr)(struct tcc_tsif_parallel_handle *h);
	
    dma_alloc_f tea_dma_alloc; // tea function.
    dma_free_f tea_dma_free; // tea function.

	//int clk;	// Mhz
	
    /* add for slave */
    unsigned int dma_total_packet_cnt, dma_intr_packet_cnt;
    int q_pos, cur_q_pos, prev_q_pos;
    int dma_total_size;

	int dma_phy_maxaddr;
	int dma_phy_rdpos;
	int dma_phy_curpos;
	int dma_virt_maxaddr;
	int dma_virt_rdpos;
	int dma_virt_curpos;
	int dma_recv_size;
	int dma_intr_cnt;
	
	unsigned char msb_first;
	unsigned char big_endian;
	unsigned char serial_mode;
	unsigned char clk_polarity;
	unsigned char valid_polarity;
	unsigned char sync_polarity;
	unsigned char sync_delay;
	unsigned int dma_phy_addr;
	unsigned int dma_phy_size;
};

extern int tcc_tsif_parallel_init(struct tcc_tsif_parallel_handle *h, volatile struct tcc_tsif_regs *regs, 
		dma_alloc_f tea_dma_alloc, dma_free_f tea_dma_free, int dma_size, int tsif_ch, int dma_controller, int dma_ch);
extern void tcc_tsif_parallel_clean(struct tcc_tsif_parallel_handle *h);

#endif /*__TCC_TSIF_PARALLEL_HWSET_H__*/
