/****************************************************************************
 *   TCC Version 0.6
 *   Copyright (c) telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
****************************************************************************/

#define	TCC83XX

#define	IO_CKC_EnableBUS_I2C()		(BITSET(HwBCLKCTR, HwBCLKCTR_I2C_ON))
#define	IO_CKC_DisableBUS_I2C()		(BITCLR(HwBCLKCTR, HwBCLKCTR_I2C_ON))

#define IO_CKC_Fxin 120000

/* =============================
   General Bit Operator
   ============================= */
// Bit manipulation macro that is modifying its argument. (task type)
#define	BITSET(X, MASK)				( (X) |= (unsigned long)(MASK) )
#define	BITSCLR(X, SMASK, CMASK)	( (X) = ((((unsigned long)(X)) | ((unsigned long)(SMASK))) & ~((unsigned long)(CMASK))) )
#define	BITCSET(X, CMASK, SMASK)	( (X) = ((((unsigned long)(X)) & ~((unsigned long)(CMASK))) | ((unsigned long)(SMASK))) )
#define	BITCLR(X, MASK)				( (X) &= ~((unsigned long)(MASK)) )
#define	BITXOR(X, MASK)				( (X) ^= (unsigned long)(MASK) )

/************************************************************************
 * I2C define
 ***********************************************************************/
#define		i2c_ack		0					// i2c acknowledgement
#define		i2c_noack		1					// i2c no acknowledgement
#define		i2c_err		0					// i2c error occurs
#define		i2c_ok			1					// i2c error does not occurs


// I2C-bus standard mode timing setting
#define		i2c_Tbuf		400		//200
#define		i2c_Tsetup		8		//4//8		// above min 250ns
#define		i2c_Thold		200		//100		// the fm_Thold is a basic time as above min 5us
#define		i2c_Tcatch		100		//50		// the half of the i2c_Thold

#define 		I2C_SUBADDR_NOUSE 	-1

// Message 
#define I2C_FREE	0
#define I2C_USED	1


enum 
{
	I2C_CODEC,
	I2C_TVOUT,
	I2C_TOUCH,
	I2C_MAX_VECTOR
} ;

enum
{
	I2C_CLK,
	I2C_DATA,
	I2C_SIGNAL_BUS
};

#define		i2c_wr			(*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr+0x04) |= (I2C_Vector[I2C_DATA].port));
#define		i2c_rd			(*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr+0x04) &= ~(I2C_Vector[I2C_DATA].port));
#define		i2c_clk_hi		(*(volatile unsigned long int *)(I2C_Vector[I2C_CLK].tgtaddr) |= (I2C_Vector[I2C_CLK].port));
#define		i2c_clk_lo		(*(volatile unsigned long int *)(I2C_Vector[I2C_CLK].tgtaddr) &= ~(I2C_Vector[I2C_CLK].port));	
#define		i2c_data_hi		(*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr) |= (I2C_Vector[I2C_DATA].port)); 	   
#define		i2c_data_lo		(*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr) &= ~(I2C_Vector[I2C_DATA].port)); 	 


//**********************************************************************
#define codec_slaveaddr	0x34	//WM8731 / TLV320 CODEC slave addr
#define touch_slaveaddr	0x90	//TSC2003 A/D conveter slave addr


typedef struct I2C_SW_READ_s
{
	unsigned int i2ctgt;
	unsigned char SlaveAddr;
	char subaddr;
	unsigned char *ptr;
	unsigned char bytes;
	unsigned char i2c_return;
} I2C_SW_READ_t;

typedef struct I2C_SW_WRITE_s
{
	unsigned int i2ctgt;
	unsigned char SlaveAddr;
	char subaddr;
	unsigned char *ptr;
	unsigned char bytes;
	unsigned char i2c_return;
} I2C_SW_WRITE_t;


//***********************************************************************
#ifdef TCC78X
#define		GPIO_A	0xF005A020	// (unsigned long int)(&HwGPADAT)
#define		GPIO_B	0xF005A040	// (unsigned long int)(&HwGPBDAT)
#define		GPIO_C	0xF005A060	// (unsigned long int)(&HwGPCDAT)
#define		GPIO_D	0xF005A080	//(unsigned long int)(&HwGPDDAT)
#define		GPIO_E	0xF005A0A0	//(unsigned long int)(&HwGPEDAT)
#endif
#if defined(TCC82XX) || defined(TCC83XX)
#define		GPIO_A	(unsigned long int)(&HwGDATA_A)
#define		GPIO_B	(unsigned long int)(&HwGDATA_B)
#define		GPIO_C	(unsigned long int)(&HwGDATA_C)
#define		GPIO_E	(unsigned long int)(&HwGDATA_E)
#define		GPIO_F	(unsigned long int)(&HwGDATA_F)
#define		GPIO_G	(unsigned long int)(&HwGDATA_G)
#endif

void CODEC_IIC_INIT(void);
void CODEC_SEND_CMD(unsigned char hb, unsigned char lb);


