//
// Copyright (c) Telechips Corporation.  All rights reserved.
//
//    

#include <linux/kernel.h>
#include <asm/string.h>

#include <asm/arch/tcc83x_virt.h>
#include "tcc83xx_i2s.h"
#include "tcc83xx_i2c.h"


static void InitOutputDMA(void);
static void InitInputDMA(void);

int I2S_Init(void)
{
	//Disable noise
	memset(&HwDADO_L0, 0, 32);
	
	HwDAVC = 0;
	HwDAMR = (0
		| HwDAMR_BPS_EN
		| HwDAMR_LPS_EN
		//| HwDAMR_NMD_EN
		//| HwDAMR_RXE_EN
		| HwDAMR_TXS_LSB16
		| HwDAMR_RXS_LSB16
		//| HwDAMR_EN_ON
		//| HwDAMR_TE_EN
		//| HwDAMR_RE_EN
		//| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
		| HwDAMR_SM_INT
		| HwDAMR_BM_INT
		| HwDAMR_FM_INT
		//| HwDAMR_CC_EN
		//| HwDAMR_BD_DIV4
		| HwDAMR_FD_DIV64
		//| HwDAMR_BP_NEGA
		//| HwDAMR_CM_EN
		//| HwDAMR_MM_EN
		//| HwDAMR_LB_EN
		);
	
	//CKC DAI clock enable
	BITSET(HwBCLKCTR1, HwBCLKCTR_DAI_ON);
	HwPCLKCFG7 = 0;
	BITCLR(HwPCLKCFG6, HwPCLKCFG6_EN1_ON);
	//TX enable
	HwDAMR |= HwDAMR_TE_EN;

	//DAI enable
	HwDAMR |= HwDAMR_EN_ON;
	
	//Clear DAI TX INT
	BITCLR(HwCLR, HwCLR_DAITX);
	BITCLR(HwCLR, HwCLR_DAIRX);

	//Enable Audio DMA clock
	HwSWRESET1|= HwSWRESET1_AUDIODMA_ON;
	HwSWRESET1 &= ~HwSWRESET1_AUDIODMA_ON;
	BITSET(HwBCLKCTR1, HwBCLKCTR1_AudioDMA_ON);

	//Config DMA Interrupt
        HwIEN |= HwIEN_ADMA;
	HwSEL |= HwSEL_ADMA;
	HwMODE &= ~HwMODE_ADMA;
	HwSYNC |= HwSYNC_ADMA;
	HwPOL &= ~HwPOL_ADMA;

	SetI2SClockRate(IS2LRCLK_44100);


	//----- 4. For power management purposes, shut the clocks off! -----
	StopI2SOutputClock();

	InitOutputDMA();
	InitInputDMA();
    
	return(1);
}

// Set the start address & length of DMA source buffer
void SetDMASourceBuffer(unsigned int sAddr, unsigned int b_length)
{
    unsigned long DMABuffer = 0;

    printk("%s, addr[0x%08X], len[%d]\n", __func__, sAddr, b_length);
    HwChCtrl &= ~HwChCtrl_DTEN_EN;

    DMABuffer = 0xFFFFFF00 / (((b_length) >> 4) << 8);
    DMABuffer = DMABuffer * (((b_length) >> 4) << 8);

    HwTxDaParam = DMABuffer | 4;

    // generate interrupt when half of buffer was played
    HwTxDaTCnt = ((b_length / 2) >> 0x04) - 1;

    printk("[%s] HwTxDaParam [0x%X]\n", __func__, (int)(DMABuffer | 4));
    printk("[%s] HwTxDaTCnt [%d]\n", __func__, ((b_length / 2) >> 0x04) - 1);

    HwTxDaSar = sAddr;

    HwTransCtrl |= (HwTransCtrl_TCN_EN
                    | HwTransCtrl_DTRPT_EN
                    | HwTransCtrl_DTBSIZE_4
                    | HwTransCtrl_DTWSIZE_32
                    | HwTransCtrl_DTLCK);

    BITCLR(HwRptCtrl, HwRptCtrl_DRI_EN);

    HwChCtrl |= (HwChCtrl_DTEN_EN
                | HwChCtrl_DTDW_16
                | HwChCtrl_DTIEN);
}

// Set the start address & length of input DMA dest buffer
void SetInputDMADestBuffer(unsigned int sAddr, unsigned int b_length)
{
    unsigned long DMABuffer = 0;

    printk("%s, addr[0x%08X], len[%d]\n", __func__, sAddr, b_length);
    HwChCtrl &= ~HwChCtrl_DREN_EN;

    DMABuffer = 0xFFFFFF00 / (((b_length) >> 4) << 8);
    DMABuffer = DMABuffer * (((b_length) >> 4) << 8);

    HwRxDaParam = DMABuffer | 4;

    // generate interrupt when half of buffer was filled
    HwRxDaTCnt = ((b_length / 2) >> 0x04) - 1;

    printk("[%s] HwRxDaParam [0x%X]\n", __func__, (int)DMABuffer | 4);
    printk("[%s] HwRxDaTCnt [%d]\n", __func__, ((b_length / 2) >> 0x04) - 1);

    HwRxDaDar = sAddr;

    HwTransCtrl |= (HwTransCtrl_RCN_EN
                    | HwTransCtrl_DRRPT_EN
                    | HwTransCtrl_DRBSIZE_4
                    | HwTransCtrl_DRWSIZE_32
                    | HwTransCtrl_DRLCK);

    BITCLR(HwRptCtrl,HwRptCtrl_DRI_EN);

    HwChCtrl |= (HwChCtrl_DREN_EN
                 | HwChCtrl_DRDW_16
                 | HwChCtrl_DRIEN);
}

static void InitOutputDMA(void)
{
    unsigned long DMABuffer = 0;

    HwChCtrl &= ~HwChCtrl_DTEN_EN;

    DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_OUT_BUF_SIZE) >> 4) << 8);
    DMABuffer = DMABuffer * (((AUDIO_DMA_OUT_BUF_SIZE) >>4)<<8);

    HwTxDaParam = DMABuffer | 4;

    // generate interrupt when half of buffer was played
    HwTxDaTCnt = ((AUDIO_DMA_OUT_PAGE_SIZE/2)>>0x04)-1;

    HwTransCtrl |= (0
                   //|HwTransCtrl_DTTRG_SL
                   | HwTransCtrl_TCN_EN
                   | HwTransCtrl_DTRPT_EN
                   | HwTransCtrl_DTBSIZE_4
                   | HwTransCtrl_DTWSIZE_32);

    BITCLR(HwRptCtrl,HwRptCtrl_DRI_EN);

    HwChCtrl |= (0
                | HwChCtrl_DTEN_EN
                | HwChCtrl_DTDW_16
                | HwChCtrl_DTIEN);
}

static void InitInputDMA(void)
{
    unsigned long DMABuffer = 0;

    printk("%s\n",__func__);
    HwChCtrl &= ~HwChCtrl_DREN_EN;

    DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_IN_BUF_SIZE) >> 4) << 8);
    DMABuffer = DMABuffer * (((AUDIO_DMA_IN_BUF_SIZE) >>4)<<8);

    HwRxDaParam = DMABuffer | 4;

    // generate interrupt when half of buffer was played
    //HwTxDaTCnt = ((AUDIO_DMA_IN_PAGE_SIZE/2)>>0x04)-1;
    HwRxDaTCnt = ((AUDIO_DMA_IN_PAGE_SIZE/2)>>0x04)-1;

    HwTransCtrl |= (0//|HwTransCtrl_DTTRG_SL
                    | HwTransCtrl_RCN_EN
                    | HwTransCtrl_DRRPT_EN
                    | HwTransCtrl_DRBSIZE_4
                    | HwTransCtrl_DRWSIZE_32);

    BITCLR(HwRptCtrl,HwRptCtrl_DRI_EN);

    HwChCtrl |= (0
                 | HwChCtrl_DREN_EN
                 | HwChCtrl_DRDW_16
                 //| HwChCtrl_DRIEN	
                 | Hw2);
}

int I2S_Deinit(void)
{
    //----- 1. Stop the I2S clocks -----
    StopI2SOutputClock();
    return 1;
}


#if 0
int I2S_WriteData(unsigned long *lpBuff, unsigned long dwLen)
{
	static unsigned long waitCount	= 0;
	static unsigned long i			= 0;

	//----- 1. Check the parameters -----
	if(!lpBuff || (dwLen == 0))
	{
        goto WRITE_ERROR;
	}

//	StartI2SClock();


	StopI2SOutputClock();
	return 1;

WRITE_ERROR:
	StopI2SOutputClock();
	return 0;
}
#endif

//------------------------------------ Helper Routines ------------------------------------

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function:		StartI2SClock()

Description:	Enables the I2S clock that drives the audio codec chip.

Returns:		N/A
-------------------------------------------------------------------*/
void StartI2SOutputClock(void)
{
	//TX enable
	HwChCtrl |= HwChCtrl_DTEN_EN;
	HwDAMR |= HwDAMR_TE_EN;

	return;
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Function:		StopI2SClock()

Description:	Disables the I2S clock that drives the audio codec chip.

Returns:		N/A
-------------------------------------------------------------------*/
void StopI2SOutputClock(void)
{
	//TX disable
	HwDAMR &= ~HwDAMR_TE_EN;
	HwChCtrl &= ~HwChCtrl_DTEN_EN;

	// reset HOP counter
	//HwTxDaTCnt = 0;
	//printk("StopI2SOutputClock()\n");
		
	return;
}

void StartI2SInputClock(void)
{
#if 1
printk("%s\n",__func__);
	//RX enable
	HwDAMR |= HwDAMR_RE_EN;
	//HwDAMR |= HwDAMR_RE_EN | Hw0;
	HwChCtrl |= HwChCtrl_DREN_EN;
        //HwCHCTRL1 |= HwCHCTRL_EN_EN;
	return;
#else
	HwDAMR |= HwDAMR_RE_EN;
	HwChCtrl |= HwChCtrl_DREN_EN;
#endif
}

void StopI2SInputClock(void)
{
#if 1
printk("%s\n",__func__);
	//RX disable
	//HwDAMR &= ~HwDAMR_RE_EN;
	HwDAMR &= ~(HwDAMR_RE_EN | Hw0);
	HwChCtrl &= ~HwChCtrl_DREN_EN;
        //HwCHCTRL1 &= ~HwCHCTRL_EN_EN;

	return;
#else
	HwDAMR &= ~HwDAMR_RE_EN;
        HwCHCTRL1 &= ~HwCHCTRL_EN_EN;
#endif
}

void EnableDAI(void)
{
	HwDAMR |= HwDAMR_EN_ON;
}

void DisableDAI(void)
{
	HwDAMR &= ~HwDAMR_EN_ON;
}

void SetI2SClockRate(unsigned long ClockRate)
{
    unsigned int dwPclk;
    unsigned int dwM, dwS, dwP, dwDiv;
    unsigned long uTmpL;

    HwPCLKCFG6 &= 0xFFFF;
    dwPclk = HwPLL0CFG;
    dwS = (dwPclk >> 16)&0x03;
    dwM = (dwPclk >> 8) &0xff;
    dwP = (dwPclk) & 0x3f;

    dwPclk = ((12000000 * dwM)/dwP) >> dwS;

    uTmpL = (unsigned long)ClockRate * 256;
    dwDiv = ((dwPclk / 2) / uTmpL) - 1;
    printk("dwPclk=%d, dwDiv=0x%x\n", dwPclk, dwDiv);
    HwPCLKCFG6 = HwPCLKCFG6_EN1_ON + HwPCLKCFG6_SEL1_PLL0 + HwPCLKCFG6_P1_ON + (dwDiv<<16);

    return;
}




