
/****************************************************************************
 *   FileName    :tca_tcc250dac.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#ifdef __KERNEL__
#   if defined(_TCC79x_)
#       include <asm/arch/tcc79x.h>
#   elif defined(_TCC83x_)
#       include <asm/arch/tcc83x_virt.h>
#   endif
#else
#   include <windows.h>
#   if defined(_TCC79x_)
#       include "TCC79x_Virtual.h"
#   elif defined(_TCC83x_)
#       include "TCC83x_Virtual.h"
#	elif defined(_TCC80x_)
#		include "TCC80x_Virtual.h"
#   endif
#endif

#include "tca_tcc250dac.h"

#ifdef _TCC250_INCLUDE_

void tca_tcc250dac_delay(unsigned int count)
{

	#ifdef __KERNEL__
	volatile unsigned int i;
		count = count * 5000;
    		for (i=0;i<count;i++);
	#else
		Sleep(count);
	#endif

}

#ifdef __KERNEL__
void tca_tcc250dac_initi2s(void)
{
    HwDAMR = (0        
#if defined(_TCC83x_) || defined(_TCC80x_)
              | HwDAMR_BPS_EN
              | HwDAMR_LPS_EN
#endif
              //| HwDAMR_NMD_EN
              //| HwDAMR_RXE_EN
              | HwDAMR_TXS_LSB16
              | HwDAMR_RXS_LSB16
              //| HwDAMR_EN_ON
              //| HwDAMR_TE_EN
              //| HwDAMR_RE_EN
              //| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
#ifdef MCLK_TO_TCC250_INCLUDE
              | HwDAMR_SM_INT
#endif
#ifdef FRAME_TO_TCC250_INCLUDE
              | HwDAMR_BM_INT
              | HwDAMR_FM_INT
#endif
              //| HwDAMR_CC_EN
              //| HwDAMR_BD_DIV4
              | HwDAMR_FD_DIV64
              //| HwDAMR_BP_NEGA
              //| HwDAMR_CM_EN
              //| HwDAMR_MM_EN
              //| HwDAMR_LB_EN
              );
}
#else
void tca_tcc250dac_initi2s(void)
{
	HwDAVC = 0;
	HwDAMR = 0
#if defined(_TCC79x_)
		//| Hw31 //Slave Mode
		//| Hw30 //Slave Mode
#elif defined(_TCC83x_) || defined(_TCC80x_)
		| Hw31 //Slave Mode
		| Hw30 //Slave Mode
#endif
		//| HwDAMR_TE_EN
		//| HwDAMR_RE_EN
		//| HwDAMR_MD_MSB			// 0: IIS, 1:MSB
		#ifdef MCLK_TO_TCC250_INCLUDE
		| HwDAMR_SM_INT
		#endif
		
		#ifdef FRAME_TO_TCC250_INCLUDE
		| HwDAMR_BM_INT
		| HwDAMR_FM_INT
		#endif
		//| HwDAMR_CC_EN
		//| HwDAMR_BP_NEGA
		//| HwDAMR_CM_EN
		//| HwDAMR_MM_EN
		//| HwDAMR_LB_EN
		//| HwDAMR_BD_DIV4
		| HwDAMR_FD_DIV64
		| HwDAMR_TXS_LSB16
		| HwDAMR_RXS_LSB16
		;

 #if defined(_TCC79x_)
    //CKC DAI clock enable
    BITSET(HwPCK_DAI, HwPCK_EN_EN);

    //DAI enable
    HwDAMR |= HwDAMR_EN_ON;

    //enable DMA Interrupt
    //HwMODE &= ~HwINT_DMA; // Edge trigger
    HwMODE |= HwINT_DMA;   //Level trigger
    HwSEL |= HwINT_DMA; 
    HwIRQMSK |= HwINT_DMA; 
#elif  defined(_TCC83x_) || defined(_TCC80x_)
    //CKC DAI clock enable
   // BITSET(HwPCLKCFG6, Hw31);

    //DAI enable
    HwDAMR |= HwDAMR_EN_ON;

    //enable DMA Interrupt
    //HwMODE &= ~HwIEN_DMA; // Edge trigger
    HwMODE |= HwIEN_DMA;   //Level trigger
    HwSEL  |= HwIEN_DMA; 
    HwIRQ  |= HwIEN_DMA; 
#endif

   // return(TRUE);
}
#endif

/*****************************************************************************
* Function Name : tca_tcc250dac_setana()
******************************************************************************
* Desription  	: Set to LDO_ANA Voltage for Audio Codec
* Parameter   	: mV   - Seeting Voltage [mV], Max 3600mV, Min 3300mV
*                 DVS  - Digital Voltage Scaling Enable
* Return      	: None
* Note          : 
******************************************************************************/
void tca_tcc250dac_setana(void *hI2C, unsigned short mV)
{
//#ifdef ANA_LDO_INCLUDE
	if(mV < 2700) mV = 2700;
	else if(mV > 3300) mV = 3300;

#ifdef _TCC250_DVS_INCLUDE
	tea_wave_i2c_write(hI2C,TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// DVS Mode
	tca_tcc250dac_delay(2);
	tea_wave_i2c_write(hI2C,TCC250_ANA_CTRL_REG,   (0x10 + ((mV / 50) - 54)));	// DVS Mode
#else
	tea_wave_i2c_write(hI2C,TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// Normal Mode
	tca_tcc250dac_delay(2);
	tea_wave_i2c_write(hI2C,TCC250_ANA_CTRL_REG,   (0x20 + ((mV / 50) - 54)));	// Normal Mode
//#endif
#endif

}

/*****************************************************************************
* Function Name : TCC250_Initial()
******************************************************************************
* Desription   	 	: The function for H/W Codec Initialize function
* Parameter   	: None
* Return        	: None
******************************************************************************/
void tca_tcc250dac_initcodec(void *hI2C)	
{

 	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write(hI2C, TCC250_PUP_OFF_OUTPUT_REG, 	0x0F);	// Enable ENACLDO and Path#1

#ifdef	TCC250_CODEC_I2S
	tea_wave_i2c_write(hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
	tea_wave_i2c_write(hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif

#ifdef	ANA_LDO_INCLUDE
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x41);	// Enable ENACLDO and Path#1
	tca_tcc250dac_delay(5);
#else
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x01);	// Disable ENACLDO and Path#1
	tca_tcc250dac_delay(5);
#endif
//	tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,		0x00);	// Microphone Control : Bias Off, Det off
//	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x00);	//Enable High Pass Filter
	
//#ifdef ANA_LDO_INCLUDE
	tca_tcc250dac_setana(hI2C, 3300);
//#endif

}
#if 0
/*****************************************************************************
* Function Name : TCC250_OutPutMute()
******************************************************************************
* Desription  	: The function for H/W Codec Mute Control function
* Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
* Return      	: None
******************************************************************************/
void TCC250_OutPutMute(void *hI2C, unsigned char ucOnOff)
{
	if(ucOnOff)	// Mute
	{
		tea_wave_i2c_write(void *hI2C,TCC250_LRMUTE_REG, 		0x63);
	}
	else	// Mute Off			
	{
		tea_wave_i2c_write(void *hI2C,TCC250_LRMUTE_REG, 		0x00);
	}
}
#endif

/*****************************************************************************
* Function Name : tca_tcc250dac_getcodecpath()
******************************************************************************
* Desription   	 	: The function for TCC250 Codec Mode
* Parameter   	: None
* Return        	: 
******************************************************************************/
unsigned char tca_tcc250dac_getcodecpath(void *hI2C)
{
	unsigned char	Reg;

	Reg = tea_wave_i2c_read(hI2C, TCC250_AUTOSTART1_REG);

	return Reg;
}

/*****************************************************************************
* Function Name : tca_tcc250dac_getcodecmute()
******************************************************************************
* Desription   	 	: The function for TCC250 Codec Mode
* Parameter   	: None
* Return        	: 
******************************************************************************/
unsigned char tca_tcc250dac_getcodecmute(void *hI2C)
{
	unsigned char	Reg;

	Reg = tea_wave_i2c_read(hI2C, TCC250_LRMUTE_REG);

	return Reg;
}


/*****************************************************************************
* Function Name : tca_tcc250dac_setinputmute()
******************************************************************************
* Desription  	: The function for H/W Codec Mute Control function
* Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
* Return      	: None
******************************************************************************/
void tca_tcc250dac_setinputmute(void *hI2C, unsigned char ucOnOff)
{
	unsigned int vMute = 0;
	unsigned int vPath = 0;

	if(ucOnOff)	// Mute
	{
		tea_wave_i2c_write(hI2C, TCC250_LRMUTE_REG, 		0x1C);
	}
	else	// Mute Off			
	{
		vPath = tca_tcc250dac_getcodecpath(hI2C) & 0x1F;
		if( vPath == 0x02 ||  vPath == 0x04 ||  vPath == 0x07 ||  vPath == 0x0A ||   vPath == 0x12  ||vPath == 0x14   
			||  vPath == 0x04 ||  vPath == 0x17 ||  vPath == 0x19 ||  vPath == 0x1B   

		)
		{
				vMute = tca_tcc250dac_getcodecmute(hI2C);				
				vMute |= 0x3;
				tea_wave_i2c_write(hI2C, TCC250_LRMUTE_REG, 		vMute);
		}
		else
			tea_wave_i2c_write(hI2C, TCC250_LRMUTE_REG, 		0x00);
	}
}
/*****************************************************************************
* Function Name : tca_tcc250dac_setoutputmute()
******************************************************************************
* Desription  	: The function for H/W Codec Mute Control function
* Parameter   	: ucOnOff - 1(Mute On), 0(Mute Off)
* Return      	: None
******************************************************************************/
void tca_tcc250dac_setoutputmute(void *hI2C, unsigned char ucOnOff)
{
	unsigned int vMute = 0;
	unsigned int vPath = 0;

	if(ucOnOff)	// Mute
	{
		tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 		0x7f);
	}
	else	// Mute Off			
	{
		vPath = tca_tcc250dac_getcodecpath(hI2C) & 0x1F;
		if( vPath == 0x02 ||  vPath == 0x04 ||  vPath == 0x07 ||  vPath == 0x0A ||   vPath == 0x12  ||vPath == 0x14   
			||  vPath == 0x04 ||  vPath == 0x17 ||  vPath == 0x19 ||  vPath == 0x1B   

		)
		{
				vMute = tca_tcc250dac_getcodecmute(hI2C);				
				vMute |= 0x3;
				tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 		vMute);
		}
		else
			tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 		0x00);
	}
}

int tca_tcc250dac_multiset(void *hI2C, int type)
{
    switch (type) {
    case TYPE_ONLY_PLAYBACK:
        tca_tcc250dac_setplay(hI2C);
        break;
    case TYPE_MICIN_PLAYBACK:
        tca_tcc250dac_setinputoutputmode2(hI2C);
        break;
    case TYPE_LINEIN_PLAYBACK:
        tca_tcc250dac_setinputoutputmode(hI2C);
        break;
    default:
        return -1;
        break;
    }
    return 0;
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setlinein()
******************************************************************************
* Desription   	 	: The function for H/W Codec Line-IN Control function
* Parameter   		: None
* Return        	: None
******************************************************************************/
// LineIn Jack -> Line2
void tca_tcc250dac_setlinein(void *hI2C)
{
//	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x5f);	// ADC off 
	
#ifdef	ANA_LDO_INCLUDE
	tea_wave_i2c_write(hI2C, TCC250_AUTOSTART2_REG, 		0x07);	// 
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x4E);	// Path 14
	tca_tcc250dac_delay(5);
#else
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);	// 
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x0E);	// Path 14 :LineIn Bypass --> HP + LineIN --> Digital Out
	tca_tcc250dac_delay(5);
#endif
	tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,			0x00);	// Microphone Control : Bias Off, Det off
	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
//	tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0C);	// DAC off , HP on
	

#ifdef	TCC250_CODEC_I2S
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif
	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x00);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x64);
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setinputoutputmode()
******************************************************************************
* Desription   	 	: The function for H/W Codec Line-IN Control function
* Parameter   		: None
* Return        	: None
******************************************************************************/
// LineIn Jack -> Line2
void tca_tcc250dac_setinputoutputmode(void *hI2C)
{
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x5f);	// ADC off 
#ifdef	ANA_LDO_INCLUDE
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);	// 
    tca_tcc250dac_delay(5);
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x58);	// Path 24 :LineIN --> Digital Out, Digital in -> Headset Out
    tca_tcc250dac_delay(5);
#else
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);	// 
    tca_tcc250dac_delay(5);
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x18);	//
    tca_tcc250dac_delay(5);
#endif
    tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,			0x00);	// Microphone Control : Bias Off, Det off
    tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
    //		tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0F);	// DAC on , HP on


#ifdef	TCC250_CODEC_I2S
    tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
    tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif
    tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x00);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

    tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x04); //DAC Mute Off / MICLMUTE On
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setinputoutputmode2()
******************************************************************************
* Desription   	 	: The function for H/W Codec Mic-IN Control function
* Parameter   		: None
* Return        	: None
******************************************************************************/
// MicIn => Digital-out & Digital-In => HSOUT
void tca_tcc250dac_setinputoutputmode2(void *hI2C)
{
    //	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x5f);	// ADC off 
#ifdef	ANA_LDO_INCLUDE
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);	// 
    tca_tcc250dac_delay(5);
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x56);	// Path 24 :LineIN --> Digital Out, Digital in -> Headset Out
    tca_tcc250dac_delay(5);
#else
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);	// 
    tca_tcc250dac_delay(5);
    tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x16);	//
    tca_tcc250dac_delay(5);
#endif

	tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,			0x03);	// Microphone Control : Bias On, Det on, Level 170mV

	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
//	tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0C);	// DAC off , HP on

#ifdef	TCC250_CODEC_I2S
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif
	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x02);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

	tea_wave_i2c_write( hI2C, TCC250_MICVOL_REG, 			0x22);	// +24dB MAX +34DB

	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x10);
}


/*****************************************************************************
* Function Name : tca_tcc250dac_setauxin()
******************************************************************************
* Desription   	 	: The function for H/W Codec FM-IN Control function
* Parameter   	: None
* Return        	: None
******************************************************************************/
// FM output -> AuxIn
void tca_tcc250dac_setauxin(void *hI2C)
{
//	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0x1f);	// MIC Mute, Left/Right Input Channel On

#ifdef	ANA_LDO_INCLUDE
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);	// 
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x50);	// Path 16
	tca_tcc250dac_delay(5);
#else
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x10);	// Path 16
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);	// 
	tca_tcc250dac_delay(5);
#endif
	tea_wave_i2c_write(  hI2C, TCC250_MIC_CTRL_REG,			0x00);	// Microphone Control : Bias Off, Det off
	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
//	tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0C);	// DAC off , HP on

#ifdef	TCC250_CODEC_I2S
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif
	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x00);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x64);


}

/*****************************************************************************
* Function Name : tca_tcc250dac_setmicin()
******************************************************************************
* Desription   	 	: The function for H/W Codec Mic-IN Control function
* Parameter   	: None
* Return        	: None
******************************************************************************/
// MicIn -> Line1
void tca_tcc250dac_setmicin(void *hI2C)
{ 
//	TCC250_SendCmd(TCC250_PUP_OFF_INPUT_REG,	0xA4);	// ADC off 
	
#ifdef	ANA_LDO_INCLUDE
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);	//
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x4C);	// Path 12
	tca_tcc250dac_delay(5);
#else
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);	//
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x0C);	// Path 12
	tca_tcc250dac_delay(5);
#endif
	tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,			0x07);	// Microphone Control : Bias On, Det on, Level 170mV

	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
//	tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0C);	// DAC off , HP on

#ifdef	TCC250_CODEC_I2S
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
#else
	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
#endif
	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x02);	// MonoADC -  Input samples sent to both Left and Right Audio Digital Interface

	tea_wave_i2c_write( hI2C, TCC250_MICVOL_REG, 			0x20);	// +24dB MAX +34DB

	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x78);
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setplay()
******************************************************************************
* Desription   	 	: The function for H/W Codec Line-IN Control function
* Parameter   		: None
* Return        	: None
******************************************************************************/
void tca_tcc250dac_setplay(void *hI2C)
{
#ifdef	ANA_LDO_INCLUDE
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x07);
	tca_tcc250dac_delay(5);
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x41);	// Enable ENACLDO and Path#1
	tca_tcc250dac_delay(5);
#else
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART2_REG, 		0x87);
	tca_tcc250dac_delay(5);
	
	tea_wave_i2c_write( hI2C, TCC250_AUTOSTART1_REG, 		0x01);	// Disable ENACLDO and Path#1
	tca_tcc250dac_delay(5);
#endif
	tea_wave_i2c_write( hI2C, TCC250_MIC_CTRL_REG,		0x00);	// Microphone Control : Bias Off, Det off
//	tea_wave_i2c_write( hI2C, TCC250_PLBCK_REG, 			0x00);	//Enable High Pass Filter

//#ifdef	TCC250_CODEC_I2S
//	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x09);	// I2S
//#else
//	tea_wave_i2c_write( hI2C, TCC250_FRAME_CTRL_REG,		0x49);	// Left Justfied (MSB First)
//#endif

		
//	tea_wave_i2c_write( hI2C, TCC250_ISSCTRL_REG,			TCC250_ISSCTRL_VALUE);	// MCLK By External OSC
//	tea_wave_i2c_write( hI2C, TCC250_PUP_OFF_OUTPUT_REG,		0x0F);	// DAC on HP on

//	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x1C);
	tea_wave_i2c_write( hI2C, TCC250_LRMUTE_REG, 			0x00);
}
/*****************************************************************************
* Function Name : tca_tcc250dac_setinputmode()
******************************************************************************
* Desription   	 	: The function for H/W Codec Line-IN Control function
* Parameter   		: None
* Return        	: None
******************************************************************************/
// LineIn Jack -> Line2
void tca_tcc250dac_setinputmode(void *hI2C)
{
#ifdef _LINE_IN_RECORD_
	tca_tcc250dac_setlinein(hI2C);
#else
	tca_tcc250dac_setmicin(hI2C);
#endif
}


/*****************************************************************************
* Function Name : tca_tcc250dac_setoutputmode()
******************************************************************************/
void tca_tcc250dac_setoutputmode(void *hI2C, unsigned int nMode)
{

	if(nMode) // using FMR output
	{
		tca_tcc250dac_setauxin(hI2C);
	}
	else// not using FMR output
	{
		tca_tcc250dac_setplay(hI2C);
	}
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setinputgain()
******************************************************************************
* Desription   	: The function for H/W Codec Input Volume Control function
* Parameter   	: ucVolume (0 ~ 40)
* Return        : None
******************************************************************************/
void tca_tcc250dac_setinputgain(void *hI2C, unsigned char ucVolume)
{

const unsigned char TCC250_HP_IN_VOLUME[41]= {
												0x5D,	// Mute <= 0xB2
												0xD4,	// Vol 01 : -27dB
												0xD6,	// Vol 02 : -26dB
												0xD8,	// Vol 03 : -25dB
												0xDA,	// Vol 04 : -24dB
												0xDC,	// Vol 05 : -23dB
												0xDE,	// Vol 06 : -22dB
												0xE0,	// Vol 07 : -21dB
												0xE2,	// Vol 08 : -20dB
												0xE4,	// Vol 09 : -19dB
												0xE6,	// Vol 10 : -18dB
												0xE8,	// Vol 11 : -17dB
												0xEA,	// Vol 12 : -16dB
												0xEC,	// Vol 13 : -15dB
												0xEE,	// Vol 14 : -14dB
												0xF0,	// Vol 15 : -13dB
												0xF2,	// Vol 16 : -12dB
												0xF4,	// Vol 17 : -11dB
												0xF6,	// Vol 18 : -10dB
												0xF7,	// Vol 19 : -9dB
												0xF8,	// Vol 20 : -8dB
												0xF9,	// Vol 21 : -7dB
												0xFA,	// Vol 22 : -6dB
												0xFB,	// Vol 23 : -5dB
												0xFC,	// Vol 24 : -4dB
												0xFD,	// Vol 25 : -3dB
												0xFE,	// Vol 26 : -2dB
												0xFF,	// Vol 27 : -1dB
												0x00,	// Vol 28 : 0 dB
												0x01,	// Vol 29 : + 1dB
												0x02,	// Vol 30 : + 2dB
												0x03,	// Vol 31 : + 3dB
												0x04,	// Vol 32 : + 4dB
												0x05,	// Vol 33 : + 5dB
												0x06,	// Vol 34 : + 6dB
												0x07,	// Vol 35 : + 7dB
												0x08,	// Vol 36 : + 8dB
												0x09,	// Vol 37 : + 9dB
												0x0a,	// Vol 38 : + 10dB
												0x0b,	// Vol 39 : + 11dB
												0x0c,	// Vol 40 : + 12dB
											};

	tea_wave_i2c_write( hI2C, TCC250_LINVOL_REG, 			0x80+TCC250_HP_IN_VOLUME[ucVolume]); // +12db MAX +12DB
	tea_wave_i2c_write( hI2C, TCC250_RINVOL_REG, 			0x80+TCC250_HP_IN_VOLUME[ucVolume]);// +12db MAX +12DB
	
}

/*****************************************************************************
* Function Name : tca_tcc250dac_setoutputgain()
******************************************************************************
* Desription   	: The function for H/W Codec Output Volume Control function
* Parameter   	: ucVolume (0 ~ 40)
* Return        : None
******************************************************************************/
void tca_tcc250dac_setoutputgain(void *hI2C, unsigned int ucVolume)
{
const unsigned char TCC250_HP_OUT_VOLUME[41]= {
												0xB2,	// Mute <= 0xB2
												0xB8,	// Vol 01 : -72dB
												0xBB,	// Vol 02 : -69dB
												0xBE,	// Vol 03 : -66dB
												0xC1,	// Vol 04 : -63dB
												0xC4,	// Vol 05 : -60dB
												0xC6,	// Vol 06 : -58dB
												0xC8,	// Vol 07 : -56dB
												0xCA,	// Vol 08 : -54dB
												0xCC,	// Vol 09 : -52dB
												0xCE,	// Vol 10 : -50dB
												0xD0,	// Vol 11 : -48dB
												0xD2,	// Vol 12 : -46dB
												0xD4,	// Vol 13 : -44dB
												0xD6,	// Vol 14 : -42dB
												0xD8,	// Vol 15 : -40dB
												0xDA,	// Vol 16 : -38dB
												0xDC,	// Vol 17 : -36dB
												0xDE,	// Vol 18 : -34dB
												0xE0,	// Vol 19 : -32dB
												0xE2,	// Vol 20 : -30dB
												0xE4,	// Vol 21 : -28dB
												0xE6,	// Vol 22 : -26dB
												0xE8,	// Vol 23 : -24dB
												0xEA,	// Vol 24 : -22dB
												0xEC,	// Vol 25 : -20dB
												0xEE,	// Vol 26 : -18dB
												0xF0,	// Vol 27 : -16dB
												0xF2,	// Vol 28 : -14dB
												0xF4,	// Vol 29 : -12dB
												0xF6,	// Vol 30 : -10dB
												0xF7,	// Vol 31 : - 9dB
												0xF8,	// Vol 32 : - 8dB
												0xF9,	// Vol 33 : - 7dB
												0xFA,	// Vol 34 : - 6dB
												0xFB,	// Vol 35 : - 5dB
												0xFC,	// Vol 36 : - 4dB
												0xFD,	// Vol 37 : - 3dB
												0xFE,	// Vol 38 : - 2dB
												0xFF,	// Vol 39 : - 1dB
												0x00,	// Vol 40 : + 0dB
											};
#ifndef __KERNEL__
	ucVolume = ucVolume/1638;
#endif

	if(ucVolume > 40) ucVolume = 40;	// Value Limit is 40

	tea_wave_i2c_write( hI2C, TCC250_HSOUTL_REG, TCC250_HP_OUT_VOLUME[ucVolume]);
	tea_wave_i2c_write( hI2C, TCC250_HSOUTR_REG, TCC250_HP_OUT_VOLUME[ucVolume]);
	
}

#endif

