/****************************************************************************
 *   FileName    : tca_tcc250dac.h
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifndef _TCC250DAC_H_
#define _TCC250DAC_H_

//#define	ANA_LDO_INCLUDE
#define	TCC250_CODEC_I2S	// I2S Interface - Undef  Left Justfied (MSB First)
#define MCLK_TO_TCC250_INCLUDE
#define FRAME_TO_TCC250_INCLUDE
#define _TCC250_DVS_INCLUDE

/***********************
Audio Register Mapping
************************/
#define	TCC250_ANA_CTRL_REG			0x20		//LDOANA Regulator Management	Read / Write	0x0C
#define	TCC250_AUTOSTART1_REG		0x10	//Audio Automatic Start Up Sequence	Read / Write	0x00
#define	TCC250_AUTOSTART2_REG		0x11	//Audio Automatic Start Up Sequence	Read / Write	0x0F
#define	TCC250_PUP_OFF_INPUT_REG	0x12	//Audio Power Up-Down ? Inputs	Read / Write	0x50
#define	TCC250_PUP_OFF_OUTPUT_REG	0x13	//Audio Power Up-Down ? Outputs	Read / Write	0x00
#define	TCC250_MIC_CTRL_REG			0x14	//Microphone Control	Read / Write	0x00
#define	TCC250_ISSCTRL_REG			0x15	//Digital Audio Interface Clock Control	Read / Write	0x02
#define	TCC250_FRAME_CTRL_REG		0x16	//Digital Audio Interface Frame Control	Read / Write	0x49
#define	TCC250_LRMUTE_REG			0x17	//Signal Path Muting, Left/Right Channel	Read / Write	0x7F
#define	TCC250_MICVOL_REG			0x18	//Microphone Volume and Mute	Read / Write	0x00
#define	TCC250_LINVOL_REG			0x19	//Left Channel (LINE/AUX) Volume and Mute	Read / Write	0x00
#define	TCC250_RINVOL_REG			0x1A	//Right Channel (LINE) Volume and Mute	Read / Write	0x00
#define	TCC250_HSOUTL_REG			0x1B	//Headset Left Channel Volume and Mute	Read / Write	0x00
#define	TCC250_HSOUTR_REG			0x1C	//Headset Right Channel Volume and Mute	Read / Write	0x00
#define	TCC250_PLAYBACK_REG			0x1D	//Left Channel Sidetone and Bypass Control	Read / Write	0x00
#define	TCC250_PLBCK_REG			0x1E	//Playback Control Parameters	Read / Write	0x10
		
#define	TCC250_ISSCTRL_FRAMEGEN	0x04
#define	TCC250_ISSCTRL_112896	0x02
#define	TCC250_ISSCTRL_MCLKGEN	0x01

#define TYPE_ONLY_PLAYBACK 0
#define TYPE_MICIN_PLAYBACK 1
#define TYPE_LINEIN_PLAYBACK 2


#if defined(MCLK_TO_TCC250_INCLUDE) && defined(FRAME_TO_TCC250_INCLUDE)// MCLK,LR,BCLK : TCC8xx --> TCC250
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896) //Full Slave Mode
#endif

#if defined(MCLK_TO_TCC250_INCLUDE) && !defined(FRAME_TO_TCC250_INCLUDE)// MCLK : TCC8xx --> TCC250, LR,BCLK : TCC250 --> TCC8xx
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_FRAMEGEN) // Slave/Master Mode
#endif

#if !defined(MCLK_TO_TCC250_INCLUDE) && defined(FRAME_TO_TCC250_INCLUDE)// MCLK : TCC250 --> TCC8xx, LR,BCLK : TCC8xx --> TCC250
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_MCLKGEN) //Master/Slave Mode
#endif

#if !defined(MCLK_TO_TCC250_INCLUDE) && !defined(FRAME_TO_TCC250_INCLUDE) // MCLK,LR,BCLK : TCC250 --> TCC8xx
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_FRAMEGEN|TCC250_ISSCTRL_MCLKGEN) //Full Master Mode
#endif

#ifdef __KERNEL__
#define _TCC250_INCLUDE_
#endif

#ifdef _TCC250_INCLUDE_
#ifdef __cplusplus
extern "C" {
#endif

void tca_tcc250dac_initcodec(void *hI2C);
void tca_tcc250dac_setinputmute(void *hI2C, unsigned char ucOnOff);
void tca_tcc250dac_setoutputmute(void *hI2C, unsigned char ucOnOff);

void tca_tcc250dac_setlinein(void *hI2C);
void tca_tcc250dac_setmicin(void *hI2C);
void tca_tcc250dac_setplay(void *hI2C);

void tca_tcc250dac_setinputoutputmode(void *hI2C);
void tca_tcc250dac_setinputmode(void *hI2C);
void tca_tcc250dac_setoutputmode(void *hI2C, unsigned int nMode);

void tca_tcc250dac_setinputgain(void *hI2C, unsigned char ucVolume);
void tca_tcc250dac_setoutputgain(void *hI2C, unsigned int ucVolume);
void tca_tcc250dac_setana(void *hI2C, unsigned short mV);

int tca_tcc250dac_multiset(void *hI2C, int type);
void tca_tcc250dac_setinputoutputmode2(void *hI2C);

extern int tea_wave_i2c_write(void *hI2C, unsigned char hb, unsigned char lb); 
extern char tea_wave_i2c_read(void *hI2C, unsigned char hb);

#ifdef __cplusplus
}
#endif

#endif

#endif

