/*
 * linux/sound/soc/codecs/tcc250.c
 *
 * Based on tcc250.c by Liam Girdwood
 * Author: Richard Purdie <richard@openedhand.com>
 * Rewritten by: <linux@telechips.com>
 * Modified: June 10, 2008
 * Description: TCC250 Soc Audio driver
 *
 * Copyright 2005 Openedhand Ltd.
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <sound/driver.h>
#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/soc-dapm.h>
#include <sound/initval.h>


#if defined(CONFIG_ARCH_TCC79X)
#include <asm/arch/tcc79x.h>
#else
#   include <asm/arch/tcc83x_virt.h>
#endif

#include "tcc250.h"
//#include "tcc250_dac.h"
#include "tca_tcc250dac.h"
#include "../tcc/tcc-pcm.h"
#include "../tcc/tca_tcchwcontrol.h"

extern void alsa_CODEC_SEND_CMD(unsigned char hb, unsigned char lb);

int tea_wave_i2c_write(void *hI2C, unsigned char hb, unsigned char lb)
{
    unsigned char data[2];
    struct snd_soc_codec *codec = (struct snd_soc_codec *)hI2C;

    data[0] = hb;
    data[1] = lb;
    if (codec->hw_write(codec->control_data, data, 2) != 2) {
        printk("[%s:%d] I2C write error !!! \n", __func__, __LINE__);
    }
    return 0;
}

char tea_wave_i2c_read(void *hI2C, unsigned char hb)
{
    unsigned char data[2];
    struct snd_soc_codec *codec = (struct snd_soc_codec *)hI2C;

    data[0] = hb;
    data[1] = 0;
    if (codec->hw_write(codec->control_data, data, 1) != 1) {
        printk("[%s:%d] I2C write error !!! \n", __func__, __LINE__);
    }
    codec->hw_read(codec->control_data, &(data[1]), 1);

    return (char)(data[1]);
}


#define DEF_VOL_LEVEL 20
struct tcc250_data {
    int vol_value;
    int is_mute;
    int input_state;
};

static struct tcc250_data s_tcc250_info = { DEF_VOL_LEVEL, 0 };

#define AUDIO_NAME "tcc250"
#define CODEC_VERSION "0.1"

struct snd_soc_codec_device soc_codec_dev_tcc250;
extern int	alsa_DDI_I2C_Initialize(void);
extern void alsa_i2c_init(void);

/* codec private data */
struct tcc250_priv {
    unsigned int sysclk;
};


/*
 * read tcc250 register cache
 */
static inline unsigned int tcc250_read_reg_cache(struct snd_soc_codec *codec,
                                                 unsigned int reg)
{
    return 0;
}

/*
 * write tcc250 register cache
 */
static inline void tcc250_write_reg_cache(struct snd_soc_codec *codec,
                                          u16 reg, unsigned int value)
{
}

/*
 * write to the TCC250 register space
 */
static int tcc250_write(struct snd_soc_codec *codec, unsigned int reg,
                        unsigned int value)
{
    return 0;
}

/* mute */
static int tcc250_mute_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
    uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
    return 0;
}

static int tcc250_mute_get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	ucontrol->value.integer.value[0] = s_tcc250_info.is_mute;
    return 0;
}

static int tcc250_mute_put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);
	s_tcc250_info.is_mute = ucontrol->value.integer.value[0];
    if (s_tcc250_info.is_mute) {
        tca_tcc250dac_setoutputgain(codec, 0);
    } else {
        tca_tcc250dac_setoutputgain(codec, s_tcc250_info.vol_value);
    }
    return 0;
}

/* volume */
static int tcc250_volume_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
    uinfo->count = 2;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 40;
	uinfo->value.integer.step = 1;
    return 0;
}

static int tcc250_volume_get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	ucontrol->value.integer.value[0] = s_tcc250_info.vol_value;
	ucontrol->value.integer.value[1] = s_tcc250_info.vol_value;

    return 0;
}

static int tcc250_volume_put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);

    s_tcc250_info.vol_value = ucontrol->value.integer.value[0];
    if (s_tcc250_info.vol_value > 40) { s_tcc250_info.vol_value = 40; }
    tca_tcc250dac_setoutputgain(codec, s_tcc250_info.vol_value);
    s_tcc250_info.is_mute = 0;
    return 0;
}

/* multi-set */
static int tcc250_select_info(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo)
{
	struct soc_enum *e = (struct soc_enum *)kcontrol->private_value;

	uinfo->type = SNDRV_CTL_ELEM_TYPE_ENUMERATED;
    uinfo->count = 1;
	uinfo->value.enumerated.items = 3;

    if (uinfo->value.enumerated.item > 2)
        uinfo->value.enumerated.item = 2;

	strcpy(uinfo->value.enumerated.name, e->texts[uinfo->value.enumerated.item]);
    return 0;
}

static int tcc250_select_get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	//struct soc_enum *e = (struct soc_enum *)kcontrol->private_value;

    ucontrol->value.enumerated.item[0] = s_tcc250_info.input_state;

    return 0;
}

static int tcc250_select_put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *ucontrol)
{
	struct snd_soc_codec *codec = snd_kcontrol_chip(kcontrol);


    if (s_tcc250_info.input_state != ucontrol->value.enumerated.item[0]) {
        s_tcc250_info.input_state = ucontrol->value.enumerated.item[0];
        tca_tcc250dac_multiset(codec, s_tcc250_info.input_state);
        tca_tcc250dac_setoutputgain(codec, s_tcc250_info.vol_value);
        if (s_tcc250_info.is_mute) {
            tca_tcc250dac_setoutputgain(codec, 0);
        } else {
            tca_tcc250dac_setoutputgain(codec, s_tcc250_info.vol_value);
        }
    }
    return s_tcc250_info.input_state;
}


static const char *tcc250_mode_select[] = {"Play", "Play_Mic", "Play_Line"};
static const struct soc_enum tcc250_enum[] = {
    SOC_ENUM_SINGLE(0, 0, 3, tcc250_mode_select),
};

static const struct snd_kcontrol_new tcc250_snd_controls[] = {
    {
        .iface = SNDRV_CTL_ELEM_IFACE_MIXER,
        .name = "MasterVolume",
        .info = tcc250_volume_info,
        .get  = tcc250_volume_get,
        .put  = tcc250_volume_put,
        .private_value = 0,
    },
    {
        .iface = SNDRV_CTL_ELEM_IFACE_MIXER,
        .name = "Mute",
        .info = tcc250_mute_info,
        .get  = tcc250_mute_get,
        .put  = tcc250_mute_put,
        .private_value = 0,
    },
    {
        .iface = SNDRV_CTL_ELEM_IFACE_MIXER,
        .name = "ModeSelect",
        .info = tcc250_select_info,
        .get  = tcc250_select_get,
        .put  = tcc250_select_put,
        .private_value = (unsigned long)(&(tcc250_enum[0])),
    },
};

#define SOC_ENUM(xname, xenum) \
{	.iface = SNDRV_CTL_ELEM_IFACE_MIXER, .name = xname,\
	.info = snd_soc_info_enum_double, \
	.get = snd_soc_get_enum_double, .put = snd_soc_put_enum_double, \
	.private_value = (unsigned long)&xenum }

/* add non dapm controls */
static int tcc250_add_controls(struct snd_soc_codec *codec)
{
    int err, i;

    for (i = 0; i < ARRAY_SIZE(tcc250_snd_controls); i++) {
        if ((err = snd_ctl_add(codec->card,
                               snd_soc_cnew(&tcc250_snd_controls[i],codec, NULL))) < 0)
            return err;
    }

    return 0;
}


static const char *intercon[][3] = {
    /* terminator */
    {NULL, NULL, NULL},
};

static int tcc250_add_widgets(struct snd_soc_codec *codec)
{
    int i;

#if 0
    for(i = 0; i < ARRAY_SIZE(tcc250_dapm_widgets); i++) {
        snd_soc_dapm_new_control(codec, &tcc250_dapm_widgets[i]);
    }
#endif

    /* set up audio path interconnects */
    for(i = 0; intercon[i][0] != NULL; i++) {
        snd_soc_dapm_connect_input(codec, intercon[i][0],
                                   intercon[i][1], intercon[i][2]);
    }

    snd_soc_dapm_new_widgets(codec);
    return 0;
}

struct _coeff_div {
    u32 mclk;
    u32 rate;
    u16 fs;
    u8 sr:4;
    u8 bosr:1;
    u8 usb:1;
};

static int tcc250_hw_params(struct snd_pcm_substream *substream,
                            struct snd_pcm_hw_params *params)
{
    return 0;
}

static int tcc250_pcm_prepare(struct snd_pcm_substream *substream)
{
    //struct snd_soc_pcm_runtime *rtd = substream->private_data;
    //struct snd_soc_device *socdev = rtd->socdev;
    //struct snd_soc_codec *codec = socdev->codec;

    //tca_tcc250dac_setplay(codec);
    return 0;
}

static void tcc250_shutdown(struct snd_pcm_substream *substream)
{
}

static int tcc250_mute(struct snd_soc_codec_dai *dai, int mute)
{
    struct snd_soc_codec *codec = dai->codec;
    if (mute) {
        s_tcc250_info.is_mute = 1;
        tca_tcc250dac_setoutputgain(codec, 0);
    }
    return 0;        
}

static int tcc250_set_dai_sysclk(struct snd_soc_codec_dai *codec_dai,
                                 int clk_id, unsigned int freq, int dir)
{
    struct snd_soc_codec *codec = codec_dai->codec;
    struct tcc250_priv *tcc250 = codec->private_data;

    switch (freq) {
    case 11289600:
    case 12000000:
    case 12288000:
    case 16934400:
    case 18432000:
        tcc250->sysclk = freq;
        return 0;
    }
    return -EINVAL;
}


static int tcc250_set_dai_fmt(struct snd_soc_codec_dai *codec_dai,
                              unsigned int fmt)
{
    //struct snd_soc_codec *codec = codec_dai->codec;
    u16 iface = 0;

    /* set master/slave audio interface */
    switch (fmt & SND_SOC_DAIFMT_MASTER_MASK) {
    case SND_SOC_DAIFMT_CBM_CFM:
        iface |= 0x0040;
        break;
    case SND_SOC_DAIFMT_CBS_CFS:
        break;
    default:
        return -EINVAL;
    }

    /* interface format */
    switch (fmt & SND_SOC_DAIFMT_FORMAT_MASK) {
    case SND_SOC_DAIFMT_I2S:
        iface |= 0x0002;
        break;
    case SND_SOC_DAIFMT_RIGHT_J:
        break;
    case SND_SOC_DAIFMT_LEFT_J:
        iface |= 0x0001;
        break;
    case SND_SOC_DAIFMT_DSP_A:
        iface |= 0x0003;
        break;
    case SND_SOC_DAIFMT_DSP_B:
        iface |= 0x0013;
        break;
    default:
        return -EINVAL;
    }

    /* clock inversion */
    switch (fmt & SND_SOC_DAIFMT_INV_MASK) {
    case SND_SOC_DAIFMT_NB_NF:
        break;
    case SND_SOC_DAIFMT_IB_IF:
        iface |= 0x0090;
        break;
    case SND_SOC_DAIFMT_IB_NF:
        iface |= 0x0080;
        break;
    case SND_SOC_DAIFMT_NB_IF:
        iface |= 0x0010;
        break;
    default:
        return -EINVAL;
    }

    /* set iface */
    //tcc250_write(codec, TCC250_IFACE, iface);
    return 0;
}

static int tcc250_dapm_event(struct snd_soc_codec *codec, int event)
{
    switch (event) {
    case SNDRV_CTL_POWER_D0: /* full On */
        /* vref/mid, osc on, dac unmute */
        //printk("SNDRV_CTL_POWER_D0 [%s%d]\n", __func__, __LINE__);
        break;
    case SNDRV_CTL_POWER_D1: /* partial On */
    case SNDRV_CTL_POWER_D2: /* partial On */
        //printk("SNDRV_CTL_POWER_D1 or SNDRV_CTL_POWER_D2[%s%d]\n", __func__, __LINE__);
        break;
    case SNDRV_CTL_POWER_D3hot: /* Off, with power */
        /* everything off except vref/vmid, */
        //printk("SNDRV_CTL_POWER_D3hot [%s%d]\n", __func__, __LINE__);
        break;
    case SNDRV_CTL_POWER_D3cold: /* Off, without power */
        /* everything off, dac mute, inactive */
        //printk("SNDRV_CTL_POWER_D3cold [%s%d]\n", __func__, __LINE__);
        break;
    }
    codec->dapm_state = event;
    return 0;
}

#define TCC250_RATES (SNDRV_PCM_RATE_8000 | SNDRV_PCM_RATE_11025 |\
                      SNDRV_PCM_RATE_16000 | SNDRV_PCM_RATE_22050 |\
                      SNDRV_PCM_RATE_32000 | SNDRV_PCM_RATE_44100 |\
                      SNDRV_PCM_RATE_48000 | SNDRV_PCM_RATE_88200 |\
                      SNDRV_PCM_RATE_96000)

#define TCC250_FORMATS (SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S20_3LE |\
                        SNDRV_PCM_FMTBIT_S24_LE)

struct snd_soc_codec_dai tcc250_dai = {
    .name = "TCC250",
    .playback = {
        .stream_name = "Playback",
        .channels_min = 1,
        .channels_max = 2,
        .rates = TCC250_RATES,
        .formats = TCC250_FORMATS,},
    .capture = {
        .stream_name = "Capture",
        .channels_min = 1,
        .channels_max = 2,
        .rates = TCC250_RATES,
        .formats = TCC250_FORMATS,},
    .ops = {
        .prepare = tcc250_pcm_prepare,
        .hw_params = tcc250_hw_params,
        .shutdown = tcc250_shutdown,
    },
    .dai_ops = {
        .digital_mute = tcc250_mute,
        .set_sysclk = tcc250_set_dai_sysclk,
        .set_fmt = tcc250_set_dai_fmt,
    }
};
EXPORT_SYMBOL_GPL(tcc250_dai);

static int tcc250_suspend(struct platform_device *pdev, pm_message_t state)
{
    return 0;
}

static int tcc250_resume(struct platform_device *pdev)
{
    return 0;
}

/*
 * initialise the TCC250 driver
 * register the mixer and dsp interfaces with the kernel
 */
static int tcc250_init(struct snd_soc_device *socdev)
{
    struct snd_soc_codec *codec = socdev->codec;
    int ret = 0;

    codec->name = "TCC250";
    codec->owner = THIS_MODULE;
    codec->read = tcc250_read_reg_cache;
    codec->write = tcc250_write;
    codec->dapm_event = tcc250_dapm_event;
    codec->dai = &tcc250_dai;
    codec->num_dai = 1;
    codec->reg_cache_size = 0;
    codec->reg_cache = NULL;

    tca_tcc250dac_initcodec(codec);
    tca_tcc250dac_setana(codec, 2800);
    tca_tcc250dac_setoutputgain(codec, s_tcc250_info.vol_value);
    tca_tcc250dac_multiset(codec, TYPE_ONLY_PLAYBACK);

    /* register pcms */
    ret = snd_soc_new_pcms(socdev, SNDRV_DEFAULT_IDX1, SNDRV_DEFAULT_STR1);
    if (ret < 0) {
        printk(KERN_ERR "tcc250: failed to create pcms\n");
        goto pcm_err;
    }

    tcc250_dapm_event(codec, SNDRV_CTL_POWER_D3hot);
    tca_i2s_mutecontrol(0);

    tcc250_add_controls(codec);
    tcc250_add_widgets(codec);
    ret = snd_soc_register_card(socdev);
    if (ret < 0) {
        printk(KERN_ERR "tcc250: failed to register card\n");
        goto card_err;
    }

    return ret;

card_err:
    snd_soc_free_pcms(socdev);
    snd_soc_dapm_free(socdev);
pcm_err:
    if (codec->reg_cache) {
        kfree(codec->reg_cache);
        codec->reg_cache = NULL;
    }
    return ret;
}

static struct snd_soc_device *tcc250_socdev;

#if defined (CONFIG_I2C) || defined (CONFIG_I2C_MODULE)
/*
 * TCC250 2 wire address is determined by GPIO5
 * state during powerup.
 *    low  = 0x1a
 *    high = 0x1b
 */
extern struct i2c_client *tcc250_i2c_client;

static int tcc250_codec_i2c_probe(void)
{
    struct snd_soc_device *socdev = tcc250_socdev;
    struct snd_soc_codec *codec = socdev->codec;
	struct tcc250_setup_data *setup;
    int ret;

    i2c_set_clientdata(tcc250_i2c_client, codec);
    codec->control_data = tcc250_i2c_client;

    ret = tcc250_init(socdev);
    if (ret < 0) {
        printk("failed to initialise TCC250\n");
        goto err;
    }

	/* i2c_address of tcc_board_tcc250.c setting */
	setup = socdev->codec_data;
	setup->i2c_address = codec->control_data->addr;
	
    return ret;

err:
    kfree(codec);
    return ret;
}

#endif

static int tcc250_probe(struct platform_device *pdev)
{
    struct snd_soc_device *socdev = platform_get_drvdata(pdev);
    struct tcc250_setup_data *setup;
    struct snd_soc_codec *codec;
    struct tcc250_priv *tcc250;
    int ret = 0;

    printk("TCC250 Audio Codec %s  [%s]\n", CODEC_VERSION, __FUNCTION__);

    setup = socdev->codec_data;
    codec = kzalloc(sizeof(struct snd_soc_codec), GFP_KERNEL);
    if (codec == NULL)
        return -ENOMEM;

    tcc250 = kzalloc(sizeof(struct tcc250_priv), GFP_KERNEL);
    if (tcc250 == NULL) {
        kfree(codec);
        return -ENOMEM;
    }

    codec->private_data = tcc250;
    socdev->codec = codec;
    mutex_init(&codec->mutex);
    INIT_LIST_HEAD(&codec->dapm_widgets);
    INIT_LIST_HEAD(&codec->dapm_paths);

    tcc250_socdev = socdev;
#if defined (CONFIG_I2C) || defined (CONFIG_I2C_MODULE)
#if 0
    if (setup->i2c_address) {
        codec->hw_write = (hw_write_t)i2c_master_send;
        codec->hw_read = (hw_read_t)i2c_master_recv;
        ret = tcc250_codec_i2c_probe();
        if (ret != 0)
            printk(KERN_ERR "can't add i2c driver");
    }
#else
	codec->hw_write = (hw_write_t)i2c_master_send;
	codec->hw_read = (hw_read_t)i2c_master_recv;
	ret = tcc250_codec_i2c_probe();
	if (ret != 0)
		printk(KERN_ERR "can't add i2c driver");
#endif
#else
    /* Add other interfaces here */
    codec->hw_write =  (hw_write_t)alsa_CODEC_SEND_CMD;

    alsa_DDI_I2C_Initialize();

    //HwGPEDAT  &= ~Hw20;
    //	HwGPBDAT    &= ~Hw21;   // Mute Ctrl OFF  //should move? should change to GPIO_A26
    alsa_i2c_init();
    //need tcc250 initialize 
    ret = tcc250_init(socdev);
    if (ret < 0) {
        printk(KERN_ERR"failed to initialise TCC250\n");	
    }
#endif

    return ret;
}

/* power down chip */
static int tcc250_remove(struct platform_device *pdev)
{
    struct snd_soc_device *socdev = platform_get_drvdata(pdev);
    struct snd_soc_codec *codec = socdev->codec;

    if (codec->control_data)
        tcc250_dapm_event(codec, SNDRV_CTL_POWER_D3cold);

    snd_soc_free_pcms(socdev);
    snd_soc_dapm_free(socdev);
#if defined (CONFIG_I2C) || defined (CONFIG_I2C_MODULE)
#endif
    kfree(codec->private_data);
    kfree(codec);

    return 0;
}

struct snd_soc_codec_device soc_codec_dev_tcc250 = {
    .probe = 	tcc250_probe,
    .remove = 	tcc250_remove,
    .suspend = 	tcc250_suspend,
    .resume =	tcc250_resume,
};

EXPORT_SYMBOL_GPL(soc_codec_dev_tcc250);

MODULE_DESCRIPTION("ASoC TCC250 driver");
MODULE_AUTHOR("Richard Purdie");
MODULE_LICENSE("GPL");

