
/****************************************************************************
 *   FileName    : tca_tcchwcontrol.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* includes
*
******************************************************************************/
#ifdef __KERNEL__
#   if defined(_TCC79x_)
#       include <mach/tcc79x.h>
#   elif defined(_TCC83x_)
#       include <mach/tcc83x_virt.h>
#   endif
#   include <asm/plat-tcc/tca_errorcode.h>
#   include <linux/module.h>
#   include <linux/moduleparam.h>
#else
#   if defined(_TCC79x_)
#       include "TCC79x_Virtual.h"
#   elif defined(_TCC83x_)
#       include "TCC83x_Virtual.h"
#   endif
#   include "tca_errorcode.h"
#endif

#include "tca_tcchwcontrol.h"
#ifdef _TCC250_INCLUDE_
#include "tca_tcc250dac.h"
#endif
/*****************************************************************************
* Function Name : tca_dma_clrstatus()
******************************************************************************
* Desription    :
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_dma_clrstatus(unsigned int nDmanum)
{
#ifdef _USING_ADMA_
	if (nDmanum) {
        HwIntStatus |= (Hw6 | Hw2);
    } else {
        HwIntStatus |= (Hw4 | Hw0);
    }
#else
	switch(	nDmanum )
	{
		case 0:
			HwCHCTRL0 |= Hw3;
		break;
		case 1:
			HwCHCTRL1 |= Hw3;
		break;
		case 2:
			HwCHCTRL2 |= Hw3;
		break;
	#ifdef _TCC79x_
		case 3:
			HwCHCTRL3 |= Hw3;
		break;
		case 4:
			HwCHCTRL4 |= Hw3;
		break;
		case 5:
			HwCHCTRL5 |= Hw3;
		break;
		case 6:
			HwCHCTRL6 |= Hw3;
		break;
		case 7:
			HwCHCTRL7 |= Hw3;
		break;
		case 8:
			HwCHCTRL8 |= Hw3;
		break;

	#endif
	}
#endif

	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_irq_getnumber()
******************************************************************************
* Desription    :
* Parameter     : 
* Return        : DMA number 
******************************************************************************/
unsigned int tca_irq_getnumber(void)
{

#ifdef _USING_ADMA_
	return IRQ_ADMA;  // audio output interrupt.
#else
    return IRQ_DMA;  // audio output interrupt.
#endif

}

/*****************************************************************************
* Function Name : tca_dma_getstatus()
******************************************************************************
* Desription    : dma status check
* Parameter     : 
* Return        :  
******************************************************************************/
unsigned int tca_dma_getstatus(unsigned int nDmanum)
{
#ifdef _USING_ADMA_
    return nDmanum ? (HwIntStatus & (Hw6 | Hw2)) : (HwIntStatus & (Hw4 | Hw0));
#else
	switch(	nDmanum )
	{
		case 0:
			return (HwCHCTRL0 & Hw3);
		break;
		case 1:
			return (HwCHCTRL1 & Hw3);
		break;
		case 2:
			return (HwCHCTRL2 & Hw3);
		break;
	#ifdef _TCC79x_
		case 3:
			return (HwCHCTRL3 & Hw3);
		break;
		case 4:
			return (HwCHCTRL4 & Hw3);
		break;
		case 5:
			return (HwCHCTRL5 & Hw3);
		break;
		case 6:
			return (HwCHCTRL6 & Hw3);
		break;
		case 7:
			return (HwCHCTRL7 & Hw3);
		break;
		case 8:
			return (HwCHCTRL8 & Hw3);
		break;

	#endif
	}
#endif
	return 0;

}

/*****************************************************************************
* Function Name : tca_i2s_mutecontrol()
******************************************************************************
* Desription    : i2s Mute on/off control
* Parameter     : 
* Return        : 
******************************************************************************/
void tca_i2s_mutecontrol(unsigned int nMode)
{
	if(nMode) //mute on
	{
	#if defined(_TCC79x_)
        #if defined(CONFIG_MACH_TCC7901S)
			BITSET(HwGPFDAT , Hw14); // Mute Ctrl ON GPIO_F14
        #else
			BITSET(HwGPFDAT , Hw7);	// Mute Ctrl ON GPIO_F7
        #endif
	#elif defined(_TCC83x_)
			BITSET(HwGDATA_E , Hw7);// Mute Ctrl ON GPIO_E7
	#endif
	}
	else //mute off
	{
	#if defined(_TCC79x_)
        #if defined(CONFIG_MACH_TCC7901S)
			BITCLR(HwGPFDAT , Hw14); // Mute Ctrl OFF GPIO_F14
        #else
			BITCLR(HwGPFDAT , Hw7);	// Mute Ctrl OFF GPIO_F7
        #endif
	#elif defined(_TCC83x_)
			BITCLR(HwGDATA_E , Hw7);// Mute Ctrl OFF GPIO_E7
	#endif
	}
//	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_tcc_initport()
******************************************************************************
* Desription    : GPIO port init
* Parameter     : 
* Return        :  
******************************************************************************/
unsigned int tca_tcc_initport(void)
{
#if defined(_TCC79x_)
	//GPIO_F7 for Mute
	//Mute GPIO Setting  //default DNP
#ifdef CONFIG_MACH_TCC7901S
	BITCSET(HwPORTCFG2, Hw12 - Hw8, Hw8);
	BITSET(HwGPFEN, Hw14);
	BITSET(HwGPFSET, Hw14);
    BITCLR(HwGPFDAT, Hw14);   // Mute Ctrl OFF GPIO_F7

    /* MIC_SP_SEL for Mic */
	BITCSET(HwPORTCFG2, Hw8 - Hw4, Hw4);
	BITSET(HwGPEEN, Hw26);
	BITSET(HwGPESET, Hw26);
    BITCLR(HwGPEDAT, Hw26);
#else
	BITCSET(HwPORTCFG2, Hw4-Hw0, Hw0);
	BITSET(HwGPFEN, Hw7);
	BITSET(HwGPFSET, Hw7);
    BITCLR(HwGPFDAT, Hw7);   // Mute Ctrl OFF GPIO_F7
#endif

#elif defined(_TCC83x_)
	//GPIO_E7 for Mute
	//Mute GPIO Setting  
	BITCSET(HwGSEL_E,  Hw3-Hw0, Hw1|Hw0); // GPIO_E6, GPIO_E5 for I2C, GPIO_E4 - GPIO_E0 for I2S
	BITSET(HwNGSEL_E,  Hw7); //GPIO_E7 for GPIO
	BITSET(HwGIOCON_E, Hw7); // direction for Output
    BITCLR(HwGDATA_E,   Hw7);   // Mute Ctrl OFF GPIO_E7
#endif
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_dma_setsrcaddr()
******************************************************************************
* Desription    : dma set source address
* Parameter     : dma number
* Return        : address
******************************************************************************/
unsigned int tca_dma_setsrcaddr(unsigned int nDmanum, unsigned int nAddr)	
{
#ifdef _USING_ADMA_
    HwTxDaSar = nAddr;
    return SUCCESS;
#else
		// Set the start address of DMA source buffer
	switch(	nDmanum )
	{
		case 0:
			HwST_SADR0 = (int)nAddr;
			break;
		case 1:
			HwST_SADR1 = (int)nAddr;
			break;
		case 2:
			HwST_SADR2 = (int)nAddr;
			break;
#ifdef _TCC79x_
	        case 3:
			HwST_SADR3 = (int)nAddr;
			break;
		case 4:
			HwST_SADR4 = (int)nAddr;
			break;
		case 5:
			HwST_SADR5 = (int)nAddr;
			break;
		case 6:
			HwST_SADR6 = (int)nAddr;
			break;
		case 7:
			HwST_SADR7 = (int)nAddr;
			break;
		case 8:
			HwST_SADR8 = (int)nAddr;
			break;

#endif
	}
#endif
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_dma_setdestaddr()
******************************************************************************
* Desription    : set dest address
* Parameter     : dma number, dest address
* Return        : SUCCESS 
******************************************************************************/
unsigned int tca_dma_setdestaddr(unsigned int nDmanum, unsigned int nAddr)	
{
#ifdef _USING_ADMA_
    return SUCCESS;
#endif
		// Set the start address of DMA source buffer
	switch(	nDmanum )
	{
		case 0:
			HwST_DADR0 = (int)(nAddr);
			break;
		case 1:
			HwST_DADR1 = (int)(nAddr);
			break;
		case 2:
			HwST_DADR2 = (int)(nAddr);
			break;
#ifdef _TCC79x_
	        case 3:
			HwST_DADR3 = (int)(nAddr);
			break;
		case 4:
			HwST_DADR4 = (int)(nAddr);
			break;
		case 5:
			HwST_DADR5 = (int)(nAddr);
			break;
		case 6:
			HwST_DADR6 = (int)(nAddr);
			break;
		case 7:
			HwST_DADR7 = (int)(nAddr);
			break;
		case 8:
			HwST_DADR8 = (int)(nAddr);
			break;
#endif
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_dma_control()
******************************************************************************
* Desription    : dma input control
* Parameter     : mode 1: on, 0:off
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_dma_control(unsigned int nMode, unsigned int nDmanum, unsigned int nInMode)				
{  
#ifdef _USING_ADMA_
    if(nMode) { //run
        tca_i2s_start(nInMode);
        HwRptCtrl=0;
        BITSET(HwChCtrl, HwChCtrl_DTIEN | HwChCtrl_DTEN_EN);
    } else {
        tca_i2s_stop(nInMode);
#if 0
        BITCLR(HwChCtrl, HwChCtrl_DTEN_EN);
        {
            int temp = HwTxDaCsar >>16;
            
            HwRptCtrl |= 0x80000000|((HwTxDaCsar&0xffff) - temp);
       
        }
        BITSET(HwChCtrl, HwChCtrl_DTEN_EN);
#else
        BITCLR(HwChCtrl, HwChCtrl_DTEN_EN);
#endif
    }
    return SUCCESS;
#else
	if(nMode) //run
	{
		tca_i2s_start(nInMode);
		switch(	nDmanum )
		{
			case 0:
				HwCHCTRL0 |= HwCHCTRL_EN_EN;
			break;
			case 1:
				HwCHCTRL1 |= HwCHCTRL_EN_EN;
			break;
			case 2:
				HwCHCTRL2 |= HwCHCTRL_EN_EN;
			break;
		#ifdef _TCC79x_
			case 3:
				HwCHCTRL3 |= HwCHCTRL_EN_EN;
			break;
			case 4:
				HwCHCTRL4 |= HwCHCTRL_EN_EN;
			break;
			case 5:
				HwCHCTRL5 |= HwCHCTRL_EN_EN;
			break;
			case 6:
				HwCHCTRL6 |= HwCHCTRL_EN_EN;
			break;
			case 7:
				HwCHCTRL7 |= HwCHCTRL_EN_EN;
			break;
			case 8:
				HwCHCTRL8 |= HwCHCTRL_EN_EN;
			break;

		#endif
		}
	}
	else //stop
	{
		tca_i2s_stop(nInMode);
		switch(	nDmanum )
		{
			case 0:
				HwCHCTRL0 &= ~HwCHCTRL_EN_EN;
			break;
			case 1:
				HwCHCTRL1 &= ~HwCHCTRL_EN_EN;
			break;
			case 2:
				HwCHCTRL2 &= ~HwCHCTRL_EN_EN;
			break;
		#ifdef _TCC79x_
			case 3:
				HwCHCTRL3 &= ~HwCHCTRL_EN_EN;
			break;
			case 4:
				HwCHCTRL4 &= ~HwCHCTRL_EN_EN;
			break;
			case 5:
				HwCHCTRL5 &= ~HwCHCTRL_EN_EN;
			break;
			case 6:
				HwCHCTRL6 &= ~HwCHCTRL_EN_EN;
			break;
			case 7:
				HwCHCTRL7 &= ~HwCHCTRL_EN_EN;
			break;
			case 8:
				HwCHCTRL8 &= ~HwCHCTRL_EN_EN;
			break;

		#endif
		}
	}
#endif
	return 0;
}


/*****************************************************************************
* Function Name : tca_dma_clrinterrupt()
******************************************************************************
* Desription    : dma clear interrupt status
* Parameter     : dma number
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_dma_clrinterrupt(unsigned nDmanum)
{
#ifdef _USING_ADMA_
    //HwChCtrl |= HwChCtrl_DTEN_EN;
#else
	switch(	nDmanum )
	{
	case 0:
		HwCHCTRL0 |= Hw3; 
		break;
	case 1:
		HwCHCTRL1 |= Hw3;
		break;
	case 2:
		HwCHCTRL2 |= Hw3; 
		break;
#ifdef _TCC79x_
	case 3:
		HwCHCTRL3 |= Hw3; 
		break;
	case 4:
		HwCHCTRL4 |= Hw3;
		break;
	case 5:
		HwCHCTRL5 |= Hw3;
		break;
	case 6:
		HwCHCTRL6 |= Hw3;
		break;
	case 7:
		HwCHCTRL7 |= Hw3;
		break;
	case 8:
		HwCHCTRL8 |= Hw3;
		break;
#endif
	}
#endif
	return 0;
}

/*****************************************************************************
* Function Name : tca_i2s_setregister()
******************************************************************************
* Desription    : set dai register
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_i2s_setregister(unsigned int nRegval)
{
	HwDAMR = nRegval;
	return 0;
}

/*****************************************************************************
* Function Name : tca_i2s_getregister()
******************************************************************************
* Desription    : get dai register
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_i2s_getregister(void)
{
	return HwDAMR;
}

/*****************************************************************************
* Function Name : tca_i2s_init()
******************************************************************************
* Desription    : i2s init
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_i2s_init(unsigned int nOutputDma, unsigned int InputDma)
{

#if defined(_TCC79x_)
	BITSET(HwBCLKCTR, Hw14); 
#elif defined(_TCC83x_)
	BITSET(HwBCLKCTR, Hw0); 
	HwGSEL_E |= Hw0; //DAO, DAI, MCLK, LRCK, BCLK for DAI
#endif


	// Clock Setting
	tea_i2s_setclock(44100);

	HwDAVC = 0;
	HwDAMR = 0
#if defined(_TCC250_INCLUDE_)
	#if defined(_TCC83x_)
		| Hw31 //Slave Mode
		| Hw30 //Slave Mode
	#endif
		#ifdef MCLK_TO_TCC250_INCLUDE
		| HwDAMR_SM_INT
		#endif
		
		#ifdef FRAME_TO_TCC250_INCLUDE
		| HwDAMR_BM_INT
		| HwDAMR_FM_INT
		#endif
		| HwDAMR_FD_DIV64
		| HwDAMR_TXS_LSB16
		| HwDAMR_RXS_LSB16
		;
#else
	#if defined(_TCC83x_)
		| Hw31 //Slave Mode
		| Hw30 //Slave Mode
	#endif
		| HwDAMR_SM_INT
		| HwDAMR_BM_INT
		| HwDAMR_FM_INT
		| HwDAMR_FD_DIV64
		| HwDAMR_TXS_LSB16
		| HwDAMR_RXS_LSB16
		;
#endif

#if defined(_TCC79x_)
    //CKC DAI clock enable
    BITSET(HwPCK_DAI, HwPCK_EN_EN);

    //DAI enable
    HwDAMR |= HwDAMR_EN_ON;

    //enable DMA Interrupt
    //HwMODE &= ~HwINT_DMA; // Edge trigger
    HwMODE |= HwINT_DMA;   //Level trigger
    HwSEL |= HwINT_DMA; 
    HwIRQMSK |= HwINT_DMA; 
#elif defined(_TCC83x_)
    //CKC DAI clock enable
   // BITSET(HwPCLKCFG6, Hw31);

    //DAI enable
    HwDAMR |= HwDAMR_EN_ON;

#ifdef _USING_ADMA_
#if defined(_TCC80x_)
    HwTMODE_A |= HwIEN_ADMA;   //Level trigger
    HwIRQSEL_A  |= HwIEN_ADMA; 
    HwIRQ_A  |= HwIEN_ADMA;
#else
    HwMODE |= HwIEN_ADMA;   //Level trigger
    HwSEL  |= HwIEN_ADMA; 
    HwIRQ  |= HwIEN_ADMA;
#endif
#else
    //enable DMA Interrupt
    //HwMODE &= ~HwIEN_DMA; // Edge trigger
#if defined(_TCC80x_)
    HwTMODE_A |= HwIEN_GDMA0;   //Level trigger
    HwIRQSEL_A |= HwIEN_GDMA0; 
    HwIRQ_A |= HwIEN_GDMA0;
#else
    HwMODE |= HwIEN_DMA;   //Level trigger
    HwSEL  |= HwIEN_DMA; 
    HwIRQ  |= HwIEN_DMA; 
#endif
#endif
#endif
       
    //----- 4. For power management purposes, shut the clocks off! -----
    tca_i2s_stop(1);
	tca_i2s_stop(0);

    tca_i2s_initoutput(nOutputDma);
    tca_i2s_initintput(InputDma);

    return(0);
}

/*****************************************************************************
* Function Name : tca_dma_setconfig()
******************************************************************************
* Desription    : dma set config
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
static void tca_dma_setconfig(
    unsigned uCH,
	unsigned uSPARAM,
	void *pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize
)
{
    switch( uCH )
    {
        case 0:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
        		HwSPARAM0 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR0	= (int)pDST;
        		HwDPARAM0 = uDPARAM;
        	}

        	// Set & Enable DMA
        	HwCHCTRL0		= uCHCTRL;
        	break;

        case 1:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
        		HwSPARAM1 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR1	= (int)pDST;
        		HwDPARAM1 = uDPARAM;
        	}

        	// Set & Enable DMA
        	HwCHCTRL1		= uCHCTRL;
        	break;
		 case 2:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
        		HwSPARAM2 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR2	= (int)pDST;
        		HwDPARAM2 = uDPARAM;
        	}

        	// Set & Enable DMA
        	HwCHCTRL2		= uCHCTRL;
        	break;
#ifdef _TCC79x_
         case 3:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
        		HwSPARAM3 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR3	= (int)pDST;
        		HwDPARAM3 = uDPARAM;
        	}

        	// Set & Enable DMA
        	HwCHCTRL3		= uCHCTRL;
        	break;
		 case 4:
        	if (uSize)
        	{
        		// Set Source Address & Source Parameter (mask + increment)
        		HwSPARAM4 = uSPARAM;

        		// Set Dest Address & Dest Parameter (mask + increment)
        		HwST_DADR4	= (int)pDST;
        		HwDPARAM4 = uDPARAM;
        	}

        	// Set & Enable DMA
        	HwCHCTRL4		= uCHCTRL;
        	break;
#endif

      }

}

/*****************************************************************************
* Function Name : tca_i2s_initoutput()
******************************************************************************
* Desription    : i2s init output
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
void tca_i2s_initoutput(unsigned int nOutputDma )
{
#if defined(_TCC79x_)

  /* unsigned int dwDMAPhysicalAddr = 0; */
    unsigned int DMABuffer = 0;

    //HwINTMSK &= ~HwINT_DMA; //~(1<<IRQ_DMA);	/* disable DMA IRQ */

    //HwCLR |= HwINT_DMA | HwINT_DAITX; 

	DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
	DMABuffer = DMABuffer * (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
    
    tca_dma_setconfig(nOutputDma, (2|DMABuffer), (void*)&HwDADO_L0_PHYSICAL, (4|0xFFFFFE00),
				HwCHCTRL_CONT_C		|
				HwCHCTRL_SYNC_EN	|
				HwCHCTRL_TYPE_SE	|
				HwCHCTRL_BSIZE_8	|
				HwCHCTRL_WSIZE_16	|
				HwCHCTRL_FLAG		|
				HwCHCTRL_IEN_EN     |
				HwCHCTRL_REP_EN,
				AUDIO_DMA_PAGE_SIZE*2);
	switch(nOutputDma)
	{
	case 0:
		HwEXTREQ0 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT0 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL0 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL0 |= HwCHCTRL_EN_EN;
		break;
	case 1:
		HwEXTREQ1 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT1 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL1 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL1 |= HwCHCTRL_EN_EN;
		break;
	case 2:
		HwEXTREQ2 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT2 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL2 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL2 |= HwCHCTRL_EN_EN;
		break;
	case 3:
		HwEXTREQ3 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT3 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL3 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL3 |= HwCHCTRL_EN_EN;
		break;	
	case 4:
		HwEXTREQ4 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT4 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL4 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL4 |= HwCHCTRL_EN_EN;
		break;	 
	case 5:
		HwEXTREQ5 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT5 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL5 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL5 |= HwCHCTRL_EN_EN;
		break;
	case 6:
		HwEXTREQ6 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT6 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL6 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL6 |= HwCHCTRL_EN_EN;
		break;
	case 7:
		HwEXTREQ7 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT7 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL7 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL7 |= HwCHCTRL_EN_EN;
		break;
	case 8:
		HwEXTREQ8 = HwEXTREQ_DAIT;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT8 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL8 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL8 |= HwCHCTRL_EN_EN;
		break;
	}

	HwINTMSK |= HwINT_DMA;
#elif defined(_TCC83x_) 
#ifndef _USING_ADMA_
    /* unsigned int dwDMAPhysicalAddr = 0; */
    unsigned int DMABuffer = 0;

    BITCLR(HwIEN , HwIEN_DMA);  //	 disable DMA IRQ 
    BITSET(HwCLR , HwIEN_DMA) ; // Intr Clear

	DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
	DMABuffer = DMABuffer * (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
    
    tca_dma_setconfig(nOutputDma, (2|DMABuffer), (void*)&HwDADO_L0_PHYSICAL, (4|0xFFFFFE00),
		HwCHCTRL_WSIZE_16	| 
		HwCHCTRL_BSIZE_8	|
		HwCHCTRL_TYPE_SE	| 
		HwCHCTRL_CONT_C		| 
		HwCHCTRL_REP_EN		|
		HwCHCTRL_SYNC_EN	|
		HwCHCTRL_FLAG,
		AUDIO_DMA_PAGE_SIZE*2);
	switch(nOutputDma)
	{
	case 0:
		HwEXTREQ0 = Hw5;//I2S_TX
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT0 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL0 &= ~Hw31; // DMA Interrupt is Occurred when the end of each Repeated DMA Operation
		HwCHCTRL0 |= HwCHCTRL_IEN_EN|HwCHCTRL_EN_EN;
		break;
	case 1:
		HwEXTREQ1 = Hw5;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT1 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL1 &= ~Hw31;
		HwCHCTRL1 |= HwCHCTRL_IEN_EN|HwCHCTRL_EN_EN;
		break;
	case 2:
		HwEXTREQ2 = Hw5;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT2 = ((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL2 &= ~Hw31;
		HwCHCTRL2 |= HwCHCTRL_IEN_EN|HwCHCTRL_EN_EN;
		break;

	}

	BITSET(HwIEN , HwIEN_DMA); 
#else
	unsigned int bitSet=0x0;

	unsigned uSize;
	//char *pSRC;	
	/* WORD dwDMAPhysicalAddr = 0; */
    unsigned int DMABuffer = 0;

    HwChCtrl &= ~HwChCtrl_DTEN_EN;

	//ADMA Clock Enable
	BITSET(HwBCLKCTR1, Hw1);

    BITCLR(HwIEN , HwIEN_ADMA);  //	 disable DMA IRQ 
    BITSET(HwCLR , HwIEN_ADMA) ; // Intr Clear

	HwSWRESET1 |= Hw1;
	HwSWRESET1 &= ~Hw1;

	DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
	DMABuffer = DMABuffer * (((AUDIO_DMA_PAGE_SIZE*2) >> 4)<<8);
    
    BITSET(HwSEL, HwIEN_ADMA);
	BITSET(HwIEN, HwIEN_ADMA);		/* and enable ADMA IRQ at INT-Controller */

	uSize = AUDIO_DMA_PAGE_SIZE*2;
//	pSRC = DMABuffer;
		
	// DAI Tx (Right) Source Address
//	HwRxCdDar = 
//	HwRxCdParam =DMABuffer;
//	HwTxDaSar 	= HwDADO_L0_PHYSICAL;
	HwTxDaParam	= DMABuffer | 2; //(4|0xFFFFFE00);	

	// ADMA Count Reginster setting
	HwTxDaTCnt = (((AUDIO_DMA_PAGE_SIZE)>>0x01)>>0x03)-1;
		
	// Tx Transfer Control Refister Setting	
	bitSet = ( HwZERO
		|HwTransCtrl_DTTRG_SL
		|HwTransCtrl_TCN_EN
		|HwTransCtrl_DTRPT_EN
		|HwTransCtrl_DTBSIZE_8
		|HwTransCtrl_DTWSIZE_16
		|HwTransCtrl_DTLCK
		);
	
	HwTransCtrl =	bitSet;

    HwRptCtrl = 0;
    //BITCLR(HwRptCtrl, HwRptCtrl_DRI_EN);
    //BITSET(HwRptCtrl, (AUDIO_DMA_PAGE_SIZE*4 / AUDIO_DMA_PAGE_SIZE) - 1);
	/* ADMA Repeat register setting */
//	BITCLR(HwRptCtrl,HwRptCtrl_DRI_EN);	//DMA Interrupt is occurred when the end of each Repeated DMAoperation.
	
	// Tx channel Control Register Setting	
	//Start DMA
	bitSet = ( HwZERO
	//	|HwChCtrl_DTEN_EN
		//|HwChCtrl_DTDW_16
		|HwChCtrl_DTIEN	
		);

  
	HwChCtrl =	bitSet;

#endif
#endif //TCC79 or TCC83
}

/*****************************************************************************
* Function Name : tca_i2s_deinit()
******************************************************************************
* Desription    : i2s deinit
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_i2s_deinit()
{
    //----- 1. Stop the I2S clocks -----
    tca_i2s_stop(0);
	tca_i2s_stop(1);

	return SUCCESS;
}


/*****************************************************************************
* Function Name : tca_i2s_start()
******************************************************************************
* Desription    :
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
void tca_i2s_start(unsigned int nMode)
{
	if(nMode) //input
	{
		//RX enable
		HwDAMR |= HwDAMR_RE_EN;
	}
	else
	{
		//TX enable
		HwDAMR |=HwDAMR_TE_EN;
	}


}


/*****************************************************************************
* Function Name : tca_i2s_stop()
******************************************************************************
* Desription    :
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
void tca_i2s_stop(unsigned int nMode)
{
	if(nMode)
	{
		//RX disable
		HwDAMR &= ~HwDAMR_RE_EN;
	}
	else
	{
		//TX enable
		HwDAMR &= ~HwDAMR_TE_EN;
	}

}

/*****************************************************************************
* Function Name : tca_i2s_initintput()
******************************************************************************
* Desription    :
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
void tca_i2s_initintput(unsigned int nInputDma)
{
#if defined(_TCC79x_)
   /* unsigned int dwDMAPhysicalAddr = 0; */
   unsigned int DMABuffer;
	
    //HwINTMSK &= ~HwINT_DMA; //~(1<<IRQ_DMA);	/* disable DMA IRQ */

    //HwCLR |= HwINT_DMA | HwINT_DAIRX; //(1<<IRQ_DMA) | (1<<IRQ_DAIRX);
    DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_IN_PAGE_SIZE*2) >> 4)<<8);
    DMABuffer = DMABuffer * (((AUDIO_DMA_IN_PAGE_SIZE*2) >> 4)<<8);
    tca_dma_setconfig(nInputDma, (4|0xFFFFFE00), (void*)0,  (2|DMABuffer),
				HwCHCTRL_CONT_C		|
				HwCHCTRL_SYNC_EN	|
				HwCHCTRL_TYPE_SE	|
				HwCHCTRL_BSIZE_8	|
				HwCHCTRL_WSIZE_16	|
				HwCHCTRL_FLAG		|
				HwCHCTRL_IEN_EN     |
				HwCHCTRL_REP_EN,
				AUDIO_DMA_IN_PAGE_SIZE*2);
	switch(nInputDma)
	{
	case 0:
		HwEXTREQ0 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT0 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL0 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL0 |= HwCHCTRL_EN_EN;
		break;
	case 1:
		HwEXTREQ1 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT1 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL1 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL1 |= HwCHCTRL_EN_EN;
		break;
	case 2:
		HwEXTREQ2 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT2 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL2 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL2 |= HwCHCTRL_EN_EN;
		break;
	case 3:
		HwEXTREQ3 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT3 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL3 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL3 |= HwCHCTRL_EN_EN;
		break;	
	case 4:
		HwEXTREQ4 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT4 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL4 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL4 |= HwCHCTRL_EN_EN;
		break;	 
	case 5:
		HwEXTREQ5 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT5 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL5 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL5 |= HwCHCTRL_EN_EN;
		break;
	case 6:
		HwEXTREQ6 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT6 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL6 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL6 |= HwCHCTRL_EN_EN;
		break;
	case 7:
		HwEXTREQ7 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT7 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL7 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL7 |= HwCHCTRL_EN_EN;
		break;
	case 8:
		HwEXTREQ8 = HwEXTREQ_DAIR;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT8 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL8 &= ~HwRPTCTRL_DRI_LAST;
		HwCHCTRL8 |= HwCHCTRL_EN_EN;
		break;
	}
#elif defined(_TCC83x_)
    /* unsigned int dwDMAPhysicalAddr = 0; */
	unsigned int DMABuffer;

    HwIEN &= ~HwIEN_DMA; //~(1<<IRQ_DMA);	/* disable DMA IRQ */

    HwCLR |= HwIEN_DMA ; //(1<<IRQ_DMA) | (1<<IRQ_DAIRX);
    DMABuffer = 0xFFFFFF00 / (((AUDIO_DMA_IN_PAGE_SIZE*2) >> 4)<<8);
    DMABuffer = DMABuffer * (((AUDIO_DMA_IN_PAGE_SIZE*2) >> 4)<<8);
    //tca_dma_setconfig(1, (4|0xFFFFFE00), (void*)0,  (2|0xFFFF8000),
	 tca_dma_setconfig(nInputDma, (4|0xFFFFFE00), (void*)0,  (2|DMABuffer),
				HwCHCTRL_CONT_C		|
				HwCHCTRL_SYNC_EN	|
				HwCHCTRL_TYPE_SE	|
				HwCHCTRL_BSIZE_8	|
				HwCHCTRL_WSIZE_16	|
				HwCHCTRL_FLAG		|
				HwCHCTRL_IEN_EN     |
				HwCHCTRL_REP_EN,
				AUDIO_DMA_IN_PAGE_SIZE*2);
	switch(nInputDma)
	{
	case 0:
		HwEXTREQ0 = Hw4;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT0 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL0 &= ~Hw31;
		HwCHCTRL0 |= HwCHCTRL_EN_EN;
		break;
	case 1:
		HwEXTREQ1 = Hw4;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT1 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL1 &= ~Hw31;
		HwCHCTRL1 |= HwCHCTRL_EN_EN;
		break;
	case 2:
		HwEXTREQ2 = Hw4;
		// now one HOP is 8*2 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
		HwHCOUNT2 = ((AUDIO_DMA_IN_PAGE_SIZE)>>0x01)>>0x03;
		HwRPTCTRL2 &= ~Hw31;
		HwCHCTRL2 |= HwCHCTRL_EN_EN;
		break;
	}
#endif
}


#ifdef __KERNEL__
EXPORT_SYMBOL(tca_i2s_init);
EXPORT_SYMBOL(tca_i2s_deinit);
EXPORT_SYMBOL(tca_i2s_setregister);
EXPORT_SYMBOL(tca_i2s_getregister);

EXPORT_SYMBOL(tca_i2s_start);
EXPORT_SYMBOL(tca_i2s_stop);
EXPORT_SYMBOL(tca_i2s_initoutput);
EXPORT_SYMBOL(tca_i2s_initintput);
EXPORT_SYMBOL(tca_i2s_mutecontrol);

EXPORT_SYMBOL(tca_irq_getnumber);

EXPORT_SYMBOL(tca_dma_clrstatus);
EXPORT_SYMBOL(tca_dma_getstatus);
EXPORT_SYMBOL(tca_dma_control);
EXPORT_SYMBOL(tca_dma_setsrcaddr);
EXPORT_SYMBOL(tca_dma_setdestaddr);
EXPORT_SYMBOL(tca_dma_clrinterrupt);

EXPORT_SYMBOL(tca_tcc_initport);
#endif
