
#ifndef __SATA_SNPS_H__
#define __SATA_SNPS_H__

/*
 * HSATA Registers
 */
#define TCC_SATA_BASE_ADDR 0xF0560000
#define HSATA_MEM_BASE    TCC_SATA_BASE_ADDR


#define HSATA_SCR0_REG       0x0024
#define HSATA_SCR0_SPD_GET(v) (((v) & 0x000000f0) >> 4)
#define HSATA_SCR1_REG       0x0028
#define HSATA_SCR2_REG       0x002C
#define HSATA_SCR3_REG       0x0030
#define HSATA_SCR4_REG       0x0034

#define HSATA_SSTATUS_REG        HSATA_SCR0_REG
#define HSATA_SERROR_REG         HSATA_SCR1_REG
#define HSATA_SERROR_ERR_BITS    0x0000ffff
#define HSATA_SCONTROL_REG       HSATA_SCR2_REG
#define HSATA_SACTIVE_REG        HSATA_SCR3_REG
#define HSATA_SNOTIFICATION_REG  HSATA_SCR4_REG

#define HSATA_FPTAGR_REG         0x0064
#define HSATA_FPBOR_REG          0x0068
#define HSATA_FPTCR_REG          0x006C
#define HSATA_DMACR_REG          0x0070

#if 1
#define HSATA_DMACR_TXMODE_BIT  0x04
#define HSATA_DMACR_TX_EN    0x01 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_RX_EN    0x02 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_TXRX_EN  0x03 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_TXRX_CLEAR HSATA_DMACR_TXMODE_BIT
#else
#define HSATA_DMACR_TXMODE_BIT  0x00
#define HSATA_DMACR_TX_EN    0x01 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_RX_EN    0x02 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_TXRX_EN  0x03 | HSATA_DMACR_TXMODE_BIT
#define HSATA_DMACR_TXRX_CLEAR HSATA_DMACR_TXMODE_BIT
#endif



#define HSATA_DBTSR_REG          0x0074
#define HSATA_INTPR_REG          0x0078
//#define HSATA_INTPR_ERR_BIT      0x00000008
//#define HSATA_INTPR_FP_BIT       0x00000002  /* new DMA setup FIS arrived */
#define HSATA_INTMR_REG          0x007C



#define HSATA_INTMR_DMAT_BIT     Hw0
#define HSATA_INTMR_NEWFP_BIT    Hw1
#define HSATA_INTMR_PMABORT      Hw2
#define HSATA_INTMR_ERRM_BIT     Hw3
#define HSATA_INTMR_NEWBIST_BIT  Hw4
#define HSATA_INTMR_PRIMERR_BIT  Hw5
#define HSATA_INTMR_CMDABORT_BIT Hw6
#define HSATA_INTMR_CMDGOOD_BIT   Hw7

#define TCC_INTR_CHECK_BIT (HSATA_INTMR_DMAT_BIT \
                            | HSATA_INTMR_NEWFP_BIT \
                            | HSATA_INTMR_PMABORT \
                            | HSATA_INTMR_ERRM_BIT \
                            | HSATA_INTMR_NEWBIST_BIT \
                            | HSATA_INTMR_PRIMERR_BIT \
                            | HSATA_INTMR_CMDABORT_BIT \
                            | HSATA_INTMR_CMDGOOD_BIT)



#define HSATA_ERRMR_REG          0x0080
#define HSATA_LLCR_REG           0x0084
#define HSATA_PHYCR_REG          0x0088
#define HSATA_PHYSR_REG          0x008C
#define HSATA_RXBISTPD_REG       0x0090
#define HSATA_RXBISTD1_REG       0x0094
#define HSATA_RXBISTD2_REG       0x0098
#define HSATA_TXBISTPD_REG       0x009C
#define HSATA_TXBISTD1_REG       0x00A0
#define HSATA_TXBISTD2_REG       0x00A4
#define HSATA_BISTCR_REG         0x00A8
#define HSATA_BISTFCTR_REG       0x00AC
#define HSATA_BISTSR_REG         0x00B0
#define HSATA_BISTDECR_REG       0x00B4

#define HSATA_TESTR_REG          0x00F4
#define HSATA_VERSONR_REG        0x00F8
#define HSATA_IDR_REG            0x00FC

#define HSATA_CMD_QWRITE  0x61  /* these don't seem to be defined in ata.h/libata.h */
#define HSATA_CMD_QREAD   0x60


#define TCC_DMADR_REG          0x0400
#define TCC_DMADR_REG_ADDR     (HSATA_MEM_BASE + TCC_DMADR_REG)


#define HSATA_ENABLE_INTERRUPTS(tcc_dev) \
   { \
      volatile u32 val32; \
      /* enable all err interrupts */ \
      /* COMRESET clears reg INTMR so .. see where this func is called */ \
      writel( 0xffffffff, tcc_dev->membase + HSATA_ERRMR_REG ); \
      val32 = readl( tcc_dev->membase + HSATA_INTMR_REG ); \
      writel( val32 | TCC_INTR_CHECK_BIT, \
              tcc_dev->membase + HSATA_INTMR_REG ); \
      val32 = readl( tcc_dev->membase + HSATA_INTMR_REG ); \
      jprintk("INTMR=0x%x", val32);\
   }


#define TCC_QCMD_MAX    128
#define TCC_SG_LIST_MAX (TCC_QCMD_MAX * 8)

#define _TCC_DMA_TX_  0x01
#define _TCC_DMA_RX_  0x02
#define _TCC_DMA_END_ 0x04
#define _TCC_DMA_START_ 0x08


#define CDR7_ERR 0

struct tcc_sg_list {
    unsigned int dma_addr;
    unsigned int length;
    int flags;
};

struct tcc_dma_Q {
    struct tcc_sg_list sg_list[TCC_SG_LIST_MAX];
    int top, bottom, pre;
};

struct tcc_sata_dev {
    void __iomem *membase;
    GDMANCTRL *dma_tx_reg, *dma_rx_reg;
    int dma_tx_irq, dma_rx_irq;

    struct tcc_dma_Q dma_q;
    int is_rx_complete, is_tx_complete;
    int is_last_rx, is_last_tx;
    wait_queue_head_t job_wait_q;
    wait_queue_head_t tx_dma_wait_q;
    wait_queue_head_t rx_dma_wait_q;
    int is_continue;
    struct task_struct *fetch_task;
};




#endif /*__SATA_SNPS_H__*/

